@@ -38,6 +38,7 @@
 main(int    argc,
      char **argv)
 {
+l_int32      i;
 char        *mazeout, *pathout;
 PIX         *pixm, *pixex, *pixd;
 PTA         *pta;
@@ -55,13 +56,22 @@ static char  mainName[] = "binmazetest";
     pixDestroy(&pixex);
     pixWrite(mazeout, pixm, IFF_PNG);
 
-    pta = searchBinaryMaze(pixm, XINIT, YINIT, XEND, YEND, NULL);
+    pta = pixSearchBinaryMaze(pixm, XINIT, YINIT, XEND, YEND, NULL);
     pixd = pixDisplayPta(pixm, pta);
     pixex = pixScaleBySampling(pixd, 4., 4.);
     pixDisplay(pixex, 450, 50);
     pixDestroy(&pixex);
     pixWrite(pathout, pixd, IFF_PNG);
 
+#if 0
+    startTimer();
+    for (i = 0; i < 100; i++) {
+        pta = pixSearchBinaryMaze(pixm, XINIT, YINIT, XEND, YEND, NULL);
+        ptaDestroy(&pta);
+    }
+    fprintf(stderr, "Time: %7.4f sec\n", stopTimer() / 100.);
+#endif
+
     pixDestroy(&pixm);
     pixDestroy(&pixd);
     ptaDestroy(&pta);
