@@ -22,15 +22,17 @@
 #include <stdlib.h>
 #include "allheaders.h"
 
-static void pixCompare(PIX *pix, PIX *pix2, char *msg1, char *msg2);
-static const l_int32  BUFSIZE = 256;
+static void pixCompare(PIX *pix, PIX *pix2, const char *msg1, const char *msg2);
+
+    /* MSVC can't handle arrays dimensioned by static const integers */
+#define L_BUF_SIZE    256
 
 
 main(int    argc,
      char **argv)
 {
 char        *filein;
-char         buf[BUFSIZE];
+char         buf[L_BUF_SIZE];
 l_int32      size;
 PIX         *pixs, *pixt1, *pixt2;
 static char  mainName[] = "colormorphtest";
@@ -41,6 +43,7 @@ static char  mainName[] = "colormorphtest";
 
     filein = argv[1];
     size = atoi(argv[2]);
+    if (size % 2 == 0) size++;
 
     if ((pixs = pixRead(filein)) == NULL)
         exit(ERROR_INT("pixs not read", mainName, 1));
@@ -73,17 +76,17 @@ static char  mainName[] = "colormorphtest";
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
 
-    system("/usr/bin/gthumb junk_write_display* &");
+    system("gthumb junk_write_display* &");
 
     pixDestroy(&pixs);
     return 0;
 }
 
     /* Simple comparison function */
-static void pixCompare(PIX   *pix1,
-                       PIX   *pix2,
-                       char  *msg1,
-                       char  *msg2)
+static void pixCompare(PIX         *pix1,
+                       PIX         *pix2,
+                       const char  *msg1,
+                       const char  *msg2)
 {
 l_int32  same;
     pixEqual(pix1, pix2, &same);
