@@ -41,10 +41,17 @@
 #include <stdlib.h>
 #include "allheaders.h"
 
+    /* Needed for HAVE_FMEMOPEN */
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif /* HAVE_CONFIG_H */
+
 #define   BMP_FILE      "test1.bmp"
 #define   FILE_1BPP     "feyn.tif"
-#define   FILE_2BPP     "weasel2.4g.png"
-#define   FILE_4BPP     "weasel4.16c.png"
+#define   FILE_2BPP     "speckle2.png"
+#define   FILE_2BPP_C   "weasel2.4g.png"
+#define   FILE_4BPP     "speckle4.png"
+#define   FILE_4BPP_C   "weasel4.16c.png"
 #define   FILE_8BPP_1   "dreyfus8.png"
 #define   FILE_8BPP_2   "weasel8.240c.png"
 #define   FILE_8BPP_3   "test8.jpg"
@@ -55,6 +62,7 @@ static l_int32 testcomp(const char *filename, PIX *pix, l_int32 comptype);
 static l_int32 testcomp_mem(PIX *pixs, PIX **ppixt, l_int32 index,
                             l_int32 format);
 static l_int32 test_writemem(PIX *pixs, l_int32 format, char *psfile);
+static PIX *make_24_bpp_pix(PIX *pixs);
 
 
 main(int    argc,
@@ -64,8 +72,10 @@ char         psname[256];
 l_uint8     *data;
 l_int32      i, d, n, success, nbytes, same;
 l_int32      w, h, bps, spp;
+l_float32    diff;
 size_t       size;
-PIX         *pix1, *pix2, *pix4, *pix8, *pix16, *pix32, *pix, *pixt;
+PIX         *pix1, *pix2, *pix4, *pix8, *pix16, *pix32;
+PIX         *pix, *pixt, *pixd;
 PIXA        *pixa;
 static char  mainName[] = "ioformats_reg";
 
@@ -81,8 +91,12 @@ static char  mainName[] = "ioformats_reg";
     if (ioFormatTest(FILE_1BPP)) success = FALSE;
     fprintf(stderr, "\nTest 2 bpp file:\n");
     if (ioFormatTest(FILE_2BPP)) success = FALSE;
+    fprintf(stderr, "\nTest 2 bpp file with cmap:\n");
+    if (ioFormatTest(FILE_2BPP_C)) success = FALSE;
     fprintf(stderr, "\nTest 4 bpp file:\n");
     if (ioFormatTest(FILE_4BPP)) success = FALSE;
+    fprintf(stderr, "\nTest 4 bpp file with cmap:\n");
+    if (ioFormatTest(FILE_4BPP_C)) success = FALSE;
     fprintf(stderr, "\nTest 8 bpp grayscale file with cmap:\n");
     if (ioFormatTest(FILE_8BPP_1)) success = FALSE;
     fprintf(stderr, "\nTest 8 bpp color file with cmap:\n");
@@ -168,7 +182,7 @@ static char  mainName[] = "ioformats_reg";
             fprintf(stderr, "nbytes = %d, size = %d\n", nbytes, size);
             pixt = pixReadMemTiff(data, size, 0);
             if (testcomp_mem(pix, &pixt, i, IFF_TIFF_G4)) success = FALSE;
-            readHeaderMemTiff(data, size, &w, &h, &bps, &spp, NULL, NULL);
+            readHeaderMemTiff(data, size, 0, &w, &h, &bps, &spp, NULL, NULL);
             fprintf(stderr, "(w,h,bps,spp) = (%d,%d,%d,%d)\n", w, h, bps, spp);
 	    FREE(data);
             pixWriteMemTiff(&data, &size, pix, IFF_TIFF_RLE);
@@ -191,7 +205,7 @@ static char  mainName[] = "ioformats_reg";
         pixWriteMemTiff(&data, &size, pix, IFF_TIFF_ZIP);
         pixt = pixReadMemTiff(data, size, 0);
         if (testcomp_mem(pix, &pixt, i, IFF_TIFF_ZIP)) success = FALSE;
-        readHeaderMemTiff(data, size, &w, &h, &bps, &spp, NULL, NULL);
+        readHeaderMemTiff(data, size, 0, &w, &h, &bps, &spp, NULL, NULL);
         fprintf(stderr, "(w,h,bps,spp) = (%d,%d,%d,%d)\n", w, h, bps, spp);
         FREE(data);
         pixWriteMemTiff(&data, &size, pix, IFF_TIFF);
@@ -210,7 +224,7 @@ static char  mainName[] = "ioformats_reg";
 
     /* ---------------- Part 4: Test non-tiff r/w to memory ---------------- */
 
-#ifndef _STANDARD_C_
+#if HAVE_FMEMOPEN
     pixDisplayWrite(NULL, -1);
     success = TRUE;
     for (i = 0; i < n; i++) {
@@ -234,9 +248,10 @@ static char  mainName[] = "ioformats_reg";
     else
         fprintf(stderr,
             "\n  ***** Failure on at least one non-tiff r/w to memory *****\n");
-#endif  /*  ~_STANDARD_C_  */
-
-    system("gthumb junk_write_display* &");
+#else
+        fprintf(stderr,
+            "\n  ***** Non-tiff r/w to memory not enabled *****\n");
+#endif  /*  HAVE_FMEMOPEN  */
 
     pixaDestroy(&pixa);
 
@@ -284,6 +299,37 @@ static char  mainName[] = "ioformats_reg";
     pixDestroy(&pixt);
     pixDestroy(&pix1);
     pixDestroy(&pix2);
+
+    /* ------------ Part 6: Test 24 bpp writing ------------ */
+
+        /* Generate a 24 bpp (not 32 bpp !!) rgb pix and write it out */
+    success = TRUE;
+    pix = pixRead("marge.jpg");
+    pixt = make_24_bpp_pix(pix);
+    pixWrite("junk24.png", pixt, IFF_PNG);
+    pixWrite("junk24.jpg", pixt, IFF_JFIF_JPEG);
+    pixWrite("junk24.tif", pixt, IFF_TIFF);
+    pixd = pixRead("junk24.png");
+    pixEqual(pix, pixd, &same);
+    if (!same) success = FALSE;
+    pixDestroy(&pixd);
+    pixd = pixRead("junk24.jpg");
+    pixCompareRGB(pix, pixd, L_COMPARE_ABS_DIFF, 0, NULL, &diff, NULL, NULL);
+    if (diff > 0.1) success = FALSE;
+    pixDestroy(&pixd);
+    pixd = pixRead("junk24.tif");
+    pixEqual(pix, pixd, &same);
+    if (!same) success = FALSE;
+    pixDestroy(&pixd);
+    if (success)
+        fprintf(stderr,
+            "\n  ******* Success on 24 bpp rgb writing *******\n");
+    else
+        fprintf(stderr,
+            "\n  ******* Failure on 24 bpp rgb writing *******\n");
+    pixDestroy(&pix);
+    pixDestroy(&pixt);
+
     exit(0);
 }
 
@@ -381,4 +427,32 @@ PIX      *pixd = NULL;
 }
 
 
+    /* Composes 24 bpp rgb pix */
+static PIX *
+make_24_bpp_pix(PIX  *pixs)
+{
+l_int32    i, j, w, h, wpls, wpld, rval, gval, bval;
+l_uint32  *lines, *lined, *datas, *datad;
+PIX       *pixd;
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    pixd = pixCreate(w, h, 24);
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+    for (i = 0; i < h; i++) {
+        lines = datas + i * wpls;
+        lined = datad + i * wpld;
+        for (j = 0; j < w; j++) {
+            extractRGBValues(lines[j], &rval, &gval, &bval);
+            *((l_uint8 *)lined + 3 * j) = rval;
+            *((l_uint8 *)lined + 3 * j + 1) = gval;
+            *((l_uint8 *)lined + 3 * j + 2) = bval;
+        }
+    }
+
+    return pixd;
+}
+
 
