@@ -33,7 +33,7 @@ static const l_int32    MODSIZE = 7;  /* set to 7 for display */
 static const l_float32  ANGLE1 = 3.14159265 / 12;
 static const l_int32    NTIMES = 24;
 
-void rotateTest(char *fname);
+void rotateTest(const char *fname);
 
 
 main(int    argc,
@@ -59,7 +59,7 @@ static char  mainName[] = "rotatetest2";
 
 
 void
-rotateTest(char *fname)
+rotateTest(const char *fname)
 {
 l_int32   w, h, d, i;
 PIX      *pixs, *pixt, *pixd1, *pixd2;
@@ -93,17 +93,17 @@ PIXCMAP  *cmap;
     }
 
     if (d == 1) {
-        pixWrite("/usr/tmp/junkbin1", pixd1, IFF_PNG);
-        pixWrite("/usr/tmp/junkbin2", pixd2, IFF_PNG);
+        pixWrite("/tmp/junkbin1", pixd1, IFF_PNG);
+        pixWrite("/tmp/junkbin2", pixd2, IFF_PNG);
     } else if (cmap) {
-        pixWrite("/usr/tmp/junkcmap1", pixd1, IFF_PNG);
-        pixWrite("/usr/tmp/junkcmap2", pixd2, IFF_PNG);
+        pixWrite("/tmp/junkcmap1", pixd1, IFF_PNG);
+        pixWrite("/tmp/junkcmap2", pixd2, IFF_PNG);
     } else if (d == 8) {
-        pixWrite("/usr/tmp/junkgray1", pixd1, IFF_JFIF_JPEG);
-        pixWrite("/usr/tmp/junkgray2", pixd2, IFF_JFIF_JPEG);
+        pixWrite("/tmp/junkgray1", pixd1, IFF_JFIF_JPEG);
+        pixWrite("/tmp/junkgray2", pixd2, IFF_JFIF_JPEG);
     } else if (d == 32) {
-        pixWrite("/usr/tmp/junkrgb1", pixd1, IFF_JFIF_JPEG);
-        pixWrite("/usr/tmp/junkrgb2", pixd2, IFF_JFIF_JPEG);
+        pixWrite("/tmp/junkrgb1", pixd1, IFF_JFIF_JPEG);
+        pixWrite("/tmp/junkrgb2", pixd2, IFF_JFIF_JPEG);
     }
 
     pixDestroy(&pixd1);
