@@ -25,15 +25,18 @@
 #include "bmf.h"
 #include "ccbord.h"
 #include "gplot.h"
+#include "heap.h"
 #include "jbclass.h"
 #include "list.h"
 #include "morph.h"
-#include "pheap.h"
 #include "pix.h"
-#include "pqueue.h"
+#include "ptra.h"
+#include "queue.h"
 #include "stack.h"
+#include "watershed.h"
 
     /* i/o */
+#include <stdio.h>
 #include "imageio.h"
 
 #endif /* LEPTONICA_ALLTYPES_H */
