@@ -19,9 +19,9 @@
 /*
  *  arrayaccess.h
  *
- *  1, 2, 4, 8 and 16 bit data access within an array of 32-bit words
+ *  1, 2, 4, 8, 16 and 32 bit data access within an array of 32-bit words
  *
- *  This is used primarily to access 1, 2, 4, 8 and 16 bit pixels
+ *  This is used primarily to access 1, 2, 4, 8, 16 and 32 bit pixels
  *  in a line of image data, represented as an array of 32-bit words.
  *
  *     pdata:  pointer to first 32-bit word in the array
@@ -33,6 +33,9 @@
  *  Even though the 2 and 4 bit set* accessors are more complicated,
  *  they are about 10% faster than the function calls.
  *
+ *  The 32 bit access is just a cast and ptr arithmetic.  We include
+ *  it so that the input ptr can be void*.
+ *
  *  At the end of this file is code for invoking the function calls
  *  instead of inlining.
  *
@@ -51,17 +54,17 @@
      *                     1 bit access                 *
      *--------------------------------------------------*/
 #define  GET_DATA_BIT(pdata, n) \
-    ((*((pdata) + ((n) >> 5)) >> (31 - ((n) & 31))) & 1)
+    ((*((l_uint32 *)(pdata) + ((n) >> 5)) >> (31 - ((n) & 31))) & 1)
 
 #define  SET_DATA_BIT(pdata, n) \
-    (*((pdata) + ((n) >> 5)) |= (0x80000000 >> ((n) & 31)))
+    (*((l_uint32 *)(pdata) + ((n) >> 5)) |= (0x80000000 >> ((n) & 31)))
 
 #define  CLEAR_DATA_BIT(pdata, n) \
-    (*((pdata) + ((n) >> 5)) &= ~(0x80000000 >> ((n) & 31)))
+    (*((l_uint32 *)(pdata) + ((n) >> 5)) &= ~(0x80000000 >> ((n) & 31)))
 
 #define  SET_DATA_BIT_VAL(pdata, n, val) \
     ({l_uint32 *_TEMP_WORD_PTR_; \
-     _TEMP_WORD_PTR_ = (pdata) + ((n) >> 5); \
+     _TEMP_WORD_PTR_ = (l_uint32 *)(pdata) + ((n) >> 5); \
      *_TEMP_WORD_PTR_ &= ~(0x80000000 >> ((n) & 31)); \
      *_TEMP_WORD_PTR_ |= ((val) << (31 - ((n) & 31))); \
     })
@@ -71,34 +74,34 @@
      *                     2 bit access                 *
      *--------------------------------------------------*/
 #define  GET_DATA_DIBIT(pdata, n) \
-    ((*((pdata) + ((n) >> 4)) >> (2 * (15 - ((n) & 15)))) & 3)
+    ((*((l_uint32 *)(pdata) + ((n) >> 4)) >> (2 * (15 - ((n) & 15)))) & 3)
 
 #define  SET_DATA_DIBIT(pdata, n, val) \
     ({l_uint32 *_TEMP_WORD_PTR_; \
-     _TEMP_WORD_PTR_ = (pdata) + ((n) >> 4); \
+     _TEMP_WORD_PTR_ = (l_uint32 *)(pdata) + ((n) >> 4); \
      *_TEMP_WORD_PTR_ &= ~(0xc0000000 >> (2 * ((n) & 15))); \
      *_TEMP_WORD_PTR_ |= ((val) << (30 - 2 * ((n) & 15))); \
     })
 
 #define  CLEAR_DATA_DIBIT(pdata, n) \
-    (*((pdata) + ((n) >> 4)) &= ~(0xc0000000 >> (2 * ((n) & 15))))
+    (*((l_uint32 *)(pdata) + ((n) >> 4)) &= ~(0xc0000000 >> (2 * ((n) & 15))))
 
 
     /*--------------------------------------------------*
      *                     4 bit access                 *
      *--------------------------------------------------*/
 #define  GET_DATA_QBIT(pdata, n) \
-     ((*((pdata) + ((n) >> 3)) >> (4 * (7 - ((n) & 7)))) & 0xf)
+     ((*((l_uint32 *)(pdata) + ((n) >> 3)) >> (4 * (7 - ((n) & 7)))) & 0xf)
 
 #define  SET_DATA_QBIT(pdata, n, val) \
     ({l_uint32 *_TEMP_WORD_PTR_; \
-     _TEMP_WORD_PTR_ = (pdata) + ((n) >> 3); \
+     _TEMP_WORD_PTR_ = (l_uint32 *)(pdata) + ((n) >> 3); \
      *_TEMP_WORD_PTR_ &= ~(0xf0000000 >> (4 * ((n) & 7))); \
      *_TEMP_WORD_PTR_ |= ((val) << (28 - 4 * ((n) & 7))); \
     })
 
 #define  CLEAR_DATA_QBIT(pdata, n) \
-    (*((pdata) + ((n) >> 3)) &= ~(0xf0000000 >> (4 * ((n) & 7))))
+    (*((l_uint32 *)(pdata) + ((n) >> 3)) &= ~(0xf0000000 >> (4 * ((n) & 7))))
 
 
     /*--------------------------------------------------*
@@ -140,6 +143,17 @@
              (*(l_uint16 *)((l_uintptr_t)((l_uint16 *)(pdata) + (n)) ^ 2) = (val))
 #endif  /* L_BIG_ENDIAN */
 
+
+    /*--------------------------------------------------*
+     *                    32 bit access                 *
+     *--------------------------------------------------*/
+#define  GET_DATA_FOUR_BYTES(pdata, n) \
+             (*((l_uint32 *)(pdata) + (n)))
+
+#define  SET_DATA_FOUR_BYTES(pdata, n, val) \
+             (*((l_uint32 *)(pdata) + (n)) = (val))
+
+
 #endif  /* COMPILER_MSVC */
 #endif  /* Inline Accessors */
 
@@ -167,6 +181,9 @@
 
 #define  GET_DATA_TWO_BYTES(pdata, n)         l_getDataTwoBytes(pdata, n)
 #define  SET_DATA_TWO_BYTES(pdata, n, val)    l_setDataTwoBytes(pdata, n, val)
+
+#define  GET_DATA_FOUR_BYTES(pdata, n)         l_getDataFourBytes(pdata, n)
+#define  SET_DATA_FOUR_BYTES(pdata, n, val)    l_setDataFourBytes(pdata, n, val)
 #endif
 
 
