@@ -318,7 +318,7 @@ PTA       *ptas, *ptad;
         return (PIX *)ERROR_PTR("transform pts not found", procName, NULL);
 
         /* Use a projective transform */
-    pixd = pixProjectiveSampled(pixs, ptad, ptas, L_BRING_IN_WHITE); 
+    pixd = pixProjectiveSampledPta(pixs, ptad, ptas, L_BRING_IN_WHITE); 
 
     ptaDestroy(&ptas);
     ptaDestroy(&ptad);
@@ -532,7 +532,7 @@ PTA       *pta;
         ptaDestroy(&pta);
         return (NUMA *)ERROR_PTR("can't fit skew", procName, NULL);
     }
-    ptaGetLinearLSF(pta, &a, &b);
+    ptaGetLinearLSF(pta, &a, &b, NULL);
     if (pa) *pa = a;
     if (pb) *pb = b;
 
