@@ -22,7 +22,7 @@
  *      Write bmp to file
  *           l_int32       pixWriteStreamBmp()
  *
- *      Read/write to memory   [not on windows]
+ *      Read/write to memory   [only on linux]
  *           PIX          *pixReadMemBmp()
  *           l_int32       pixWriteMemBmp()
  */
@@ -484,8 +484,7 @@ RGBA_QUAD  *pquad;
 #include "config_auto.h"
 #endif  /* HAVE_CONFIG_H */
 
-#if HAVE_FMEMOPEN || \
- (!defined(__MINGW32__) && !defined(_CYGWIN_ENVIRON) && !defined(_STANDARD_C_))
+#if HAVE_FMEMOPEN
 
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
@@ -580,7 +579,7 @@ pixWriteMemBmp(l_uint8  **pdata,
         "pixWriteMemBmp", 1);
 }
 
-#endif  /* HAVE_FMEMOPEN || (!defined(__MINGW32__) && etc ) */
+#endif  /* HAVE_FMEMOPEN */
 
 /* --------------------------------------------*/
 #endif  /* USE_BMPIO */
