@@ -24,10 +24,10 @@
  *           BOX      *boxCopy()
  *           BOX      *boxClone()
  *           void      boxDestroy()
- *           void      boxFree()
  *
  *      Box accessors
  *           l_int32   boxGetGeometry()
+ *           l_int32   boxSetGeometry()
  *           l_int32   boxGetRefcount()
  *           l_int32   boxChangeRefcount()
  *
@@ -50,8 +50,9 @@
  *           l_int32   boxaInsertBox()
  *           l_int32   boxaRemoveBox()
  *
- *      Boxaa creation, destruction
+ *      Boxaa creation, copy, destruction
  *           BOXAA    *boxaaCreate()
+ *           BOXAA    *boxaaCopy()
  *           void      boxaaDestroy()
  *
  *      Boxaa array extension
@@ -84,6 +85,9 @@
  *      Box print (for debug)
  *           l_int32   boxPrintStreamInfo()
  *
+ *      Backward compatibility old boxaa read functions
+ *           BOXAA    *boxaaReadVersion2()
+ *           BOXAA    *boxaaReadStreamVersion2()
  */
 
 #include <stdio.h>
@@ -132,12 +136,9 @@ BOX  *box;
             return (BOX *)ERROR_PTR("y < 0 and box off +quad", procName, NULL);
     }
 
-    if ((box = (BOX *) CALLOC(1, sizeof(BOX))) == NULL)
+    if ((box = (BOX *)CALLOC(1, sizeof(BOX))) == NULL)
         return (BOX *)ERROR_PTR("box not made", procName, NULL);
-    box->x = x;
-    box->y = y;
-    box->w = w;
-    box->h = h;
+    boxSetGeometry(box, x, y, w, h);
     box->refcount = 1;
 
     return box;
@@ -207,34 +208,13 @@ BOX  *box;
         L_WARNING("ptr address is null!", procName);
         return;
     }
-
     if ((box = *pbox) == NULL)
         return;
 
-    boxFree(box);
-    *pbox = NULL;
-    return;
-}
-
-
-/*!
- *  boxFree()
- *
- *      Input:  box
- *      Return: void
- *
- *  Notes:
- *      (1) Decrements the ref count and, if 0, destroys the box.
- */
-void
-boxFree(BOX  *box)
-{
-    if (!box) return;
-
-	/* Decrement the ref count.  If it is 0, destroy the box. */
     boxChangeRefcount(box, -1);
     if (boxGetRefcount(box) <= 0)
         FREE(box);
+    *pbox = NULL;
     return;
 }
 
@@ -258,6 +238,10 @@ boxGetGeometry(BOX      *box,
 {
     PROCNAME("boxGetGeometry");
 
+    if (px) *px = 0;
+    if (py) *py = 0;
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
     if (!box)
         return ERROR_INT("box not defined", procName, 1);
     if (px) *px = box->x;
@@ -268,6 +252,32 @@ boxGetGeometry(BOX      *box,
 }
 
 
+/*!
+ *  boxSetGeometry()
+ *
+ *      Input:  box
+ *              x, y, w, h (use -1 to leave unchanged)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+boxSetGeometry(BOX     *box,
+               l_int32  x,
+               l_int32  y,
+               l_int32  w,
+               l_int32  h)
+{
+    PROCNAME("boxSetGeometry");
+
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+    if (x != -1) box->x = x;
+    if (y != -1) box->y = y;
+    if (w != -1) box->w = w;
+    if (h != -1) box->h = h;
+    return 0;
+}
+
+
 l_int32
 boxGetRefcount(BOX  *box)
 {
@@ -471,8 +481,8 @@ boxaExtendArray(BOXA  *boxa)
         return ERROR_INT("boxa not defined", procName, 1);
 
     if ((boxa->box = (BOX **)reallocNew((void **)&boxa->box,
-                               sizeof(l_intptr_t) * boxa->nalloc,
-                               2 * sizeof(l_intptr_t) * boxa->nalloc)) == NULL)
+                               sizeof(BOX *) * boxa->nalloc,
+                               2 * sizeof(BOX *) * boxa->nalloc)) == NULL)
         return ERROR_INT("new ptr array not returned", procName, 1);
 
     boxa->nalloc = 2 * boxa->nalloc;
@@ -552,6 +562,10 @@ BOX  *box;
 
     PROCNAME("boxaGetBoxGeometry");
 
+    if (px) *px = 0;
+    if (py) *py = 0;
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
     if (!boxa)
         return ERROR_INT("boxa not defined", procName, 1);
     if (index < 0 || index >= boxa->n)
@@ -572,7 +586,7 @@ BOX  *box;
  *  boxaReplaceBox()
  *
  *      Input:  boxa
- *              index  (to the index-th pix)
+ *              index  (to the index-th box)
  *              box (insert to replace existing one)
  *      Return: 0 if OK, 1 on error
  *
@@ -699,7 +713,7 @@ boxaaCreate(l_int32  n)
 {
 BOXAA  *baa;
 
-    PROCNAME("numaaCreate");
+    PROCNAME("boxaaCreate");
 
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
@@ -716,6 +730,44 @@ BOXAA  *baa;
 }
 
 
+/*!
+ *  boxaaCopy()
+ *
+ *      Input:  baas (input boxaa to be copied)
+ *              copyflag (L_COPY, L_CLONE)
+ *      Return: baad (new boxaa, composed of copies or clones of the boxa
+ *                    in baas), or null on error
+ *
+ *  Notes:
+ *      (1) L_COPY makes a copy of each boxa in baas.
+ *          L_CLONE makes a clone of each boxa in baas.
+ */
+BOXAA *
+boxaaCopy(BOXAA   *baas,
+          l_int32  copyflag)
+{
+l_int32  i, n;
+BOXA    *boxa;
+BOXAA   *baad;
+
+    PROCNAME("boxaaCopy");
+
+    if (!baas)
+        return (BOXAA *)ERROR_PTR("baas not defined", procName, NULL);
+    if (copyflag != L_COPY && copyflag != L_CLONE)
+        return (BOXAA *)ERROR_PTR("invalid copyflag", procName, NULL);
+
+    n = boxaaGetCount(baas);
+    baad = boxaaCreate(n);
+    for (i = 0; i < n; i++) {
+        boxa = boxaaGetBoxa(baas, i, copyflag);
+        boxaaAddBoxa(baad, boxa, L_INSERT);
+    }
+
+    return baad;
+}
+
+
 /*!
  *  boxaaDestroy()
  *
@@ -728,7 +780,7 @@ boxaaDestroy(BOXAA  **pbaa)
 l_int32  i;
 BOXAA   *baa;
 
-    PROCNAME("boxnumaaDestroy");
+    PROCNAME("boxaaDestroy");
 
     if (pbaa == NULL) {
         L_WARNING("ptr address is NULL!", procName);
@@ -807,8 +859,8 @@ boxaaExtendArray(BOXAA  *baa)
         return ERROR_INT("baa not defined", procName, 1);
 
     if ((baa->boxa = (BOXA **)reallocNew((void **)&baa->boxa,
-                              sizeof(l_intptr_t) * baa->nalloc,
-                              2 * sizeof(l_intptr_t) * baa->nalloc)) == NULL)
+                              sizeof(BOXA *) * baa->nalloc,
+                              2 * sizeof(BOXA *) * baa->nalloc)) == NULL)
             return ERROR_INT("new ptr array not returned", procName, 1);
 
     baa->nalloc *= 2;
@@ -1089,9 +1141,6 @@ BOXAA  *baa;
  *
  *      Input:  stream
  *      Return: boxaa, or null on error
- *
- *  Notes:
- *      (1) We use PIXA_VERSION_NUMBER for the boxaa.
  */
 BOXAA *
 boxaaReadStream(FILE  *fp)
@@ -1108,7 +1157,7 @@ BOXAA   *baa;
 
     if (fscanf(fp, "\nBoxaa Version %d\n", &version) != 1)
         return (BOXAA *)ERROR_PTR("not a boxaa file", procName, NULL);
-    if (version != PIXA_VERSION_NUMBER)
+    if (version != BOXAA_VERSION_NUMBER)
         return (BOXAA *)ERROR_PTR("invalid boxa version", procName, NULL);
     if (fscanf(fp, "Number of boxa = %d\n", &n) != 1)
         return (BOXAA *)ERROR_PTR("not a boxaa file", procName, NULL);
@@ -1117,8 +1166,8 @@ BOXAA   *baa;
         return (BOXAA *)ERROR_PTR("boxaa not made", procName, NULL);
 
     for (i = 0; i < n; i++) {
-        if (fscanf(fp, " Boxa[%d]: x = %d, y = %d, w = %d, h = %d\n",
-                &ignore, &x, &y, &w, &h) != 5)
+        if (fscanf(fp, "\nBoxa[%d] extent: x = %d, y = %d, w = %d, h = %d",
+                   &ignore, &x, &y, &w, &h) != 5)
             return (BOXAA *)ERROR_PTR("boxa descr not valid", procName, NULL);
         if ((boxa = boxaReadStream(fp)) == NULL)
             return (BOXAA *)ERROR_PTR("boxa not made", procName, NULL);
@@ -1128,7 +1177,6 @@ BOXAA   *baa;
     return baa;
 }
 
-
 /*!
  *  boxaaWrite()
  *
@@ -1165,9 +1213,6 @@ FILE  *fp;
  *      Input: stream
  *             boxaa
  *      Return: 0 if OK, 1 on error
- *
- *  Notes:
- *      (1) We use PIXA_VERSION_NUMBER for the boxaa.
  */
 l_int32
 boxaaWriteStream(FILE   *fp,
@@ -1185,14 +1230,15 @@ BOXA    *boxa;
         return ERROR_INT("baa not defined", procName, 1);
 
     n = boxaaGetCount(baa);
-    fprintf(fp, "\nBoxaa Version %d\n", PIXA_VERSION_NUMBER);
+    fprintf(fp, "\nBoxaa Version %d\n", BOXAA_VERSION_NUMBER);
     fprintf(fp, "Number of boxa = %d\n", n);
+
     for (i = 0; i < n; i++) {
         if ((boxa = boxaaGetBoxa(baa, i, L_CLONE)) == NULL)
             return ERROR_INT("boxa not found", procName, 1);
         boxaGetExtent(boxa, NULL, NULL, &box);
         boxGetGeometry(box, &x, &y, &w, &h);
-        fprintf(fp, " Boxa[%d]: x = %d, y = %d, w = %d, h = %d\n",
+        fprintf(fp, "\nBoxa[%d] extent: x = %d, y = %d, w = %d, h = %d",
                 i, x, y, w, h);
         boxaWriteStream(fp, boxa);
         boxDestroy(&box);
@@ -1239,9 +1285,6 @@ BOXA  *boxa;
  *
  *      Input:  stream
  *      Return: boxa, or null on error
- *
- *  Notes:
- *      (1) We use PIXA_VERSION_NUMBER for the boxa.
  */
 BOXA *
 boxaReadStream(FILE  *fp)
@@ -1258,7 +1301,7 @@ BOXA    *boxa;
 
     if (fscanf(fp, "\nBoxa Version %d\n", &version) != 1)
         return (BOXA *)ERROR_PTR("not a boxa file", procName, NULL);
-    if (version != PIXA_VERSION_NUMBER)
+    if (version != BOXA_VERSION_NUMBER)
         return (BOXA *)ERROR_PTR("invalid boxa version", procName, NULL);
     if (fscanf(fp, "Number of boxes = %d\n", &n) != 1)
         return (BOXA *)ERROR_PTR("not a boxa file", procName, NULL);
@@ -1315,9 +1358,6 @@ FILE  *fp;
  *      Input: stream
  *             boxa
  *      Return: 0 if OK, 1 on error
- *
- *  Notes:
- *      (1) We use PIXA_VERSION_NUMBER for the boxa.
  */
 l_int32
 boxaWriteStream(FILE  *fp,
@@ -1334,7 +1374,7 @@ BOX     *box;
         return ERROR_INT("boxa not defined", procName, 1);
 
     n = boxaGetCount(boxa);
-    fprintf(fp, "\nBoxa Version %d\n", PIXA_VERSION_NUMBER);
+    fprintf(fp, "\nBoxa Version %d\n", BOXA_VERSION_NUMBER);
     fprintf(fp, "Number of boxes = %d\n", n);
     for (i = 0; i < n; i++) {
         if ((box = boxaGetBox(boxa, i, L_CLONE)) == NULL)
@@ -1382,3 +1422,86 @@ boxPrintStreamInfo(FILE  *fp,
 }
 
 
+
+/*---------------------------------------------------------------------*
+ *    Version for reading v.2 boxaa; kept for backward compatibility   *
+ *---------------------------------------------------------------------*/
+/*!
+ *  boxaaReadVersion2()
+ *
+ *      Input:  filename
+ *      Return: boxaa, or null on error
+ *
+ *  Notes:
+ *      (1) These old functions only work on version 2 boxaa.
+ *          They will go to an archive directory sometime after Sept 2009.
+ *      (2) The current format uses BOXAA_VERSION_NUMBER == 3)
+ */
+BOXAA *
+boxaaReadVersion2(const char  *filename)
+{
+FILE   *fp;
+BOXAA  *baa;
+
+    PROCNAME("boxaaReadVersion2");
+
+    if (!filename)
+        return (BOXAA *)ERROR_PTR("filename not defined", procName, NULL);
+    if ((fp = fopenReadStream(filename)) == NULL)
+        return (BOXAA *)ERROR_PTR("stream not opened", procName, NULL);
+
+    if ((baa = boxaaReadStreamVersion2(fp)) == NULL) {
+        fclose(fp);
+        return (BOXAA *)ERROR_PTR("boxaa not read", procName, NULL);
+    }
+
+    fclose(fp);
+    return baa;
+}
+
+
+/*!
+ *  boxaaReadStreamVersion2()
+ *
+ *      Input:  stream
+ *      Return: boxaa, or null on error
+ *
+ */
+BOXAA *
+boxaaReadStreamVersion2(FILE  *fp)
+{
+l_int32  n, i, x, y, w, h, version;
+l_int32  ignore;
+BOXA    *boxa;
+BOXAA   *baa;
+
+    PROCNAME("boxaaReadStreamVersion2");
+
+    if (!fp)
+        return (BOXAA *)ERROR_PTR("stream not defined", procName, NULL);
+
+    if (fscanf(fp, "\nBoxaa Version %d\n", &version) != 1)
+        return (BOXAA *)ERROR_PTR("not a boxaa file", procName, NULL);
+    if (version != 2) {
+        fprintf(stderr, "This is version %d\n", version);
+        return (BOXAA *)ERROR_PTR("Not old version 2", procName, NULL);
+    }
+    if (fscanf(fp, "Number of boxa = %d\n", &n) != 1)
+        return (BOXAA *)ERROR_PTR("not a boxaa file", procName, NULL);
+
+    if ((baa = boxaaCreate(n)) == NULL)
+        return (BOXAA *)ERROR_PTR("boxaa not made", procName, NULL);
+
+    for (i = 0; i < n; i++) {
+        if (fscanf(fp, " Boxa[%d]: x = %d, y = %d, w = %d, h = %d\n",
+                &ignore, &x, &y, &w, &h) != 5)
+            return (BOXAA *)ERROR_PTR("boxa descr not valid", procName, NULL);
+        if ((boxa = boxaReadStream(fp)) == NULL)
+            return (BOXAA *)ERROR_PTR("boxa not made", procName, NULL);
+        boxaaAddBoxa(baa, boxa, L_INSERT);
+    }
+
+    return baa;
+}
+
+
