@@ -261,6 +261,7 @@ static const l_int32   xpostab[] = {-1, -1, 0, 1, 1, 1, 0, -1};
 static const l_int32   ypostab[] = {0, -1, -1, -1, 0, 1, 1, 1};
 static const l_int32   qpostab[] = {6, 6, 0, 0, 2, 2, 4, 4};
 
+
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG_PRINT   0
 #endif   /* NO CONSOLE_IO */
@@ -469,8 +470,8 @@ ccbaExtendArray(CCBORDA  *ccba)
         return ERROR_INT("ccba not defined", procName, 1);
 
     if ((ccba->ccb = (CCBORD **)reallocNew((void **)&ccba->ccb,
-                                sizeof(l_intptr_t) * ccba->nalloc,
-                                2 * sizeof(l_intptr_t) * ccba->nalloc)) == NULL)
+                                sizeof(CCBORD *) * ccba->nalloc,
+                                2 * sizeof(CCBORD *) * ccba->nalloc)) == NULL)
         return ERROR_INT("new ptr array not returned", procName, 1);
 
     ccba->nalloc = 2 * ccba->nalloc;
@@ -1109,7 +1110,6 @@ l_int32
 ccbaGenerateGlobalLocs(CCBORDA  *ccba)
 {
 l_int32  ncc, nb, n, i, j, k, xul, yul, x, y;
-BOX     *box;
 CCBORD  *ccb;
 PTAA    *ptaal, *ptaag;
 PTA     *ptal, *ptag;
@@ -1123,12 +1123,8 @@ PTA     *ptal, *ptag;
     for (i = 0; i < ncc; i++) {
         ccb = ccbaGetCcb(ccba, i);
 
-            /* Get the (xul, yul) of the c.c. */
-        if ((box = boxaGetBox(ccb->boxa, 0, L_CLONE)) == NULL)
-            return ERROR_INT("bounding rectangle not found", procName, 1);
-        xul = box->x;  /* UL corner in global coords */
-        yul = box->y;
-        boxDestroy(&box);
+            /* Get the UL corner in global coords, (xul, yul), of the c.c. */
+        boxaGetBoxGeometry(ccb->boxa, 0, &xul, &yul, NULL, NULL);
 
             /* Make a new global ptaa, removing any old one */
         ptaal = ccb->local;
@@ -1241,7 +1237,7 @@ PTAA    *ptaal;  /* local chain code */
  *  ccbaStepChainsToPixCoords()
  *
  *      Input:  ccba (with step chains numaa of borders)
- *              coordtype  (GLOBAL_COORDS or LOCAL_COORDS)
+ *              coordtype  (CCB_GLOBAL_COORDS or CCB_LOCAL_COORDS)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -1257,7 +1253,6 @@ ccbaStepChainsToPixCoords(CCBORDA  *ccba,
 {
 l_int32  ncc, nb, n, i, j, k;
 l_int32  xul, yul, xstart, ystart, x, y, stepdir;
-BOX     *box;
 BOXA    *boxa;
 CCBORD  *ccb;
 NUMA    *na;
@@ -1269,7 +1264,7 @@ PTA     *ptas, *ptan;
 
     if (!ccba)
         return ERROR_INT("ccba not defined", procName, 1);
-    if (coordtype != GLOBAL_COORDS && coordtype != LOCAL_COORDS)
+    if (coordtype != CCB_GLOBAL_COORDS && coordtype != CCB_LOCAL_COORDS)
         return ERROR_INT("coordtype not valid", procName, 1);
 
     ncc = ccbaGetCount(ccba);  /* number of c.c. */
@@ -1284,28 +1279,26 @@ PTA     *ptas, *ptan;
 
             /* For global coords, get the (xul, yul) of the c.c.;
              * otherwise, use relative coords. */
-        if (coordtype == LOCAL_COORDS) {
+        if (coordtype == CCB_LOCAL_COORDS) {
             xul = 0;
             yul = 0;
         }
-        else {   /* coordtype == GLOBAL_COORDS */
-            if ((box = boxaGetBox(boxa, 0, L_CLONE)) == NULL)
+        else {  /* coordtype == CCB_GLOBAL_COORDS */
+                /* Get UL corner in global coords */
+            if (boxaGetBoxGeometry(boxa, 0, &xul, &yul, NULL, NULL))
                 return ERROR_INT("bounding rectangle not found", procName, 1);
-            xul = box->x;  /* UL corner in global coords */
-            yul = box->y;
-            boxDestroy(&box);
         }
 
             /* Make a new ptaa, removing any old one */
         nb = numaaGetCount(naa);   /* number of borders */
         if ((ptaan = ptaaCreate(nb)) == NULL)
             return ERROR_INT("ptaan not made", procName, 1);
-        if (coordtype == LOCAL_COORDS) {
+        if (coordtype == CCB_LOCAL_COORDS) {
             if (ccb->local)   /* remove old one */
                 ptaaDestroy(&ccb->local);
             ccb->local = ptaan;  /* save new local chain */
         }
-        else {   /* coordtype == GLOBAL_COORDS */
+        else {   /* coordtype == CCB_GLOBAL_COORDS */
             if (ccb->global)   /* remove old one */
                 ptaaDestroy(&ccb->global);
             ccb->global = ptaan;  /* save new global chain */
@@ -1341,7 +1334,7 @@ PTA     *ptas, *ptan;
  *  ccbaGenerateSPGlobalLocs()
  *
  *      Input:  ccba
- *              ptsflag  (SAVE_ALL_PTS or SAVE_TURNING_PTS)
+ *              ptsflag  (CCB_SAVE_ALL_PTS or CCB_SAVE_TURNING_PTS)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -1360,7 +1353,6 @@ ccbaGenerateSPGlobalLocs(CCBORDA  *ccba,
 {
 l_int32  ncc, npt, i, j, xul, yul, x, y, delx, dely;
 l_int32  xp, yp, delxp, delyp;   /* prev point and increments */
-BOX     *box;
 CCBORD  *ccb;
 PTA     *ptal, *ptag;
 
@@ -1380,12 +1372,9 @@ PTA     *ptal, *ptag;
     for (i = 0; i < ncc; i++) {
         ccb = ccbaGetCcb(ccba, i);
 
-            /* Get the (xul, yul) of the c.c. */
-        if ((box = boxaGetBox(ccb->boxa, 0, L_CLONE)) == NULL)
+            /* Get the UL corner in global coords, (xul, yul), of the c.c. */
+        if (boxaGetBoxGeometry(ccb->boxa, 0, &xul, &yul, NULL, NULL))
             return ERROR_INT("bounding rectangle not found", procName, 1);
-        xul = box->x;  /* UL corner in global coords */
-        yul = box->y;
-        boxDestroy(&box);  /* clone ref */
 
             /* Make a new spglobal pta, removing any old one */
         ptal = ccb->splocal;
@@ -1397,13 +1386,13 @@ PTA     *ptal, *ptag;
         ccb->spglobal = ptag;  /* save new one */
 
             /* Convert local to global */
-        if (ptsflag == SAVE_ALL_PTS) {
+        if (ptsflag == CCB_SAVE_ALL_PTS) {
             for (j = 0; j < npt; j++) {
                 ptaGetIPt(ptal, j, &x, &y);
                 ptaAddPt(ptag, x  + xul, y + yul);
             }
         }
-        else {   /* ptsflag = SAVE_TURNING_PTS */
+        else {   /* ptsflag = CCB_SAVE_TURNING_PTS */
             ptaGetIPt(ptal, 0, &xp, &yp);   /* get the 1st pt */
             ptaAddPt(ptag, xp  + xul, yp + yul);   /* save the 1st pt */
             if (npt == 2) {  /* get and save the 2nd pt  */
@@ -1958,15 +1947,11 @@ PTA     *pta;
             if ((box = boxaGetBox(boxa, j, L_CLONE)) == NULL)
                 return (PIX *)ERROR_PTR("b. box not found", procName, NULL);
             if (j == 0) {
-                xul = box->x;
-                yul = box->y;
+                boxGetGeometry(box, &xul, &yul, &w, &h);
                 xoff = yoff = 0;
-            } else {
-                xoff = box->x;
-                yoff = box->y;
-            }
-            w = box->w;
-            h = box->h;
+            } else
+                boxGetGeometry(box, &xoff, &yoff, &w, &h);
+            boxDestroy(&box);
 
                 /* Render the border in a minimum-sized pix;
                  * subtract xoff and yoff because the pixel
@@ -1974,7 +1959,6 @@ PTA     *pta;
                  * we need it relative to just the hole border. */
             if ((pixt = pixCreate(w, h, 1)) == NULL)
                 return (PIX *)ERROR_PTR("pixt not made", procName, NULL);
-            boxDestroy(&box);
             pta = ptaaGetPta(ptaa, j, L_CLONE);
             n = ptaGetCount(pta);   /* number of pixels in the border */
             for (k = 0; k < n; k++) {
@@ -2050,7 +2034,6 @@ ccbaDisplayImage2(CCBORDA  *ccba)
 {
 l_int32  ncc, nb, n, i, j, k, x, y, xul, yul, w, h;
 l_int32  fpx, fpy, spx, spy, xs, ys;
-BOX     *box;
 BOXA    *boxa;
 CCBORD  *ccb;
 PIX     *pixd, *pixc, *pixs;
@@ -2073,17 +2056,12 @@ PTA     *pta;
         ccb = ccbaGetCcb(ccba, i);
         if ((boxa = ccb->boxa) == NULL)
             return (PIX *)ERROR_PTR("boxa not found", procName, NULL);
-        if ((box = boxaGetBox(boxa, 0, L_CLONE)) == NULL)
+        if (boxaGetBoxGeometry(boxa, 0, &xul, &yul, &w, &h))
             return (PIX *)ERROR_PTR("b. box not found", procName, NULL);
-        xul = box->x;
-        yul = box->y;
-        w = box->w;
-        h = box->h;
         if ((pixc = pixCreate(w + 2, h + 2, 1)) == NULL)
             return (PIX *)ERROR_PTR("pixc not made", procName, NULL);
         if ((pixs = pixCreateTemplate(pixc)) == NULL)
             return (PIX *)ERROR_PTR("pixs not made", procName, NULL);
-        boxDestroy(&box);
 
         if ((ptaa = ccb->local) == NULL) {
             L_WARNING("local chain array not found", procName);
@@ -2202,12 +2180,11 @@ ccbaWriteStream(FILE     *fp,
 char      strbuf[256];
 l_uint8   bval;
 l_uint8  *datain, *dataout;
-l_int32   i, j, k, val, startx, starty; 
+l_int32   i, j, k, bx, by, bw, bh, val, startx, starty; 
 l_int32   inbytes, outbytes;
 l_int32   ncc, nb, n;
 l_uint32  w, h;
 BBUFFER  *bbuf;
-BOX      *box;
 CCBORD   *ccb;
 NUMA     *na;
 NUMAA    *naa;
@@ -2215,6 +2192,10 @@ PTA      *pta;
 
     PROCNAME("ccbaWriteStream");
 
+#if  !HAVE_LIBZ  /* defined in environ.h */
+    return ERROR_INT("no libz: can't write data", procName, 1);
+#else
+
     if (!fp)
         return ERROR_INT("stream not open", procName, 1);
     if (!ccba)
@@ -2232,12 +2213,12 @@ PTA      *pta;
     bbufferRead(bbuf, (l_uint8 *)&h, 4);  /* height */
     for (i = 0; i < ncc; i++) {
         ccb = ccbaGetCcb(ccba, i);
-        box = boxaGetBox(ccb->boxa, 0, L_CLONE);
-        bbufferRead(bbuf, (l_uint8 *)&box->x, 4);  /* ulx of c.c. */
-        bbufferRead(bbuf, (l_uint8 *)&box->y, 4);  /* uly of c.c. */
-        bbufferRead(bbuf, (l_uint8 *)&box->w, 4);  /* w of c.c. */
-        bbufferRead(bbuf, (l_uint8 *)&box->h, 4);  /* h of c.c. */
-        boxDestroy(&box);
+        if (boxaGetBoxGeometry(ccb->boxa, 0, &bx, &by, &bw, &bh))
+            return ERROR_INT("bounding box not found", procName, 1);
+        bbufferRead(bbuf, (l_uint8 *)&bx, 4);  /* ulx of c.c. */
+        bbufferRead(bbuf, (l_uint8 *)&by, 4);  /* uly of c.c. */
+        bbufferRead(bbuf, (l_uint8 *)&bw, 4);  /* w of c.c. */
+        bbufferRead(bbuf, (l_uint8 *)&bh, 4);  /* h of c.c. */
         if ((naa = ccb->step) == NULL) {
             ccbaGenerateStepChains(ccba);
             naa = ccb->step;
@@ -2281,6 +2262,8 @@ PTA      *pta;
     FREE(datain);
     FREE(dataout);
     return 0;
+
+#endif  /* !HAVE_LIBZ */
 }
 
 
@@ -2352,6 +2335,10 @@ NUMAA    *step;
 
     PROCNAME("ccbaReadStream");
 
+#if  !HAVE_LIBZ  /* defined in environ.h */
+    return (CCBORDA *)ERROR_PTR("no libz: can't read data", procName, NULL);
+#else
+
     if (!fp)
         return (CCBORDA *)ERROR_PTR("stream not open", procName, NULL);
 
@@ -2436,6 +2423,8 @@ NUMAA    *step;
     FREE(dataout);
 
     return ccba;
+
+#endif  /* !HAVE_LIBZ */
 }
 
 
@@ -2492,6 +2481,7 @@ char     line2[] = "<svg>";
 char     line3[] = "<polygon style=\"stroke-width:1;stroke:black;\" points=\"";
 char     line4[] = "\" />";
 char     line5[] = "</svg>";
+char     space[] = " ";
 l_int32  i, j, ncc, npt, x, y;
 CCBORD  *ccb;
 PTA     *pta;
@@ -2527,7 +2517,7 @@ SARRAY  *sa;
         ccbDestroy(&ccb);
     }
     sarrayAddString(sa, line5, 1);
-    sarrayAddString(sa, " ", 1);
+    sarrayAddString(sa, space, 1);
 
     svgstr = sarrayToString(sa, 1);
 /*    fprintf(stderr, "%s", svgstr); */
@@ -2536,5 +2526,3 @@ SARRAY  *sa;
     return svgstr;
 }
 
-
-
