@@ -27,6 +27,8 @@
  *           l_int32     pixcmapAddBlackOrWhite()
  *           l_int32     pixcmapSetBlackAndWhite()
  *           l_int32     pixcmapGetCount()
+ *           l_int32     pixcmapGetDepth()
+ *           l_int32     pixcmapGetMinDepth()
  *           l_int32     pixcmapGetFreeCount()
  *           l_int32     pixcmapClear()
  *
@@ -35,16 +37,23 @@
  *           l_int32     pixcmapResetColor()
  *           l_int32     pixcmapGetIndex()
  *           l_int32     pixcmapHasColor()
+ *           l_int32     pixcmapCountGrayColors()
  *           l_int32     pixcmapGetRankIntensity()
  *           l_int32     pixcmapGetNearestIndex()
+ *           l_int32     pixcmapGetNearestGrayIndex()
  *           l_int32     pixcmapGetExtremeValue()
  *
+ *      Colormap conversion
+ *           PIXCMAP    *pixcmapGrayToColor()
+ *           PIXCMAP    *pixcmapColorToGray()
+ *
  *      Colormap I/O
  *           l_int32     pixcmapReadStream()
  *           l_int32     pixcmapWriteStream()
  *
  *      Extract colormap arrays
  *           l_int32     pixcmapToArrays()
+ *           l_int32     pixcmapToRGBTable()
  *
  *      Colormap transforms
  *           l_int32     pixcmapGammaTRC()
@@ -98,23 +107,30 @@ PIXCMAP    *cmap;
  *  pixcmapCreateRandom()
  *
  *      Input:  depth (bpp, of pix; 2, 4 or 8)
+ *              hasblack (1 if the first color is black; 0 if no black)
+ *              haswhite (1 if the last color is white; 0 if no white)
  *      Return: cmap, or null on error
  *
  *  Notes:
  *      (1) This sets up a colormap with random colors,
- *          where the first color is black, the last color
- *          is white, and the remaining (2^(depth) - 2)
- *          colors are chosen randomly.
- *      (2) Because rand() is seeded, it might disrupt otherwise 
+ *          where the first color is optionally black, the last color
+ *          is optionally white, and the remaining colors are
+ *          chosen randomly.
+ *      (2) The number of randomly chosen colors is:
+ *               2^(depth) - haswhite - hasblack
+ *      (3) Because rand() is seeded, it might disrupt otherwise 
  *          deterministic results if also used elsewhere in a program.
- *      (3) Modern rand()s have equal randomness in low and high order
- *          bits, but older ones don't, so we use the high order bits.
  *      (4) rand() is not threadsafe, and will generate garbage if run
  *          on multiple threads at once -- though garbage is generally
  *          what you want from a random number generator!
+ *      (5) Modern rand()s have equal randomness in low and high order
+ *          bits, but older ones don't.  Here, we're just using rand()
+ *          to choose colors for output.
  */
 PIXCMAP *
-pixcmapCreateRandom(l_int32  depth)
+pixcmapCreateRandom(l_int32  depth,
+                    l_int32  hasblack,
+                    l_int32  haswhite)
 {
 l_int32   ncolors, i;
 l_int32   red[256], green[256], blue[256];
@@ -122,19 +138,23 @@ PIXCMAP  *cmap;
 
     PROCNAME("pixcmapCreateRandom");
 
-    if (depth != 2 && depth !=4 && depth != 8)
+    if (depth != 2 && depth != 4 && depth != 8)
         return (PIXCMAP *)ERROR_PTR("depth not in {2, 4, 8}", procName, NULL);
+    if (hasblack != 0) hasblack = 1;
+    if (haswhite != 0) haswhite = 1;
 
     cmap = pixcmapCreate(depth);
-    pixcmapAddColor(cmap, 0, 0, 0);  /* first color is black */
     ncolors = 1 << depth;
-    for (i = 1; i < ncolors - 1; i++) {
-        red[i] = (rand() >> 16) % ncolors;
-        green[i] = (rand() >> 16) % ncolors;
-        blue[i] = (rand() >> 16) % ncolors;
+    if (hasblack)  /* first color is optionally black */
+        pixcmapAddColor(cmap, 0, 0, 0);
+    for (i = hasblack; i < ncolors - haswhite; i++) {
+        red[i] = (l_uint32)rand() & 0xff;
+        green[i] = (l_uint32)rand() & 0xff;
+        blue[i] = (l_uint32)rand() & 0xff;
         pixcmapAddColor(cmap, red[i], green[i], blue[i]);
     }
-    pixcmapAddColor(cmap, 255, 255, 255);  /* last color is white */
+    if (haswhite)  /* last color is optionally white */
+        pixcmapAddColor(cmap, 255, 255, 255);
 
     return cmap;
 }
@@ -277,13 +297,13 @@ RGBA_QUAD  *cta;
  *              rval, gval, bval (colormap entry to be added; each number
  *                                is in range [0, ... 255])
  *              &index (<return> index of color)
- *      Return: 0 if OK, 1 on error
+ *      Return: 0 if OK, 1 on error; 2 if unable to add color
  *
  *  Notes:
  *      (1) This only adds color if not already there.
- *      (2) This sets index to the input color.
- *      (3) Returns 1 if unable to add this color, so the caller
- *          should check the return value.
+ *      (2) This returns the index of the new (or existing) color.
+ *      (3) Returns 2 with a warning if unable to add this color;
+ *          the caller should check the return value.
  */
 l_int32
 pixcmapAddNewColor(PIXCMAP  *cmap,
@@ -300,13 +320,17 @@ pixcmapAddNewColor(PIXCMAP  *cmap,
     if (!cmap)
         return ERROR_INT("cmap not defined", procName, 1);
 
+        /* Check if the color is already present. */
     if (!pixcmapGetIndex(cmap, rval, gval, bval, pindex))  /* found */
         return 0;
 
-    if (cmap->n >= cmap->nalloc)
-        return ERROR_INT("no free color entries", procName, 1);
+        /* We need to add the color.  Is there room? */
+    if (cmap->n >= cmap->nalloc) {
+        L_WARNING("no free color entries", procName);
+        return 2;
+    }
 
-        /* ok, we have to add a color and there is room */
+        /* There's room.  Add it. */
     pixcmapAddColor(cmap, rval, gval, bval);
     *pindex = pixcmapGetCount(cmap) - 1;
     return 0;
@@ -318,7 +342,7 @@ pixcmapAddNewColor(PIXCMAP  *cmap,
  *
  *      Input:  cmap
  *              color (0 for black, 1 for white) 
- *              &index (<return> index of color)
+ *              &index (<optional return> index of color; can be null)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -332,27 +356,29 @@ pixcmapAddBlackOrWhite(PIXCMAP  *cmap,
                        l_int32   color,
                        l_int32  *pindex)
 {
+l_int32  index;
+
     PROCNAME("pixcmapAddBlackOrWhite");
 
-    if (!pindex)
-        return ERROR_INT("&index not defined", procName, 1);
-    *pindex = 0;
+    if (pindex) *pindex = 0;
     if (!cmap)
         return ERROR_INT("cmap not defined", procName, 1);
 
     if (color == 0) {  /* black */
         if (pixcmapGetFreeCount(cmap) > 0)
-            pixcmapAddNewColor(cmap, 0, 0, 0, pindex);
+            pixcmapAddNewColor(cmap, 0, 0, 0, &index);
         else
-            pixcmapGetRankIntensity(cmap, 0.0, pindex);
+            pixcmapGetRankIntensity(cmap, 0.0, &index);
     }
     else {  /* white */
         if (pixcmapGetFreeCount(cmap) > 0)
-            pixcmapAddNewColor(cmap, 255, 255, 255, pindex);
+            pixcmapAddNewColor(cmap, 255, 255, 255, &index);
         else
-            pixcmapGetRankIntensity(cmap, 1.0, pindex);
+            pixcmapGetRankIntensity(cmap, 1.0, &index);
     }
 
+    if (pindex)
+        *pindex = index;
     return 0;
 }
 
@@ -397,7 +423,7 @@ l_int32  index;
  *      Return: count, or 0 on error
  */
 l_int32
-pixcmapGetCount(PIXCMAP *cmap)
+pixcmapGetCount(PIXCMAP  *cmap)
 {
     PROCNAME("pixcmapGetCount");
 
@@ -415,7 +441,7 @@ pixcmapGetCount(PIXCMAP *cmap)
  *      Return: free entries, or 0 on error
  */
 l_int32
-pixcmapGetFreeCount(PIXCMAP *cmap)
+pixcmapGetFreeCount(PIXCMAP  *cmap)
 {
     PROCNAME("pixcmapGetFreeCount");
 
@@ -426,6 +452,60 @@ pixcmapGetFreeCount(PIXCMAP *cmap)
 }
 
 
+/*!
+ *  pixcmapGetDepth()
+ *
+ *      Input:  cmap
+ *      Return: depth, or 0 on error
+ */
+l_int32
+pixcmapGetDepth(PIXCMAP  *cmap)
+{
+    PROCNAME("pixcmapGetDepth");
+
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 0);
+
+    return cmap->depth;
+}
+
+
+/*!
+ *  pixcmapGetMinDepth()
+ *
+ *      Input:  cmap
+ *              &mindepth (<return> minimum depth to support the colormap)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) On error, &mindepth is returned as 0.
+ */
+l_int32
+pixcmapGetMinDepth(PIXCMAP  *cmap,
+                   l_int32  *pmindepth)
+{
+l_int32  ncolors;
+
+    PROCNAME("pixcmapGetMinDepth");
+
+    if (!pmindepth)
+        return ERROR_INT("&mindepth not defined", procName, 1);
+    *pmindepth = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+
+    ncolors = pixcmapGetCount(cmap);
+    if (ncolors <= 4)
+        *pmindepth = 2;
+    else if (ncolors <= 16)
+        *pmindepth = 4;
+    else  /* ncolors > 16 */
+        *pmindepth = 8;
+
+    return 0;
+}
+
+
 /*!
  *  pixcmapClear()
  *
@@ -435,7 +515,7 @@ pixcmapGetFreeCount(PIXCMAP *cmap)
  *  Note: this removes the colors by setting the count to 0.
  */
 l_int32
-pixcmapClear(PIXCMAP *cmap)
+pixcmapClear(PIXCMAP  *cmap)
 {
     PROCNAME("pixcmapClear");
 
@@ -578,11 +658,11 @@ l_int32  *rmap, *gmap, *bmap;
 
     PROCNAME("pixcmapHasColor");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!pcolor)
         return ERROR_INT("&color not defined", procName, 1);
     *pcolor = FALSE;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
 
     if (pixcmapToArrays(cmap, &rmap, &gmap, &bmap))
         return ERROR_INT("colormap arrays not made", procName, 1);
@@ -601,6 +681,48 @@ l_int32  *rmap, *gmap, *bmap;
 }
 
 
+/*!
+ *  pixcmapCountGrayColors()
+ *
+ *      Input:  cmap
+ *              &ngray (<return> number of gray colors)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This counts the unique gray colors, including black and white.
+ */
+l_int32
+pixcmapCountGrayColors(PIXCMAP  *cmap,
+                       l_int32  *pngray)
+{
+l_int32   n, i, rval, gval, bval, count;
+l_int32  *array;
+
+    PROCNAME("pixcmapCountGrayColors");
+
+    if (!pngray)
+        return ERROR_INT("&ngray not defined", procName, 1);
+    *pngray = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+
+    array = (l_int32 *)CALLOC(256, sizeof(l_int32));
+    n = pixcmapGetCount(cmap);
+    count = 0;
+    for (i = 0; i < n; i++) {
+        pixcmapGetColor(cmap, i, &rval, &gval, &bval);
+        if ((rval == gval) && (rval == bval) && (array[rval] == 0)) {
+            array[rval] = 1;
+            count++;
+        }
+    }
+
+    FREE(array);
+    *pngray = count;
+    return 0;
+}
+
+
 /*!
  *  pixcmapGetRankIntensity()
  *
@@ -701,6 +823,58 @@ RGBA_QUAD  *cta;
 }
 
 
+/*!
+ *  pixcmapGetNearestGrayIndex()
+ *
+ *      Input:  cmap
+ *              val (gray value to search for; in range [0, ... 255])
+ *              &index (<return> the index of the nearest color)
+ *      Return: 0 if OK, 1 on error (caller must check)
+ *
+ *  Notes:
+ *      (1) This should be used on gray colormaps.  It uses only the
+ *          green value of the colormap.
+ *      (2) Returns the index of the exact color if possible, otherwise the
+ *          index of the color closest to the target color.
+ */
+l_int32
+pixcmapGetNearestGrayIndex(PIXCMAP  *cmap,
+                           l_int32   val,
+                           l_int32  *pindex)
+{
+l_int32     i, n, dist, mindist;
+RGBA_QUAD  *cta;
+
+    PROCNAME("pixcmapGetNearestGrayIndex");
+
+    if (!pindex)
+        return ERROR_INT("&index not defined", procName, 1);
+    *pindex = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+    if (val < 0 || val > 255)
+        return ERROR_INT("val not in [0 ... 255]", procName, 1);
+
+    if ((cta = (RGBA_QUAD *)cmap->array) == NULL)
+        return ERROR_INT("cta not defined(!)", procName, 1);
+    n = pixcmapGetCount(cmap);
+
+    mindist = 256;
+    for (i = 0; i < n; i++) {
+        dist = cta[i].green - val;
+        dist = L_ABS(dist);
+        if (dist < mindist) {
+            *pindex = i;
+            if (dist == 0)
+                break;
+            mindist = dist;
+        }
+    }
+
+    return 0;
+}
+
+
 /*!
  *  pixcmapGetExtremeValue()
  *
@@ -727,12 +901,15 @@ l_int32  i, n, rval, gval, bval, extrval, extgval, extbval;
 
     PROCNAME("pixcmapGetExtremeValue");
 
+    if (!prval && !pgval && !pbval)
+        return ERROR_INT("no result requested for return", procName, 1);
+    if (prval) *prval = 0;
+    if (pgval) *pgval = 0;
+    if (pbval) *pbval = 0;
     if (!cmap)
         return ERROR_INT("cmap not defined", procName, 1);
     if (type != L_CHOOSE_MIN && type != L_CHOOSE_MAX)
         return ERROR_INT("invalid type", procName, 1);
-    if (!prval && !pgval && !pbval)
-        return ERROR_INT("no result requested for return", procName, 1);
 
     if (type == L_CHOOSE_MIN) {
         extrval = 100000;
@@ -765,6 +942,96 @@ l_int32  i, n, rval, gval, bval, extrval, extgval, extbval;
 }
 
 
+/*-------------------------------------------------------------*
+ *                       Colormap conversion                   *
+ *-------------------------------------------------------------*/
+/*!
+ *  pixcmapGrayToColor()
+ *
+ *      Input:  color
+ *      Return: cmap, or null on error
+ *
+ *  Notes:
+ *      (1) This creates a colormap that maps from gray to
+ *          a specific color.  In the mapping, each component
+ *          is faded to white, depending on the gray value.
+ *      (2) In use, this is simply attached to a grayscale pix
+ *          to give it the input color.
+ */
+PIXCMAP *
+pixcmapGrayToColor(l_uint32  color)
+{
+l_int32   i, rval, gval, bval;
+PIXCMAP  *cmap;
+
+    extractRGBValues(color, &rval, &gval, &bval);
+    cmap = pixcmapCreate(8);
+    for (i = 0; i < 256; i++) {
+        pixcmapAddColor(cmap, rval + (i * (255 - rval)) / 255,
+                        gval + (i * (255 - gval)) / 255,
+                        bval + (i * (255 - bval)) / 255);
+    }
+
+    return cmap;
+}
+
+
+/*!
+ *  pixcmapColorToGray()
+ *
+ *      Input:  cmap
+ *              rwt, gwt, bwt  (non-negative; these should add to 1.0)
+ *      Return: cmap (gray), or null on error
+ *
+ *  Notes:
+ *      (1) This creates a gray colormap from an arbitrary colormap.
+ *      (2) In use, attach the output gray colormap to the pix
+ *          (or a copy of it) that provided the input colormap.
+ */
+PIXCMAP *
+pixcmapColorToGray(PIXCMAP   *cmaps,
+                   l_float32  rwt,
+                   l_float32  gwt,
+                   l_float32  bwt)
+{
+l_int32    i, n, rval, gval, bval, val;
+l_float32  sum;
+PIXCMAP   *cmapd;
+
+    PROCNAME("pixcmapColorToGray");
+
+    if (!cmaps)
+        return (PIXCMAP *)ERROR_PTR("cmaps not defined", procName, NULL);
+    if (rwt < 0.0 || gwt < 0.0 || bwt < 0.0)
+        return (PIXCMAP *)ERROR_PTR("weights not all >= 0.0", procName, NULL);
+
+        /* Make sure the sum of weights is 1.0; otherwise, you can get
+         * overflow in the gray value. */
+    sum = rwt + gwt + bwt;
+    if (sum == 0.0) {
+        L_WARNING("all weights zero; setting equal to 1/3", procName);
+        rwt = gwt = bwt = 0.33333;
+        sum = 1.0;
+    }
+    if (L_ABS(sum - 1.0) > 0.0001) {  /* maintain ratios with sum == 1.0 */
+        L_WARNING("weights don't sum to 1; maintaining ratios", procName);
+        rwt = rwt / sum;
+        gwt = gwt / sum;
+        bwt = bwt / sum;
+    }
+
+    cmapd = pixcmapCopy(cmaps);
+    n = pixcmapGetCount(cmapd);
+    for (i = 0; i < n; i++) {
+        pixcmapGetColor(cmapd, i, &rval, &gval, &bval);
+        val = (l_int32)(rwt * rval + gwt * gval + bwt * bval + 0.5);
+        pixcmapResetColor(cmapd, i, val, val, val);
+    }
+        
+    return cmapd;
+}
+
+
 /*-------------------------------------------------------------*
  *                         Colormap I/O                        *
  *-------------------------------------------------------------*/
@@ -893,6 +1160,49 @@ RGBA_QUAD  *cta;
 }
 
 
+/*!
+ *  pixcmapToRGBTable()
+ *
+ *      Input:  colormap
+ *              &tab (<return> table of rgba values for the colormap)
+ *              &ncolors (<optional return> size of table)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixcmapToRGBTable(PIXCMAP    *cmap,
+                  l_uint32  **ptab,
+                  l_int32    *pncolors)
+{
+l_int32    i, ncolors, rval, gval, bval;
+l_uint32  *tab;
+
+    PROCNAME("pixcmapToRGBTable");
+
+    if (!ptab)
+        return ERROR_INT("&tab not defined", procName, 1);
+    *ptab = NULL;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+
+    ncolors = pixcmapGetCount(cmap);
+    if (pncolors)
+        *pncolors = ncolors;
+    if ((tab = (l_uint32 *)CALLOC(ncolors, sizeof(l_uint32))) == NULL)
+        return ERROR_INT("tab not made", procName, 1);
+    *ptab = tab;
+   
+    for (i = 0; i < ncolors; i++) {
+        pixcmapGetColor(cmap, i, &rval, &gval, &bval);
+        composeRGBPixel(rval, gval, bval, &tab[i]);
+    }
+
+/*    for (i = 0; i < ncolors; i++)
+        fprintf(stderr, "Color[%d] = %x\n", i, tab[i]); */
+
+    return 0;
+}
+
+
 /*-------------------------------------------------------------*
  *                     Colormap transforms                     *
  *-------------------------------------------------------------*/
@@ -1054,7 +1364,7 @@ l_int32   i, ncolors, rval, gval, bval;
  *      - replaces: r --> h, g --> s, b --> v
  */
 l_int32
-pixcmapConvertRGBToHSV(PIXCMAP   *cmap)
+pixcmapConvertRGBToHSV(PIXCMAP  *cmap)
 {
 l_int32   i, ncolors, rval, gval, bval, hval, sval, vval;
 
@@ -1085,7 +1395,7 @@ l_int32   i, ncolors, rval, gval, bval, hval, sval, vval;
  *      - replaces: h --> r, s --> g, v --> b
  */
 l_int32
-pixcmapConvertHSVToRGB(PIXCMAP   *cmap)
+pixcmapConvertHSVToRGB(PIXCMAP  *cmap)
 {
 l_int32   i, ncolors, rval, gval, bval, hval, sval, vval;
 
