@@ -98,7 +98,7 @@ static char * makeBarrelshiftString(l_int32 delx, l_int32 dely);
 static SARRAY * sarrayMakeInnerLoopDWACode(SEL *sel, l_int32 index);
 static SARRAY * sarrayMakeWplsCode(SEL *sel);
 
-static  char   *wpldecls[] = {
+static char wpldecls[][53] = {
             "l_int32             wpls2;",
             "l_int32             wpls2, wpls3;",
             "l_int32             wpls2, wpls3, wpls4;",
@@ -130,7 +130,7 @@ static  char   *wpldecls[] = {
             "l_int32             wpls29, wpls30;",
             "l_int32             wpls29, wpls30, wpls31;"};
 
-static  char   *wplgendecls[] = {
+static char wplgendecls[][30] = {
             "l_int32             wpls2;",
             "l_int32             wpls3;",
             "l_int32             wpls4;",
@@ -162,7 +162,7 @@ static  char   *wplgendecls[] = {
             "l_int32             wpls30;",
             "l_int32             wpls31;"};
 
-static  char   *wpldefs[] = {
+static char wpldefs[][25] = {
             "    wpls2 = 2 * wpls;",
             "    wpls3 = 3 * wpls;",
             "    wpls4 = 4 * wpls;",
@@ -194,23 +194,23 @@ static  char   *wpldefs[] = {
             "    wpls30 = 30 * wpls;",
             "    wpls31 = 31 * wpls;"};
 
-static char   *wplstrp[] = {"+ wpls", "+ wpls2", "+ wpls3", "+ wpls4",
-                            "+ wpls5", "+ wpls6", "+ wpls7", "+ wpls8",
-                            "+ wpls9", "+ wpls10", "+ wpls11", "+ wpls12",
-                            "+ wpls13", "+ wpls14", "+ wpls15", "+ wpls16",
-                            "+ wpls17", "+ wpls18", "+ wpls19", "+ wpls20",
-                            "+ wpls21", "+ wpls22", "+ wpls23", "+ wpls24",
-                            "+ wpls25", "+ wpls26", "+ wpls27", "+ wpls28",
-                            "+ wpls29", "+ wpls30", "+ wpls31"};
+static char wplstrp[][10] = {"+ wpls", "+ wpls2", "+ wpls3", "+ wpls4",
+                             "+ wpls5", "+ wpls6", "+ wpls7", "+ wpls8",
+                             "+ wpls9", "+ wpls10", "+ wpls11", "+ wpls12",
+                             "+ wpls13", "+ wpls14", "+ wpls15", "+ wpls16",
+                             "+ wpls17", "+ wpls18", "+ wpls19", "+ wpls20",
+                             "+ wpls21", "+ wpls22", "+ wpls23", "+ wpls24",
+                             "+ wpls25", "+ wpls26", "+ wpls27", "+ wpls28",
+                             "+ wpls29", "+ wpls30", "+ wpls31"};
 
-static char   *wplstrm[] = {"- wpls", "- wpls2", "- wpls3", "- wpls4",
-                            "- wpls5", "- wpls6", "- wpls7", "- wpls8",
-                            "- wpls9", "- wpls10", "- wpls11", "- wpls12",
-                            "- wpls13", "- wpls14", "- wpls15", "- wpls16",
-                            "- wpls17", "- wpls18", "- wpls19", "- wpls20",
-                            "- wpls21", "- wpls22", "- wpls23", "- wpls24",
-                            "- wpls25", "- wpls26", "- wpls27", "- wpls28",
-                            "- wpls29", "- wpls30", "- wpls31"};
+static char wplstrm[][10] = {"- wpls", "- wpls2", "- wpls3", "- wpls4",
+                             "- wpls5", "- wpls6", "- wpls7", "- wpls8",
+                             "- wpls9", "- wpls10", "- wpls11", "- wpls12",
+                             "- wpls13", "- wpls14", "- wpls15", "- wpls16",
+                             "- wpls17", "- wpls18", "- wpls19", "- wpls20",
+                             "- wpls21", "- wpls22", "- wpls23", "- wpls24",
+                             "- wpls25", "- wpls26", "- wpls27", "- wpls28",
+                             "- wpls29", "- wpls30", "- wpls31"};
 
 
 /*!
@@ -376,7 +376,7 @@ SARRAY  *sa1, *sa2, *sa3;
         /* Add static globals */
     sprintf(bigbuf, "\nstatic l_int32   NUM_SELS_GENERATED = %d;", nsels);
     sarrayAddString(sa3, bigbuf, L_COPY);
-    sprintf(bigbuf, "static char  *SEL_NAMES[] = {");
+    sprintf(bigbuf, "static char  SEL_NAMES[][80] = {");
     sarrayAddString(sa3, bigbuf, L_COPY);
     for (i = 0; i < nsels - 1; i++) {
         sprintf(bigbuf,
@@ -480,6 +480,8 @@ fmorphautogen2(SELA        *sela,
 char    *filestr, *linestr, *fname;
 char    *str_doc1, *str_doc2, *str_doc3, *str_doc4, *str_def1;
 char     bigbuf[BUFFER_SIZE];
+char     breakstring[] = "        break;";
+char     staticstring[] = "static void";
 l_int32  i, nsels, nbytes, actstart, end, newstart;
 l_int32  argstart, argend, loopstart, loopend, finalstart, finalend;
 SARRAY  *sa1, *sa2, *sa3, *sa4, *sa5, *sa6;
@@ -574,7 +576,7 @@ SEL     *sel;
         sprintf(bigbuf, "        %s(datad, w, h, wpld, datas, wpls);",
                sarrayGetString(sa2, i, L_NOCOPY));
         sarrayAddString(sa4, bigbuf, L_COPY);
-        sarrayAddString(sa4, "        break;", L_COPY);
+        sarrayAddString(sa4, breakstring, L_COPY);
     }
 
         /* Finish the dispatcher and introduce the low-level code */
@@ -593,7 +595,7 @@ SEL     *sel;
         /* Do all the static functions */
     for (i = 0; i < 2 * nsels; i++) {
 	    /* Generate the function header and add the common args */
-        sarrayAddString(sa4, "static void", L_COPY);
+        sarrayAddString(sa4, staticstring, L_COPY);
         fname = sarrayGetString(sa2, i, L_NOCOPY);
         sprintf(bigbuf, "%s(l_uint32  *datad,", fname);
         sarrayAddString(sa4, bigbuf, L_COPY);
@@ -648,6 +650,7 @@ SEL     *sel;
 static SARRAY *
 sarrayMakeWplsCode(SEL  *sel)
 {
+char     spacestring[] = "    ";
 l_int32  i, j, ymax, dely, allvshifts;
 l_int32  vshift[32];
 SARRAY  *sa;
@@ -713,7 +716,7 @@ SARRAY  *sa;
         }
     }
     
-    sarrayAddString(sa, "    ", 1);
+    sarrayAddString(sa, spacestring, 1);
 
         /* Add definitions */
     for (i = 2; i <= ymax; i++) {
