@@ -40,21 +40,22 @@
  *
  *          Simple thresholding to 2 bpp
  *              void       thresholdTo2bppLow()
- *              void       thresholdTo2bppLineLow()
  *
  *      Thresholding from 8 bpp to 4 bpp
  *
  *          Simple thresholding to 4 bpp
  *              void       thresholdTo4bppLow()
- *              void       thresholdTo4bppLineLow()
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include "allheaders.h"
 
+#ifndef  NO_CONSOLE_IO
+#define DEBUG_UNROLLING 0
+#endif   /* ~NO_CONSOLE_IO */
+
 
 /*------------------------------------------------------------------*
  *             Binarization by Floyd-Steinberg Dithering            *
@@ -271,25 +272,106 @@ thresholdToBinaryLineLow(l_uint32  *lined,
                          l_int32    d,
                          l_int32    thresh)
 {
-l_int32  j, gval;
+l_int32  j, k, gval, scount, dcount;
+l_uint32 sword, dword;
 
     PROCNAME("thresholdToBinaryLineLow");
 
     switch (d)
     {
     case 4:
+            /* Unrolled as 4 source words, 1 dest word */
+        for (j = 0, scount = 0, dcount = 0; j + 31 < w; j += 32) {
+            dword = 0;
+            for (k = 0; k < 4; k++) {
+                sword = lines[scount++];
+                dword <<= 8;
+                gval = (sword >> 28) & 0xf;
+                    /* Trick used here and below: if gval < thresh then
+                     * gval - thresh < 0, so its high-order bit is 1, and
+                     * ((gval - thresh) >> 31) & 1 == 1; likewise, if
+                     * gval >= thresh, then ((gval - thresh) >> 31) & 1 == 0
+                     * Doing it this way avoids a random (and thus easily
+                     * mispredicted) branch on each pixel. */
+                dword |= ((gval - thresh) >> 24) & 128;
+                gval = (sword >> 24) & 0xf;
+                dword |= ((gval - thresh) >> 25) & 64;
+                gval = (sword >> 20) & 0xf;
+                dword |= ((gval - thresh) >> 26) & 32;
+                gval = (sword >> 16) & 0xf;
+                dword |= ((gval - thresh) >> 27) & 16;
+                gval = (sword >> 12) & 0xf;
+                dword |= ((gval - thresh) >> 28) & 8;
+                gval = (sword >> 8) & 0xf;
+                dword |= ((gval - thresh) >> 29) & 4;
+                gval = (sword >> 4) & 0xf;
+                dword |= ((gval - thresh) >> 30) & 2;
+                gval = sword & 0xf;
+                dword |= ((gval - thresh) >> 31) & 1;
+            }
+            lined[dcount++] = dword;
+        }
+
+        if (j < w) {
+          dword = 0;
+          for (; j < w; j++) {
+              if ((j & 7) == 0) {
+                  sword = lines[scount++];
+              }
+              gval = (sword >> 28) & 0xf;
+              sword <<= 4;
+              dword |= (((gval - thresh) >> 31) & 1) << (31 - (j & 31));
+          }
+          lined[dcount] = dword;
+        }
+#if DEBUG_UNROLLING
+#define CHECK_BIT(a, b, c) if (GET_DATA_BIT(a, b) != c) { \
+    fprintf(stderr, "Error: mismatch at %d/%d(%d), %d vs %d\n", \
+            j, w, d, GET_DATA_BIT(a, b), c); }
         for (j = 0; j < w; j++) {
             gval = GET_DATA_QBIT(lines, j);
-            if (gval < thresh) 
-                SET_DATA_BIT(lined, j);
+            CHECK_BIT(lined, j, gval < thresh ? 1 : 0);
         }
+#endif
         break;
     case 8:
+            /* Unrolled as 8 source words, 1 dest word */
+        for (j = 0, scount = 0, dcount = 0; j + 31 < w; j += 32) {
+            dword = 0;
+            for (k = 0; k < 8; k++) {
+                sword = lines[scount++];
+                dword <<= 4;
+                gval = (sword >> 24) & 0xff;
+                dword |= ((gval - thresh) >> 28) & 8;
+                gval = (sword >> 16) & 0xff;
+                dword |= ((gval - thresh) >> 29) & 4;
+                gval = (sword >> 8) & 0xff;
+                dword |= ((gval - thresh) >> 30) & 2;
+                gval = sword & 0xff;
+                dword |= ((gval - thresh) >> 31) & 1;
+            }
+            lined[dcount++] = dword;
+        }
+
+        if (j < w) {
+            dword = 0;
+            for (; j < w; j++) {
+                if ((j & 3) == 0) {
+                    sword = lines[scount++];
+                }
+                gval = (sword >> 24) & 0xff;
+                sword <<= 8;
+                dword |= (((gval - thresh) >> 31) & 1) << (31 - (j & 31));
+            }
+            lined[dcount] = dword;
+        }
+#if DEBUG_UNROLLING
         for (j = 0; j < w; j++) {
             gval = GET_DATA_BYTE(lines, j);
-            if (gval < thresh) 
-                SET_DATA_BIT(lined, j);
+            CHECK_BIT(lined, j, gval < thresh ? 1 : 0);
         }
+#undef CHECK_BIT
+#endif
         break;
     default:
         ERROR_VOID("src depth not 4 or 8 bpp", procName);
@@ -750,10 +832,13 @@ l_int32  *tabval, *tab38, *tab14;
  *  thresholdTo2bppLow()
  *
  *  Low-level function for thresholding from 8 bpp (datas) to
- *  2 bpp (datad).  If tabval is defined, it uses the thresholds
- *  that are implicitly defined by the table, which is a 256-entry
- *  lookup table that gives the 2-bit output value for each 
- *  possible input.
+ *  2 bpp (datad), using thresholds implicitly defined through @tab,
+ *  a 256-entry lookup table that gives a 2-bit output value
+ *  for each possible input.
+ *
+ *  For each line, unroll the loop so that for each 32 bit src word,
+ *  representing four consecutive 8-bit pixels, we compose one byte
+ *  of output consisiting of four 2-bit pixels.
  */
 void
 thresholdTo2bppLow(l_uint32  *datad,
@@ -763,43 +848,23 @@ thresholdTo2bppLow(l_uint32  *datad,
                    l_int32    wpls,
                    l_int32   *tab)
 {
-l_int32    i;
+l_uint8    sval1, sval2, sval3, sval4, dval;
+l_int32    i, j, k;
 l_uint32  *lines, *lined;
 
     for (i = 0; i < h; i++) {
         lines = datas + i * wpls;
         lined = datad + i * wpld;
-        thresholdTo2bppLineLow(lined, lines, wpls, tab);
-    }
-    return;
-}
-
-
-/*
- *  thresholdTo2bppLineLow()
- *
- *  Unroll the loop to the extent that for each 32 bit src word,
- *  representing four consecutive 8-bit pixels, we compose one byte
- *  of output consisiting of four 2-bit pixels.
- */
-void
-thresholdTo2bppLineLow(l_uint32  *lined,
-                       l_uint32  *lines,
-                       l_int32    wpls,
-                       l_int32   *tab)
-{
-l_uint8   sval1, sval2, sval3, sval4, dval;
-l_int32   j, k;
-
-    for (j = 0; j < wpls; j++) {
-        k = 4 * j;
-        sval1 = GET_DATA_BYTE(lines, k);
-        sval2 = GET_DATA_BYTE(lines, k + 1);
-        sval3 = GET_DATA_BYTE(lines, k + 2);
-        sval4 = GET_DATA_BYTE(lines, k + 3);
-        dval = (tab[sval1] << 6) | (tab[sval2] << 4) |
-               (tab[sval3] << 2) | tab[sval4];
-        SET_DATA_BYTE(lined, j, dval);
+        for (j = 0; j < wpls; j++) {
+            k = 4 * j;
+            sval1 = GET_DATA_BYTE(lines, k);
+            sval2 = GET_DATA_BYTE(lines, k + 1);
+            sval3 = GET_DATA_BYTE(lines, k + 2);
+            sval4 = GET_DATA_BYTE(lines, k + 3);
+            dval = (tab[sval1] << 6) | (tab[sval2] << 4) |
+                   (tab[sval3] << 2) | tab[sval4];
+            SET_DATA_BYTE(lined, j, dval);
+        }
     }
     return;
 }
@@ -812,9 +877,13 @@ l_int32   j, k;
  *  thresholdTo4bppLow()
  *
  *  Low-level function for thresholding from 8 bpp (datas) to
- *  4 bpp (datad), using thresholds implicitly defined
- *  through the tabval, a 256-entry lookup table that gives
- *  a 4-bit output value for each possible input.
+ *  4 bpp (datad), using thresholds implicitly defined through @tab,
+ *  a 256-entry lookup table that gives a 4-bit output value
+ *  for each possible input.
+ *  
+ *  For each line, unroll the loop so that for each 32 bit src word,
+ *  representing four consecutive 8-bit pixels, we compose two bytes
+ *  of output consisiting of four 4-bit pixels.
  */
 void
 thresholdTo4bppLow(l_uint32  *datad,
@@ -824,44 +893,24 @@ thresholdTo4bppLow(l_uint32  *datad,
                    l_int32    wpls,
                    l_int32   *tab)
 {
-l_int32    i;
+l_uint8    sval1, sval2, sval3, sval4;
+l_uint16   dval;
+l_int32    i, j, k;
 l_uint32  *lines, *lined;
 
     for (i = 0; i < h; i++) {
         lines = datas + i * wpls;
         lined = datad + i * wpld;
-        thresholdTo4bppLineLow(lined, lines, wpls, tab);
-    }
-    return;
-}
-
-
-/*
- *  thresholdTo4bppLineLow()
- *
- *  Unroll the loop to the extent that for each 32 bit src word,
- *  representing four consecutive 8-bit pixels, we compose two bytes
- *  of output consisiting of four 4-bit pixels.
- */
-void
-thresholdTo4bppLineLow(l_uint32  *lined,
-                       l_uint32  *lines,
-                       l_int32    wpls,
-                       l_int32   *tab)
-{
-l_uint8   sval1, sval2, sval3, sval4;
-l_uint16  dval;
-l_int32   j, k;
-
-    for (j = 0; j < wpls; j++) {
-        k = 4 * j;
-        sval1 = GET_DATA_BYTE(lines, k);
-        sval2 = GET_DATA_BYTE(lines, k + 1);
-        sval3 = GET_DATA_BYTE(lines, k + 2);
-        sval4 = GET_DATA_BYTE(lines, k + 3);
-        dval = (tab[sval1] << 12) | (tab[sval2] << 8) |
-               (tab[sval3] << 4) | tab[sval4];
-        SET_DATA_TWO_BYTES(lined, j, dval);
+        for (j = 0; j < wpls; j++) {
+            k = 4 * j;
+            sval1 = GET_DATA_BYTE(lines, k);
+            sval2 = GET_DATA_BYTE(lines, k + 1);
+            sval3 = GET_DATA_BYTE(lines, k + 2);
+            sval4 = GET_DATA_BYTE(lines, k + 3);
+            dval = (tab[sval1] << 12) | (tab[sval2] << 8) |
+                   (tab[sval3] << 4) | tab[sval4];
+            SET_DATA_TWO_BYTES(lined, j, dval);
+        }
     }
     return;
 }
