@@ -321,9 +321,7 @@ l_uint8                       *comment = NULL;
                     rval = L_MIN(L_MAX(rval, 0), 255);
                     gval = L_MIN(L_MAX(gval, 0), 255);
                     bval = L_MIN(L_MAX(bval, 0), 255);
-                    SET_DATA_BYTE(ppixel, COLOR_RED, rval);
-                    SET_DATA_BYTE(ppixel, COLOR_GREEN, gval);
-                    SET_DATA_BYTE(ppixel, COLOR_BLUE, bval);
+                    composeRGBPixel(rval, gval, bval, ppixel);
                     ppixel++;
                 }
             }
@@ -424,11 +422,18 @@ FILE  *fp;
  *          expense of slower encoding and decoding.
  *      (3) There are three possibilities:
  *          * Grayscale image, no colormap: compress as 8 bpp image.
- *          * 24 bpp full color image: copy each line into the color
+ *          * rgb full color image: copy each line into the color
  *            line buffer, and compress as three 8 bpp images.
  *          * 8 bpp colormapped image: convert each line to three
  *            8 bpp line images in the color line buffer, and
  *            compress as three 8 bpp images.
+ *      (4) The only valid pixel depths in leptonica are 1, 2, 4, 8, 16
+ *          and 32 bpp.  However, it is possible, and in some cases desirable,
+ *          to write out a jpeg file using an rgb pix that has 24 bpp.
+ *          This can be created by appending the raster data for a 24 bpp
+ *          image (with proper scanline padding) directly to a 24 bpp
+ *          pix that was created without a data array.  See note in
+ *          pixWriteStreamPng() for an example.
  */
 l_int32
 pixWriteStreamJpeg(FILE    *fp,
@@ -473,14 +478,14 @@ const char                  *text;
     w = pixGetWidth(pix);
     h = pixGetHeight(pix);
     d = pixGetDepth(pix);
-    if (d != 8 && d != 32)
-        return ERROR_INT("bpp must be 8 or 32", procName, 1);
+    if (d != 8 && d != 24 && d != 32)
+        return ERROR_INT("bpp must be 8, 24 or 32", procName, 1);
 
     if (quality <= 0)
         quality = 75;  /* default */
 
-    if (d == 32)
-        colorflg = 2;    /* 24 bpp rgb; no colormap */
+    if (d == 32 || d == 24)
+        colorflg = 2;    /* rgb; no colormap */
     else if ((cmap = pixGetColormap(pix)) == NULL)
         colorflg = 0;    /* 8 bpp grayscale; no colormap */
     else {
@@ -553,16 +558,22 @@ const char                  *text;
                 rowbuffer[3 * j + COLOR_BLUE] = bmap[byteval];
             }
         }
-        else { /* 24 bpp color */
-            ppixel = line;
-            for (j = k = 0; j < w; j++) {
-                rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_RED);
-                rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_GREEN);
-                rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_BLUE);
-                ppixel++;
+        else {  /* colorflg == 2 */
+            if (d == 24) {  /* See note 4 above; special case of 24 bpp rgb */
+                jpeg_write_scanlines(&cinfo, (JSAMPROW *)&line, 1);
+            }
+            else {  /* standard 32 bpp rgb */
+                ppixel = line;
+                for (j = k = 0; j < w; j++) {
+                    rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_RED);
+                    rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_GREEN);
+                    rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_BLUE);
+                    ppixel++;
+                }
             }
         }
-        jpeg_write_scanlines(&cinfo, &rowbuffer, 1);
+        if (d != 24)
+            jpeg_write_scanlines(&cinfo, &rowbuffer, 1);
     }
 
     jpeg_finish_compress(&cinfo);
@@ -864,8 +875,7 @@ getTwoByteParameter(l_uint8  *array,
 /*---------------------------------------------------------------------*
  *                         Read/write to memory                        *
  *---------------------------------------------------------------------*/
-#if HAVE_FMEMOPEN || \
- (!defined(__MINGW32__) && !defined(_CYGWIN_ENVIRON) && !defined(_STANDARD_C_))
+#if HAVE_FMEMOPEN
 
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
@@ -880,7 +890,7 @@ extern FILE *fmemopen(void *data, size_t size, const char *mode);
  *                               palette image if color)
  *              reduction (scaling factor: 1, 2, 4 or 8)
  *              &pnwarn (<optional return> number of warnings)
- *              hint: a bitwise OR of L_HINT_* values
+ *              hint (bitwise OR of L_HINT_* values; use 0 for no hint)
  *      Return: pix, or null on error
  *
  *  Notes:
@@ -981,7 +991,7 @@ pixWriteMemJpeg(l_uint8  **pdata,
         "pixWriteMemJpeg", 1);
 }
 
-#endif  /* HAVE_FMEMOPEN || (!defined(__MINGW32__) && etc ) */
+#endif  /* HAVE_FMEMOPEN */
 
 
 /*---------------------------------------------------------------------*
