@@ -20,9 +20,10 @@
  *
  *      Basic operations on kernels for image convolution
  *
- *         Create/destroy
+ *         Create/destroy/copy
  *            L_KERNEL   *kernelCreate()
  *            void        kernelDestroy()
+ *            L_KERNEL   *kernelCopy()
  *
  *         Accessors:
  *            l_int32     kernelGetElement()
@@ -32,6 +33,10 @@
  *            l_int32     kernelGetSum()
  *            l_int32     kernelGetMinMax()
  *
+ *         Normalize/invert
+ *            L_KERNEL   *kernelNormalize()
+ *            L_KERNEL   *kernelInvert()
+ *
  *         Helper function
  *            l_float32 **create2dFloatArray()
  *
@@ -58,6 +63,7 @@
  *
  *      Simple parametric kernels
  *            L_KERNEL   *makeGaussianKernel()
+ *            L_KERNEL   *makeGaussianKernelSep()
  *            L_KERNEL   *makeDoGKernel()
  */
 
@@ -132,6 +138,36 @@ L_KERNEL  *kel;
 }
 
 
+/*!
+ *  kernelCopy()
+ *
+ *      Input:  kels (source kernel)
+ *      Return: keld (copy of kels), or null on error
+ */
+L_KERNEL *
+kernelCopy(L_KERNEL  *kels)
+{
+l_int32    i, j, sx, sy, cx, cy;
+L_KERNEL  *keld;
+
+    PROCNAME("kernelCopy");
+
+    if (!kels)
+        return (L_KERNEL *)ERROR_PTR("kels not defined", procName, NULL);
+
+    kernelGetParameters(kels, &sy, &sx, &cy, &cx);
+    if ((keld = kernelCreate(sy, sx)) == NULL)
+        return (L_KERNEL *)ERROR_PTR("keld not made", procName, NULL);
+    keld->cy = cy;
+    keld->cx = cx;
+    for (i = 0; i < sy; i++)
+        for (j = 0; j < sx; j++)
+            keld->data[i][j] = kels->data[i][j];
+
+    return keld;
+}
+
+
 /*----------------------------------------------------------------------*
  *                               Accessors                              *
  *----------------------------------------------------------------------*/
@@ -152,6 +188,9 @@ kernelGetElement(L_KERNEL   *kel,
 {
     PROCNAME("kernelGetElement");
 
+    if (!pval)
+        return ERROR_INT("&val not defined", procName, 1);
+    *pval = 0;
     if (!kel)
         return ERROR_INT("kernel not defined", procName, 1);
     if (row < 0 || row >= kel->sy)
@@ -209,6 +248,10 @@ kernelGetParameters(L_KERNEL  *kel,
 {
     PROCNAME("kernelGetParameters");
 
+    if (psy) *psy = 0;
+    if (psx) *psx = 0;
+    if (pcy) *pcy = 0;
+    if (pcx) *pcx = 0;
     if (!kel)
         return ERROR_INT("kernel not defined", procName, 1);
     if (psy) *psy = kel->sy; 
@@ -258,11 +301,11 @@ l_int32    sx, sy, i, j;
 
     if (!psum)
         return ERROR_INT("&sum not defined", procName, 1);
+    *psum = 0.0;
     if (!kel)
         return ERROR_INT("kernel not defined", procName, 1);
 
     kernelGetParameters(kel, &sy, &sx, NULL, NULL);
-    *psum = 0.0;
     for (i = 0; i < sy; i++) {
         for (j = 0; j < sx; j++) {
             *psum += kel->data[i][j];
@@ -290,6 +333,8 @@ l_float32  val, minval, maxval;
 
     PROCNAME("kernelGetMinmax");
 
+    if (!pmin && !pmax)
+        return ERROR_INT("neither &min nor &max defined", procName, 1);
     if (pmin) *pmin = 0.0;
     if (pmax) *pmax = 0.0;
     if (!kel)
@@ -316,6 +361,91 @@ l_float32  val, minval, maxval;
 }
 
 
+/*----------------------------------------------------------------------*
+ *                          Normalize/Invert                            *
+ *----------------------------------------------------------------------*/
+/*!
+ *  kernelNormalize()
+ *
+ *      Input:  kels (source kel, to be normalized)
+ *              normsum (desired sum of elements in keld)
+ *      Return: keld (normalized version of kels), or null on error
+ *                   or if sum of elements is very close to 0)
+ *
+ *  Notes:
+ *      (1) If the sum of kernel elements is close to 0, do not
+ *          try to calculate the normalized kernel.  Instead,
+ *          return a copy of the input kernel, with an error message.
+ */
+L_KERNEL *
+kernelNormalize(L_KERNEL  *kels,
+                l_float32  normsum)
+{
+l_int32    i, j, sx, sy, cx, cy;
+l_float32  sum, factor;
+L_KERNEL  *keld;
+
+    PROCNAME("kernelNormalize");
+
+    if (!kels)
+        return (L_KERNEL *)ERROR_PTR("kels not defined", procName, NULL);
+
+    kernelGetSum(kels, &sum);
+    if (L_ABS(sum) < 0.01) {
+        L_ERROR("null sum; not normalizing; returning a copy", procName);
+        return kernelCopy(kels);
+    }
+
+    kernelGetParameters(kels, &sy, &sx, &cy, &cx);
+    if ((keld = kernelCreate(sy, sx)) == NULL)
+        return (L_KERNEL *)ERROR_PTR("keld not made", procName, NULL);
+    keld->cy = cy;
+    keld->cx = cx;
+
+    factor = normsum / sum;
+    for (i = 0; i < sy; i++)
+        for (j = 0; j < sx; j++)
+            keld->data[i][j] = factor * kels->data[i][j];
+
+    return keld;
+}
+
+
+/*!
+ *  kernelInvert()
+ *
+ *      Input:  kels (source kel, to be inverted)
+ *      Return: keld (spatially inverted, about the origin), or null on error
+ *
+ *  Notes:
+ *      (1) For convolution, the kernel is spatially inverted before
+ *          a "correlation" operation is done between the kernel and the image.
+ */
+L_KERNEL *
+kernelInvert(L_KERNEL  *kels)
+{
+l_int32    i, j, sx, sy, cx, cy;
+L_KERNEL  *keld;
+
+    PROCNAME("kernelInvert");
+
+    if (!kels)
+        return (L_KERNEL *)ERROR_PTR("kels not defined", procName, NULL);
+
+    kernelGetParameters(kels, &sy, &sx, &cy, &cx);
+    if ((keld = kernelCreate(sy, sx)) == NULL)
+        return (L_KERNEL *)ERROR_PTR("keld not made", procName, NULL);
+    keld->cy = sy - 1 - cy;
+    keld->cx = sx - 1 - cx;
+
+    for (i = 0; i < sy; i++)
+        for (j = 0; j < sx; j++)
+            keld->data[i][j] = kels->data[sy - 1 - i][sx - 1 - j];
+
+    return keld;
+}
+
+
 /*----------------------------------------------------------------------*
  *                            Helper function                           *
  *----------------------------------------------------------------------*/
@@ -899,6 +1029,47 @@ L_KERNEL  *kel;
 }
 
 
+/*!
+ *  makeGaussianKernelSep()
+ *
+ *      Input:  halfheight, halfwidth (sx = 2 * halfwidth + 1, etc)
+ *              stdev (standard deviation)
+ *              max (value at (cx,cy))
+ *              &kelx (<return> x part of kernel)
+ *              &kely (<return> y part of kernel)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) See makeGaussianKernel() for description of input parameters.
+ *      (2) These kernels are constructed so that the result of both
+ *          normalized and un-normalized convolution will be the same
+ *          as when convolving with pixConvolve() using the full kernel.
+ *      (3) The trick for the un-normalized convolution is to have the
+ *          product of the two kernel elemets at (cx,cy) be equal to max,
+ *          not max**2.  That's why the max for kely is 1.0.  If instead
+ *          we use sqrt(max) for both, the results are slightly less
+ *          accurate, when compared to using the full kernel in
+ *          makeGaussianKernel().
+ */
+l_int32
+makeGaussianKernelSep(l_int32    halfheight,
+                      l_int32    halfwidth,
+                      l_float32  stdev,
+                      l_float32  max,
+                      L_KERNEL **pkelx,
+                      L_KERNEL **pkely)
+{
+    PROCNAME("makeGaussianKernelSep");
+
+    if (!pkelx || !pkely)
+        return ERROR_INT("&kelx and &kely not defined", procName, 1);
+
+    *pkelx = makeGaussianKernel(0, halfwidth, stdev, max);
+    *pkely = makeGaussianKernel(halfheight, 0, stdev, 1.0);
+    return 0;
+}
+
+
 /*!
  *  makeDoGKernel()
  *
