@@ -29,14 +29,14 @@
  *          PIX             *generateBinaryMaze()
  *          static MAZEEL   *mazeelCreate()
  *
- *          PIX             *searchBinaryMaze()
+ *          PIX             *pixSearchBinaryMaze()
  *          static l_int32   localSearchForBackground()
  *
  *      Generalizing a maze to a grayscale image, the search is
  *      now for the "shortest" or least cost path, for some given
  *      cost function.
  *
- *          PIX             *searchGrayMaze()
+ *          PIX             *pixSearchGrayMaze()
  *
  *      Display functions
  *          PIX             *pixDisplayPta()
@@ -139,7 +139,7 @@ l_float32  frand, wallpf, testp;
 MAZEEL    *el, *elp;
 PIX       *pixd;  /* the destination maze */
 PIX       *pixm;  /* for bookkeeping, to indicate pixels already visited */
-PQUEUE    *pq;
+L_QUEUE   *lq;
 
     if (w < MIN_MAZE_WIDTH)
         w = MIN_MAZE_WIDTH;
@@ -165,16 +165,16 @@ PQUEUE    *pq;
     pixd = pixCreate(w, h, 1);
     pixm = pixCreate(w, h, 1);
 
-    pq = pqueueCreate(0);
+    lq = lqueueCreate(0);
 
         /* prime the queue with the first pixel; it is OFF */
     el = mazeelCreate(xi, yi, START_LOC);
     pixSetPixel(pixm, xi, yi, 1);  /* mark visited */
-    pqueueAdd(pq, el);
+    lqueueAdd(lq, el);
 
         /* while we're at it ... */
-    while (pqueueGetCount(pq) > 0) {
-        elp = (MAZEEL *)pqueueRemove(pq);
+    while (lqueueGetCount(lq) > 0) {
+        elp = (MAZEEL *)lqueueRemove(lq);
         x = elp->x;
         y = elp->y;
         dir = elp->dir;
@@ -191,7 +191,7 @@ PQUEUE    *pq;
                 }
                 else {  /* not a wall */
                     el = mazeelCreate(x - 1, y, DIR_WEST);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
@@ -208,7 +208,7 @@ PQUEUE    *pq;
                 }
                 else {  /* not a wall */
                     el = mazeelCreate(x, y - 1, DIR_NORTH);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
@@ -225,7 +225,7 @@ PQUEUE    *pq;
                 }
                 else {  /* not a wall */
                     el = mazeelCreate(x + 1, y, DIR_EAST);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
@@ -242,14 +242,14 @@ PQUEUE    *pq;
                 }
                 else {  /* not a wall */
                     el = mazeelCreate(x, y + 1, DIR_SOUTH);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
         FREE(elp);
     }
 
-    pqueueDestroy(&pq, TRUE);
+    lqueueDestroy(&lq, TRUE);
     pixDestroy(&pixm);
     return pixd;
 }
@@ -274,7 +274,7 @@ MAZEEL *el;
  *                           Binary maze search                        *
  *---------------------------------------------------------------------*/
 /*!
- *  searchBinaryMaze()
+ *  pixSearchBinaryMaze()
  *
  *      Input:  pixs (1 bpp, maze)
  *              xi, yi  (beginning point; use same initial point
@@ -321,31 +321,31 @@ MAZEEL *el;
  *
  */
 PTA *
-searchBinaryMaze(PIX     *pixs,
-                 l_int32  xi,
-                 l_int32  yi, 
-                 l_int32  xf,
-                 l_int32  yf,
-                 PIX    **ppixd)
+pixSearchBinaryMaze(PIX     *pixs,
+                    l_int32  xi,
+                    l_int32  yi, 
+                    l_int32  xf,
+                    l_int32  yf,
+                    PIX    **ppixd)
 {
-l_int32    i, j, x, y, w, h, found;
+l_int32    i, j, x, y, w, h, d, found;
 l_uint32   val, rpixel, gpixel, bpixel;
+void     **lines1, **linem1, **linep8, **lined32;
 MAZEEL    *el, *elp;
 PIX       *pixd;  /* the shortest path written on the maze image */
 PIX       *pixm;  /* for bookkeeping, to indicate pixels already visited */
 PIX       *pixp;  /* for bookkeeping, to indicate direction to parent */
-PQUEUE    *pq;
+L_QUEUE   *lq;
 PTA       *pta;
 
-    PROCNAME("searchBinaryMaze");
+    PROCNAME("pixSearchBinaryMaze");
 
     if (ppixd) *ppixd = NULL;
     if (!pixs)
         return (PTA *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (pixGetDepth(pixs) != 1)
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 1)
         return (PTA *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
     if (xi <= 0 || xi >= w)
         return (PTA *)ERROR_PTR("xi not valid", procName, NULL);
     if (yi <= 0 || yi >= h)
@@ -366,19 +366,23 @@ PTA       *pta;
 
     pixm = pixCreate(w, h, 1);  /* initialized to OFF */
     pixp = pixCreate(w, h, 8);  /* direction to parent stored as enum val */
+    lines1 = pixGetLinePtrs(pixs, NULL);
+    linem1 = pixGetLinePtrs(pixm, NULL);
+    linep8 = pixGetLinePtrs(pixp, NULL);
 
-    pq = pqueueCreate(0);
+
+    lq = lqueueCreate(0);
 
         /* prime the queue with the first pixel; it is OFF */
     el = mazeelCreate(xi, yi, 0);  /* don't need direction here */
     pixSetPixel(pixm, xi, yi, 1);  /* mark visited */
-    pqueueAdd(pq, el);
+    lqueueAdd(lq, el);
 
         /* fill up the pix storing directions to parents,
          * stopping when we hit the point (xf, yf)  */
     found = FALSE;
-    while (pqueueGetCount(pq) > 0) {
-        elp = (MAZEEL *)pqueueRemove(pq);
+    while (lqueueGetCount(lq) > 0) {
+        elp = (MAZEEL *)lqueueRemove(lq);
         x = elp->x;
         y = elp->y;
         if (x == xf && y == yf) {
@@ -388,58 +392,59 @@ PTA       *pta;
         }
             
         if (x > 0) {  /* check to west */
-            pixGetPixel(pixm, x - 1, y, &val);
+            val = GET_DATA_BIT(linem1[y], x - 1);
             if (val == 0) {  /* not yet visited */
-                pixSetPixel(pixm, x - 1, y, 1);  /* mark visited */
-                pixGetPixel(pixs, x - 1, y, &val);
+                SET_DATA_BIT(linem1[y], x - 1);  /* mark visited */
+                val = GET_DATA_BIT(lines1[y], x - 1);
                 if (val == 0) {  /* bg, not a wall */
-                    pixSetPixel(pixp, x - 1, y, DIR_EAST);  /* parent to E */
+                    SET_DATA_BYTE(linep8[y], x - 1, DIR_EAST);  /* parent E */
                     el = mazeelCreate(x - 1, y, 0);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
         if (y > 0) {  /* check north */
-            pixGetPixel(pixm, x, y - 1, &val);
+            val = GET_DATA_BIT(linem1[y - 1], x);
             if (val == 0) {  /* not yet visited */
-                pixSetPixel(pixm, x, y - 1, 1);  /* mark visited */
-                pixGetPixel(pixs, x, y - 1, &val);
+                SET_DATA_BIT(linem1[y - 1], x);  /* mark visited */
+                val = GET_DATA_BIT(lines1[y - 1], x);
                 if (val == 0) {  /* bg, not a wall */
-                    pixSetPixel(pixp, x, y - 1, DIR_SOUTH);  /* parent to S */
+                    SET_DATA_BYTE(linep8[y - 1], x, DIR_SOUTH);  /* parent S */
                     el = mazeelCreate(x, y - 1, 0);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
         if (x < w - 1) {  /* check east */
-            pixGetPixel(pixm, x + 1, y, &val);
+            val = GET_DATA_BIT(linem1[y], x + 1);
             if (val == 0) {  /* not yet visited */
-                pixSetPixel(pixm, x + 1, y, 1);  /* mark visited */
-                pixGetPixel(pixs, x + 1, y, &val);
+                SET_DATA_BIT(linem1[y], x + 1);  /* mark visited */
+                val = GET_DATA_BIT(lines1[y], x + 1);
                 if (val == 0) {  /* bg, not a wall */
-                    pixSetPixel(pixp, x + 1, y, DIR_WEST);  /* parent to W */
+                    SET_DATA_BYTE(linep8[y], x + 1, DIR_WEST);  /* parent W */
                     el = mazeelCreate(x + 1, y, 0);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
         if (y < h - 1) {  /* check south */
-            pixGetPixel(pixm, x, y + 1, &val);
+            val = GET_DATA_BIT(linem1[y + 1], x);
             if (val == 0) {  /* not yet visited */
-                pixSetPixel(pixm, x, y + 1, 1);  /* mark visited */
-                pixGetPixel(pixs, x, y + 1, &val);
+                SET_DATA_BIT(linem1[y + 1], x);  /* mark visited */
+                val = GET_DATA_BIT(lines1[y + 1], x);
                 if (val == 0) {  /* bg, not a wall */
-                    pixSetPixel(pixp, x, y + 1, DIR_NORTH);  /* parent to N */
+                    SET_DATA_BYTE(linep8[y + 1], x, DIR_NORTH);  /* parent N */
                     el = mazeelCreate(x, y + 1, 0);
-                    pqueueAdd(pq, el);
+                    lqueueAdd(lq, el);
                 }
             }
         }
         FREE(elp);
     }
 
-    pqueueDestroy(&pq, TRUE);
+    lqueueDestroy(&lq, TRUE);
     pixDestroy(&pixm);
+    FREE(linem1);
 
     if (ppixd) {
         pixd = pixUnpackBinary(pixs, 32, 1);
@@ -449,16 +454,19 @@ PTA       *pta;
     composeRGBPixel(0, 255, 0, &gpixel);
     composeRGBPixel(0, 0, 255, &bpixel);  /* end point */
 
+
     if (!found) {
         L_INFO(" No path found", procName);
         if (pixd) {  /* paint all visited locations */
+            lined32 = pixGetLinePtrs(pixd, NULL);
             for (i = 0; i < h; i++) {
                 for (j = 0; j < w; j++) {
-                    pixGetPixel(pixp, j, i, &val);
+                    val = GET_DATA_BYTE(linep8[i], j);
                     if (val != 0 && pixd)
-                        pixSetPixel(pixd, j, i, gpixel);
+                        SET_DATA_FOUR_BYTES(lined32[i], j, gpixel);
                 }
             }
+            FREE(lined32);
         }
     }
     else {   /* write path onto pixd */
@@ -489,6 +497,8 @@ PTA       *pta;
     }
 
     pixDestroy(&pixp);
+    FREE(lines1);
+    FREE(linep8);
     return pta;
 }
 
@@ -517,8 +527,7 @@ l_uint32  val;
         /* For each value of r, restrict the search to the boundary
          * pixels in a square centered on (x,y), clipping to the
          * image boundaries if necessary.  */
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    pixGetDimensions(pix, &w, &h, NULL);
     for (r = 1; r < maxrad; r++) {
         for (i = -r; i <= r; i++) {
             if (y + i < 0 || y + i >= h)
@@ -546,7 +555,7 @@ l_uint32  val;
  *                            Gray maze search                         *
  *---------------------------------------------------------------------*/
 /*!
- *  searchGrayMaze()
+ *  pixSearchGrayMaze()
  *
  *      Input:  pixs (1 bpp, maze)
  *              xi, yi  (beginning point; use same initial point
@@ -558,11 +567,6 @@ l_uint32  val;
  *      Return: pta (shortest path), or null if either no path
  *              exists or on error
  *
- *  Note: Because here is a considerable amount of overhead in calling
- *        pixGetPixel() and pixSetPixel(), this function can be sped
- *        up with little effort using raster line pointers and the
- *        GET_DATA* and SET_DATA* macros.
- *
  *  Commentary:
  *      Consider first a slight generalization of the binary maze
  *      search problem.  Suppose that you can go through walls,
@@ -596,7 +600,7 @@ l_uint32  val;
  *      This continues until the destination pixel is popped off
  *      a stack.   The minimum path is then derived from the distance map,
  *      going back from the end point as before.  This is just Dijkstra's
- *      algorithm for a directed graph; here, the underlyaing graph
+ *      algorithm for a directed graph; here, the underlying graph
  *      (consisting of the pixels and four edges connecting each pixel
  *      to its 4-neighbor) is a special case of a directed graph, where
  *      each edge is bi-directional.  The implementation of this generalized
@@ -621,7 +625,7 @@ l_uint32  val;
  *      Rather than using an array of stacks, a more practical
  *      approach is to implement with a priority queue, which is
  *      a queue that is sorted so that the elements with the largest
- *      (or smallest) key values always comes off first.  The
+ *      (or smallest) key values always come off first.  The
  *      priority queue is efficiently implemented as a heap, and
  *      this is how we do it.  Suppose you run the algorithm
  *      using a priority queue, doing the bookkeeping with an
@@ -638,7 +642,7 @@ l_uint32  val;
  *
  *      Do we really have to use a sorted queue?  Can we solve this
  *      generalized maze with an unsorted queue of pixels?  (Or even
- *      an unsorted stack, doing a depth-first search?)
+ *      an unsorted stack, doing a depth-first search (DFS)?)
  *      Consider a different algorithm for this generalized maze, where
  *      we travel again breadth first, but this time use a single,
  *      unsorted queue.  An auxiliary image is used as before to
@@ -674,10 +678,21 @@ l_uint32  val;
  *      Because the shortest path can theoretically go anywhere,
  *      we must keep going.  How do we know when to stop?   Dijkstra
  *      uses an ordered queue to systematically remove nodes from
- *      further consideration.  With an unordered queue, the brute
- *      force answer is: stop when the queue (or stack) is empty,
- *      because then every pixel in the image has been assigned its
- *      minimum "distance" from the start pixel.
+ *      further consideration.  (Each time a pixel is popped, we're
+ *      done with it; it's "finalized" in the Dijkstra sense because
+ *      we know the shortest path to it.)  However, with an unordered
+ *      queue, the brute force answer is: stop when the queue
+ *      (or stack) is empty, because then every pixel in the image
+ *      has been assigned its minimum "distance" from the start pixel.
+ *
+ *      This is similar to the situation when you use a stack for the
+ *      simpler uniform-step problem: with breadth-first search (BFS)
+ *      the pixels on the queue are automatically ordered, so you are
+ *      done when you locate the end pixel as a neighbor of a popped pixel;
+ *      whereas depth-first search (DFS), using a stack, requires,
+ *      in general, a search of every accessible pixel.  Further, if
+ *      a pixel is revisited with a smaller distance, that distance is
+ *      recorded and the pixel is put on the stack again.
  *
  *      But surely, you ask, can't we stop sooner?  What if the
  *      start and end pixels are very close to each other?
@@ -688,38 +703,38 @@ l_uint32  val;
  *      could be just over a wall from the start.  With the unordered
  *      queue, you very quickly get a high distance for the end
  *      pixel, which will be relaxed to the minimum distance only
- *      after all the pixels of the path have been visited, many
- *      of them multiple times.  So that's the price for not ordering
- *      the queue!
+ *      after all the pixels of the path have been visited and placed
+ *      on the queue, multiple times for many of them.  So that's the
+ *      price for not ordering the queue!
  */
 PTA *
-searchGrayMaze(PIX     *pixs,
-               l_int32  xi,
-               l_int32  yi, 
-               l_int32  xf,
-               l_int32  yf,
-               PIX    **ppixd)
+pixSearchGrayMaze(PIX     *pixs,
+                  l_int32  xi,
+                  l_int32  yi, 
+                  l_int32  xf,
+                  l_int32  yf,
+                  PIX    **ppixd)
 {
-l_int32   x, y, w, h;
+l_int32   x, y, w, h, d;
 l_uint32  val, valr, vals, rpixel, gpixel, bpixel;
+void    **lines8, **liner32, **linep8;
 l_int32   cost, dist, distparent, sival, sivals;
 MAZEEL   *el, *elp;
 PIX      *pixd;  /* optionally plot the path on this RGB version of pixs */
 PIX      *pixr;  /* for bookkeeping, to indicate the minimum distance */
                  /* to pixels already visited */
 PIX      *pixp;  /* for bookkeeping, to indicate direction to parent */
-PHEAP    *ph;
+L_HEAP   *lh;
 PTA      *pta;
 
-    PROCNAME("searchGrayMaze");
+    PROCNAME("pixSearchGrayMaze");
 
     if (ppixd) *ppixd = NULL;
     if (!pixs)
         return (PTA *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (pixGetDepth(pixs) != 8)
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 8)
         return (PTA *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
     if (xi <= 0 || xi >= w)
         return (PTA *)ERROR_PTR("xi not valid", procName, NULL);
     if (yi <= 0 || yi >= h)
@@ -730,8 +745,11 @@ PTA      *pta;
     pixr = pixCreate(w, h, 32);
     pixSetAll(pixr);  /* initialize to max value */
     pixp = pixCreate(w, h, 8);  /* direction to parent stored as enum val */
+    lines8 = pixGetLinePtrs(pixs, NULL);
+    linep8 = pixGetLinePtrs(pixp, NULL);
+    liner32 = pixGetLinePtrs(pixr, NULL);
 
-    ph = pheapCreate(0, L_SORT_INCREASING);  /* always remove closest pixels */
+    lh = lheapCreate(0, L_SORT_INCREASING);  /* always remove closest pixels */
 
         /* prime the heap with the first pixel */
     pixGetPixel(pixs, xi, yi, &val);
@@ -740,14 +758,14 @@ PTA      *pta;
     pixGetPixel(pixs, xi, yi, &val);
     el->val = val;
     pixSetPixel(pixr, xi, yi, 0);  /* distance is 0 */
-    pheapAdd(ph, el);
+    lheapAdd(lh, el);
 
         /* breadth-first search with priority queue (implemented by
            a heap), labeling direction to parents in pixp and minimum
            distance to visited pixels in pixr.  Stop when we pull
            the destination point (xf, yf) off the queue. */
-    while (pheapGetCount(ph) > 0) {
-        elp = (MAZEEL *)pheapRemove(ph);
+    while (lheapGetCount(lh) > 0) {
+        elp = (MAZEEL *)lheapRemove(lh);
         if (!elp)
             return (PTA *)ERROR_PTR("heap broken!!", procName, NULL);
         x = elp->x;
@@ -761,69 +779,69 @@ PTA      *pta;
         sival = val;
             
         if (x > 0) {  /* check to west */
-            pixGetPixel(pixs, x - 1, y, &vals);
-            pixGetPixel(pixr, x - 1, y, &valr);
+            vals = GET_DATA_BYTE(lines8[y], x - 1);
+            valr = GET_DATA_FOUR_BYTES(liner32[y], x - 1);
             sivals = (l_int32)vals;
             cost = 1 + L_ABS(sivals - sival);  /* cost to move to this pixel */
             dist = distparent + cost;
             if (dist < valr) {  /* shortest path so far to this pixel */
-                pixSetPixel(pixr, x - 1, y, dist);  /* new distance */
-                pixSetPixel(pixp, x - 1, y, DIR_EAST);  /* parent to E */
+                SET_DATA_FOUR_BYTES(liner32[y], x - 1, dist);  /* new dist */
+                SET_DATA_BYTE(linep8[y], x - 1, DIR_EAST);  /* parent to E */
                 el = mazeelCreate(x - 1, y, 0);
                 el->val = vals;
                 el->distance = dist;
-                pheapAdd(ph, el);
+                lheapAdd(lh, el);
             }
         }
         if (y > 0) {  /* check north */
-            pixGetPixel(pixs, x, y - 1, &vals);
-            pixGetPixel(pixr, x, y - 1, &valr);
+            vals = GET_DATA_BYTE(lines8[y - 1], x);
+            valr = GET_DATA_FOUR_BYTES(liner32[y - 1], x);
             sivals = (l_int32)vals;
             cost = 1 + L_ABS(sivals - sival);  /* cost to move to this pixel */
             dist = distparent + cost;
             if (dist < valr) {  /* shortest path so far to this pixel */
-                pixSetPixel(pixr, x, y - 1, dist);  /* new distance */
-                pixSetPixel(pixp, x, y - 1, DIR_SOUTH);  /* parent to S */
+                SET_DATA_FOUR_BYTES(liner32[y - 1], x, dist);  /* new dist */
+                SET_DATA_BYTE(linep8[y - 1], x, DIR_SOUTH);  /* parent to S */
                 el = mazeelCreate(x, y - 1, 0);
                 el->val = vals;
                 el->distance = dist;
-                pheapAdd(ph, el);
+                lheapAdd(lh, el);
             }
         }
         if (x < w - 1) {  /* check east */
-            pixGetPixel(pixs, x + 1, y, &vals);
-            pixGetPixel(pixr, x + 1, y, &valr);
+            vals = GET_DATA_BYTE(lines8[y], x + 1);
+            valr = GET_DATA_FOUR_BYTES(liner32[y], x + 1);
             sivals = (l_int32)vals;
             cost = 1 + L_ABS(sivals - sival);  /* cost to move to this pixel */
             dist = distparent + cost;
             if (dist < valr) {  /* shortest path so far to this pixel */
-                pixSetPixel(pixr, x + 1, y, dist);  /* new distance */
-                pixSetPixel(pixp, x + 1, y, DIR_WEST);  /* parent to W */
+                SET_DATA_FOUR_BYTES(liner32[y], x + 1, dist);  /* new dist */
+                SET_DATA_BYTE(linep8[y], x + 1, DIR_WEST);  /* parent to W */
                 el = mazeelCreate(x + 1, y, 0);
                 el->val = vals;
                 el->distance = dist;
-                pheapAdd(ph, el);
+                lheapAdd(lh, el);
             }
         }
         if (y < h - 1) {  /* check south */
-            pixGetPixel(pixs, x, y + 1, &vals);
-            pixGetPixel(pixr, x, y + 1, &valr);
+            vals = GET_DATA_BYTE(lines8[y + 1], x);
+            valr = GET_DATA_FOUR_BYTES(liner32[y + 1], x);
             sivals = (l_int32)vals;
             cost = 1 + L_ABS(sivals - sival);  /* cost to move to this pixel */
             dist = distparent + cost;
             if (dist < valr) {  /* shortest path so far to this pixel */
-                pixSetPixel(pixr, x, y + 1, dist);  /* new distance */
-                pixSetPixel(pixp, x, y + 1, DIR_NORTH);  /* parent to N */
+                SET_DATA_FOUR_BYTES(liner32[y + 1], x, dist);  /* new dist */
+                SET_DATA_BYTE(linep8[y + 1], x, DIR_NORTH);  /* parent to N */
                 el = mazeelCreate(x, y + 1, 0);
                 el->val = vals;
                 el->distance = dist;
-                pheapAdd(ph, el);
+                lheapAdd(lh, el);
             }
         }
         FREE(elp);
     }
 
-    pheapDestroy(&ph, TRUE);
+    lheapDestroy(&lh, TRUE);
 
     if (ppixd) {
         pixd = pixConvert8To32(pixs);
@@ -836,7 +854,7 @@ PTA      *pta;
     x = xf;
     y = yf;
     pta = ptaCreate(0);
-    while (1) {
+    while (1) {  /* write path onto pixd */
         ptaAddPt(pta, x, y);
         if (x == xi && y == yi)
             break;
@@ -865,6 +883,9 @@ PTA      *pta;
 
     pixDestroy(&pixp);
     pixDestroy(&pixr);
+    FREE(lines8);
+    FREE(linep8);
+    FREE(liner32);
     return pta;
 }
 
