@@ -34,8 +34,8 @@
  *         PIX     *pixCloseSafeBrick()
  *
  *     Binary composed morphological (raster) ops with brick Sels
- *         l_int32  selectComposableSizes()
  *         l_int32  selectComposableSels()
+ *         l_int32  selectComposableSizes()
  *         PIX     *pixDilateCompBrick()
  *         PIX     *pixErodeCompBrick()
  *         PIX     *pixOpenCompBrick()
@@ -76,6 +76,8 @@
  *      You can get the result as a new Pix, in-place back into the src Pix,
  *      or written to another existing Pix.  For large Sels, these are
  *      considerably faster than the corresponding pix*Brick() functions.
+ *      N.B.:  The size of the Sels that are actually used are typically
+ *      close to, but not exactly equal to, the size input to the function.
  *
  *  (3) Brick Sels: pix*BrickDwa(), where * = {Dilate, Erode, Open, Close}.
  *      These are separable dwa (destination word accumulation)
@@ -143,8 +145,8 @@
  *  of a hit-miss Sel), followed by the HMT.
  *  Both of these 'generalized' functions are idempotent.
  *
- *  These functions are extensively tested in prog/morphtest3.c and
- *  prog/morphtest4.c
+ *  These functions are extensively tested in prog/binmorph1_reg.c,
+ *  prog/binmorph2_reg.c, and prog/binmorph3_reg.c.
  */
 
 #include <stdio.h>
@@ -171,17 +173,23 @@ static PIX * processMorphArgs2(PIX *pixd, PIX *pixs, SEL *sel);
 /*!
  *  pixDilate()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
  *  Notes:
  *      (1) This dilates src using hits in Sel.
- *      (2) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (2) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (3) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixDilate(NULL, pixs, ...);
+ *          (b) pixDilate(pixs, pixs, ...);
+ *          (c) pixDilate(pixd, pixs, ...);
+ *      (4) The size of the result is determined by pixs.
  */
 PIX *
 pixDilate(PIX  *pixd,
@@ -217,17 +225,23 @@ PIX     *pixt;
 /*!
  *  pixErode()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
  *  Notes:
  *      (1) This erodes src using hits in Sel.
- *      (2) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (2) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (3) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixErode(NULL, pixs, ...);
+ *          (b) pixErode(pixs, pixs, ...);
+ *          (c) pixErode(pixd, pixs, ...);
+ *      (4) The size of the result is determined by pixs.
  */
 PIX *
 pixErode(PIX  *pixd,
@@ -282,8 +296,9 @@ PIX     *pixt;
 /*!
  *  pixHMT()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd (<optional>; this can be null, equal to pixs,
+ *                    or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
@@ -291,10 +306,15 @@ PIX     *pixt;
  *      (1) The hit-miss transform erodes the src, using both hits
  *          and misses in the Sel.  It ANDs the shifted src for hits
  *          and ANDs the inverted shifted src for misses.
- *      (2) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (2) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (3) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixHMT(NULL, pixs, ...);
+ *          (b) pixHMT(pixs, pixs, ...);
+ *          (c) pixHMT(pixd, pixs, ...);
+ *      (4) The size of the result is determined by pixs.
  */
 PIX *
 pixHMT(PIX  *pixd,
@@ -344,7 +364,7 @@ PIX     *pixt;
         }
     }
 
-        /* clear near edges */
+        /* Clear near edges */
     selFindMaxTranslations(sel, &xp, &yp, &xn, &yn);
     if (xp > 0)
         pixRasterop(pixd, 0, 0, xp, h, PIX_CLR, NULL, 0, 0);
@@ -363,17 +383,23 @@ PIX     *pixt;
 /*!
  *  pixOpen()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
  *  Notes:
  *      (1) Generic morphological opening, using hits in the Sel.
- *      (2) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (2) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (3) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixOpen(NULL, pixs, ...);
+ *          (b) pixOpen(pixs, pixs, ...);
+ *          (c) pixOpen(pixd, pixs, ...);
+ *      (4) The size of the result is determined by pixs.
  */
 PIX *
 pixOpen(PIX  *pixd,
@@ -399,8 +425,9 @@ PIX  *pixt;
 /*!
  *  pixClose()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd (<optional>; this can be null, equal to pixs,
+ *                    or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
@@ -409,10 +436,15 @@ PIX  *pixt;
  *      (2) This implementation is a strict dual of the opening if
  *          symmetric boundary conditions are used (see notes at top
  *          of this file).
- *      (3) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (3) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (4) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixClose(NULL, pixs, ...);
+ *          (b) pixClose(pixs, pixs, ...);
+ *          (c) pixClose(pixd, pixs, ...);
+ *      (5) The size of the result is determined by pixs.
  */
 PIX *
 pixClose(PIX  *pixd,
@@ -438,8 +470,9 @@ PIX  *pixt;
 /*!
  *  pixCloseSafe()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd (<optional>; this can be null, equal to pixs,
+ *                    or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
@@ -452,10 +485,15 @@ PIX  *pixt;
  *          It thus enforces a correct extensive result for closing.
  *      (3) If symmetric b.c. are used, it is not necessary to add
  *          and remove this border.
- *      (4) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (4) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (5) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixCloseSafe(NULL, pixs, ...);
+ *          (b) pixCloseSafe(pixs, pixs, ...);
+ *          (c) pixCloseSafe(pixd, pixs, ...);
+ *      (6) The size of the result is determined by pixs.
  */
 PIX *
 pixCloseSafe(PIX  *pixd,
@@ -474,12 +512,7 @@ PIX     *pixt1, *pixt2;
     if (pixGetDepth(pixs) != 1)
         return (PIX *)ERROR_PTR("pixs not 1 bpp", procName, pixd);
 
-    if (pixd) {
-        if (!pixSizesEqual(pixs, pixd))
-            L_WARNING("pix src and dest sizes unequal", procName);
-    }
-
-        /* symmetric b.c. handles correctly without added pixels */
+        /* Symmetric b.c. handles correctly without added pixels */
     if (MORPH_BC == SYMMETRIC_MORPH_BC)
         return pixClose(pixd, pixs, sel);
 
@@ -506,8 +539,9 @@ PIX     *pixt1, *pixt2;
 /*!
  *  pixOpenGeneralized()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd (<optional>; this can be null, equal to pixs,
+ *                    or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
@@ -516,10 +550,15 @@ PIX     *pixt1, *pixt2;
  *          misses in the Sel.
  *      (2) This does a hit-miss transform, followed by a dilation
  *          using the hits.
- *      (3) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (3) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (4) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixOpenGeneralized(NULL, pixs, ...);
+ *          (b) pixOpenGeneralized(pixs, pixs, ...);
+ *          (c) pixOpenGeneralized(pixd, pixs, ...);
+ *      (5) The size of the result is determined by pixs.
  */
 PIX *
 pixOpenGeneralized(PIX  *pixd,
@@ -537,7 +576,6 @@ PIX  *pixt;
         return (PIX *)ERROR_PTR("pixt not made", procName, pixd);
     pixDilate(pixd, pixt, sel);
     pixDestroy(&pixt);
-
     return pixd;
 }
     
@@ -545,8 +583,9 @@ PIX  *pixt;
 /*!
  *  pixCloseGeneralized()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd (<optional>; this can be null, equal to pixs,
+ *                    or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *      Return: pixd
  *
@@ -556,10 +595,15 @@ PIX  *pixt;
  *      (2) This does a dilation using the hits, followed by a
  *          hit-miss transform.
  *      (3) This operation is a dual of the generalized opening.
- *      (4) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (4) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (5) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixCloseGeneralized(NULL, pixs, ...);
+ *          (b) pixCloseGeneralized(pixs, pixs, ...);
+ *          (c) pixCloseGeneralized(pixd, pixs, ...);
+ *      (6) The size of the result is determined by pixs.
  */
 PIX *
 pixCloseGeneralized(PIX  *pixd,
@@ -588,8 +632,9 @@ PIX  *pixt;
 /*!
  *  pixDilateBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
  *      Return: pixd
@@ -598,10 +643,15 @@ PIX  *pixt;
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do separably if both hsize and vsize are > 1.
- *      (4) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (4) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (5) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixDilateBrick(NULL, pixs, ...);
+ *          (b) pixDilateBrick(pixs, pixs, ...);
+ *          (c) pixDilateBrick(pixd, pixs, ...);
+ *      (6) The size of the result is determined by pixs.
  */
 PIX *
 pixDilateBrick(PIX     *pixd,
@@ -645,8 +695,9 @@ SEL  *sel, *selh, *selv;
 /*!
  *  pixErodeBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
  *      Return: pixd
@@ -655,10 +706,15 @@ SEL  *sel, *selh, *selv;
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do separably if both hsize and vsize are > 1.
- *      (4) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (4) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (5) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixErodeBrick(NULL, pixs, ...);
+ *          (b) pixErodeBrick(pixs, pixs, ...);
+ *          (c) pixErodeBrick(pixd, pixs, ...);
+ *      (6) The size of the result is determined by pixs.
  */
 PIX *
 pixErodeBrick(PIX     *pixd,
@@ -702,20 +758,26 @@ SEL  *sel, *selh, *selv;
 /*!
  *  pixOpenBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do separably if both hsize and vsize are > 1.
- *      (4) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (4) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (5) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixOpenBrick(NULL, pixs, ...);
+ *          (b) pixOpenBrick(pixs, pixs, ...);
+ *          (c) pixOpenBrick(pixd, pixs, ...);
+ *      (6) The size of the result is determined by pixs.
  */
 PIX *
 pixOpenBrick(PIX     *pixd,
@@ -761,20 +823,26 @@ SEL  *sel, *selh, *selv;
 /*!
  *  pixCloseBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do separably if both hsize and vsize are > 1.
- *      (4) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (4) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (5) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixCloseBrick(NULL, pixs, ...);
+ *          (b) pixCloseBrick(pixs, pixs, ...);
+ *          (c) pixCloseBrick(pixd, pixs, ...);
+ *      (6) The size of the result is determined by pixs.
  */
 PIX *
 pixCloseBrick(PIX     *pixd,
@@ -820,11 +888,12 @@ SEL  *sel, *selh, *selv;
 /*!
  *  pixCloseSafeBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
@@ -835,10 +904,15 @@ SEL  *sel, *selh, *selv;
  *          32-bit words in the expanded image.  As a result, there is
  *          no special processing for pixels near the boundary, and there
  *          are no boundary effects.  The border is removed at the end.
- *      (5) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (5) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (6) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixCloseBrick(NULL, pixs, ...);
+ *          (b) pixCloseBrick(pixs, pixs, ...);
+ *          (c) pixCloseBrick(pixd, pixs, ...);
+ *      (7) The size of the result is determined by pixs.
  */
 PIX *
 pixCloseSafeBrick(PIX     *pixd,
@@ -909,8 +983,8 @@ SEL     *sel, *selh, *selv;
  *
  *      Input:  size (of composed sel)
  *              direction (L_HORIZ, L_VERT)
- *              &sel1 (<return> contiguous sel)
- *              &sel2 (<return> comb sel)
+ *              &sel1 (<optional return> contiguous sel; can be null)
+ *              &sel2 (<optional return> comb sel; can be null)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -939,22 +1013,26 @@ l_int32  factor1, factor2;
 
     PROCNAME("selectComposableSels");
 
+    if (!psel1 && !psel2)
+        return ERROR_INT("neither &sel1 nor &sel2 are defined", procName, 1);
+    if (psel1) *psel1 = NULL;
+    if (psel2) *psel2 = NULL;
     if (size < 1 || size > 250 * 250)
         return ERROR_INT("size < 1", procName, 1);
     if (direction != L_HORIZ && direction != L_VERT)
         return ERROR_INT("invalid direction", procName, 1);
-    if (!psel1 || !psel2)
-        return ERROR_INT("&sel1 or &sel2 not defined", procName, 1);
-    *psel1 = *psel2 = NULL;
 
     if (selectComposableSizes(size, &factor1, &factor2))
         return ERROR_INT("factors not found", procName, 1);
 
-    if (direction == L_HORIZ)
-        *psel1 = selCreateBrick(1, factor1, 0, factor1 / 2, SEL_HIT);
-    else
-        *psel1 = selCreateBrick(factor1, 1, factor1 / 2 , 0, SEL_HIT);
-    *psel2 = selCreateComb(factor1, factor2, direction);
+    if (psel1) {
+        if (direction == L_HORIZ)
+            *psel1 = selCreateBrick(1, factor1, 0, factor1 / 2, SEL_HIT);
+        else
+            *psel1 = selCreateBrick(factor1, 1, factor1 / 2 , 0, SEL_HIT);
+    }
+    if (psel2)
+        *psel2 = selCreateComb(factor1, factor2, direction);
     return 0;
 }
 
@@ -975,6 +1053,8 @@ l_int32  factor1, factor2;
  *      (3) We choose an overall cost function where the penalty for
  *          the size difference between input and actual is 4 times
  *          the penalty for additional rasterops.
+ *      (4) Returned values: factor1 >= factor2
+ *          If size > 1, then factor1 > 1.
  */
 l_int32
 selectComposableSizes(l_int32   size,
@@ -996,7 +1076,7 @@ l_int32  diff[256];  /* diff between product (sel size) and input size */
     if (!pfactor1 || !pfactor2)
         return ERROR_INT("&factor1 or &factor2 not defined", procName, 1);
     
-    midval = (l_int32)(sqrt(size) + 0.001);
+    midval = (l_int32)(sqrt((l_float64)size) + 0.001);
     if (midval * midval == size) {
         *pfactor1 = *pfactor2 = midval;
         return 0;
@@ -1051,21 +1131,40 @@ l_int32  diff[256];  /* diff between product (sel size) and input size */
 /*!
  *  pixDilateCompBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do compositely for each dimension > 1.
  *      (4) Do separably if both hsize and vsize are > 1.
- *      (5) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (5) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (6) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixDilateCompBrick(NULL, pixs, ...);
+ *          (b) pixDilateCompBrick(pixs, pixs, ...);
+ *          (c) pixDilateCompBrick(pixd, pixs, ...);
+ *      (7) The dimensions of the resulting image are determined by pixs.
+ *      (8) CAUTION: both hsize and vsize are being decomposed.
+ *          The decomposer chooses a product of sizes (call them
+ *          'terms') for each that is close to the input size,
+ *          but not necessarily equal to it.  It attempts to optimize:
+ *             (a) for consistency with the input values: the product
+ *                 of terms is close to the input size
+ *             (b) for efficiency of the operation: the sum of the
+ *                 terms is small; ideally about twice the square
+ *                 root of the input size.
+ *          So, for example, if the input hsize = 37, which is
+ *          a prime number, the decomposer will break this into two
+ *          terms, 6 and 6, so that the net result is a dilation
+ *          with hsize = 36.
  */
 PIX *
 pixDilateCompBrick(PIX     *pixd,
@@ -1073,7 +1172,7 @@ pixDilateCompBrick(PIX     *pixd,
                    l_int32  hsize,
                    l_int32  vsize)
 {
-PIX  *pixt;
+PIX  *pixt1, *pixt2, *pixt3;
 SEL  *selh1, *selh2, *selv1, *selv2;
 
     PROCNAME("pixDilateCompBrick");
@@ -1085,6 +1184,8 @@ SEL  *selh1, *selh2, *selv1, *selv2;
     if (hsize < 1 || vsize < 1)
         return (PIX *)ERROR_PTR("hsize and vsize not >= 1", procName, pixd);
 
+    pixt1 = pixAddBorder(pixs, 32, 0);
+
     if (hsize == 1 && vsize == 1)
         return pixCopy(pixd, pixs);
     if (hsize > 1)
@@ -1092,20 +1193,21 @@ SEL  *selh1, *selh2, *selv1, *selv2;
     if (vsize > 1)
         selectComposableSels(vsize, L_VERT, &selv1, &selv2);
     if (vsize == 1) {
-        pixt = pixDilate(NULL, pixs, selh1);
-        pixd = pixDilate(pixd, pixt, selh2);
+        pixt2 = pixDilate(NULL, pixt1, selh1);
+        pixt3 = pixDilate(NULL, pixt2, selh2);
     } 
     else if (hsize == 1) {
-        pixt = pixDilate(NULL, pixs, selv1);
-        pixd = pixDilate(pixd, pixt, selv2);
+        pixt2 = pixDilate(NULL, pixt1, selv1);
+        pixt3 = pixDilate(NULL, pixt2, selv2);
     } 
     else {
-        pixt = pixDilate(NULL, pixs, selh1);
-        pixd = pixDilate(pixd, pixt, selh2);
-        pixDilate(pixt, pixd, selv1);
-        pixDilate(pixd, pixt, selv2);
+        pixt2 = pixDilate(NULL, pixt1, selh1);
+        pixt3 = pixDilate(NULL, pixt2, selh2);
+        pixDilate(pixt2, pixt3, selv1);
+        pixDilate(pixt3, pixt2, selv2);
     }
-    pixDestroy(&pixt);
+    pixDestroy(&pixt1);
+    pixDestroy(&pixt2);
 
     if (hsize > 1) {
         selDestroy(&selh1);
@@ -1116,6 +1218,12 @@ SEL  *selh1, *selh2, *selv1, *selv2;
         selDestroy(&selv2);
     }
 
+    pixt1 = pixRemoveBorder(pixt3, 32);
+    pixDestroy(&pixt3);
+    if (!pixd)
+        return pixt1;
+    pixCopy(pixd, pixt1);
+    pixDestroy(&pixt1);
     return pixd;
 }
 
@@ -1123,21 +1231,40 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  *  pixErodeCompBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do compositely for each dimension > 1.
  *      (4) Do separably if both hsize and vsize are > 1.
- *      (5) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (5) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (6) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixErodeCompBrick(NULL, pixs, ...);
+ *          (b) pixErodeCompBrick(pixs, pixs, ...);
+ *          (c) pixErodeCompBrick(pixd, pixs, ...);
+ *      (7) The dimensions of the resulting image are determined by pixs.
+ *      (8) CAUTION: both hsize and vsize are being decomposed.
+ *          The decomposer chooses a product of sizes (call them
+ *          'terms') for each that is close to the input size,
+ *          but not necessarily equal to it.  It attempts to optimize:
+ *             (a) for consistency with the input values: the product
+ *                 of terms is close to the input size
+ *             (b) for efficiency of the operation: the sum of the
+ *                 terms is small; ideally about twice the square
+ *                 root of the input size.
+ *          So, for example, if the input hsize = 37, which is
+ *          a prime number, the decomposer will break this into two
+ *          terms, 6 and 6, so that the net result is a dilation
+ *          with hsize = 36.
  */
 PIX *
 pixErodeCompBrick(PIX     *pixd,
@@ -1195,21 +1322,40 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  *  pixOpenCompBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do compositely for each dimension > 1.
  *      (4) Do separably if both hsize and vsize are > 1.
- *      (5) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (5) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (6) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixOpenCompBrick(NULL, pixs, ...);
+ *          (b) pixOpenCompBrick(pixs, pixs, ...);
+ *          (c) pixOpenCompBrick(pixd, pixs, ...);
+ *      (7) The dimensions of the resulting image are determined by pixs.
+ *      (8) CAUTION: both hsize and vsize are being decomposed.
+ *          The decomposer chooses a product of sizes (call them
+ *          'terms') for each that is close to the input size,
+ *          but not necessarily equal to it.  It attempts to optimize:
+ *             (a) for consistency with the input values: the product
+ *                 of terms is close to the input size
+ *             (b) for efficiency of the operation: the sum of the
+ *                 terms is small; ideally about twice the square
+ *                 root of the input size.
+ *          So, for example, if the input hsize = 37, which is
+ *          a prime number, the decomposer will break this into two
+ *          terms, 6 and 6, so that the net result is a dilation
+ *          with hsize = 36.
  */
 PIX *
 pixOpenCompBrick(PIX     *pixd,
@@ -1275,21 +1421,40 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  *  pixCloseCompBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) The origin is at (x, y) = (hsize/2, vsize/2)
  *      (3) Do compositely for each dimension > 1.
  *      (4) Do separably if both hsize and vsize are > 1.
- *      (5) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (5) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (6) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixCloseCompBrick(NULL, pixs, ...);
+ *          (b) pixCloseCompBrick(pixs, pixs, ...);
+ *          (c) pixCloseCompBrick(pixd, pixs, ...);
+ *      (7) The dimensions of the resulting image are determined by pixs.
+ *      (8) CAUTION: both hsize and vsize are being decomposed.
+ *          The decomposer chooses a product of sizes (call them
+ *          'terms') for each that is close to the input size,
+ *          but not necessarily equal to it.  It attempts to optimize:
+ *             (a) for consistency with the input values: the product
+ *                 of terms is close to the input size
+ *             (b) for efficiency of the operation: the sum of the
+ *                 terms is small; ideally about twice the square
+ *                 root of the input size.
+ *          So, for example, if the input hsize = 37, which is
+ *          a prime number, the decomposer will break this into two
+ *          terms, 6 and 6, so that the net result is a dilation
+ *          with hsize = 36.
  */
 PIX *
 pixCloseCompBrick(PIX     *pixd,
@@ -1355,11 +1520,12 @@ SEL  *selh1, *selh2, *selv1, *selv2;
 /*!
  *  pixCloseSafeCompBrick()
  *
- *      Input:  pixd  (<optional>)
- *              pixs
+ *      Input:  pixd  (<optional>; this can be null, equal to pixs,
+ *                     or different from pixs)
+ *              pixs (1 bpp)
  *              hsize (width of brick Sel)
  *              vsize (height of brick Sel)
- *      Return: pixd
+ *      Return: pixd, or null on error
  *
  *  Notes:
  *      (1) Sel is a brick with all elements being hits
@@ -1371,10 +1537,28 @@ SEL  *selh1, *selh2, *selv1, *selv2;
  *          32-bit words in the expanded image.  As a result, there is
  *          no special processing for pixels near the boundary, and there
  *          are no boundary effects.  The border is removed at the end.
- *      (6) Three modes of usage:
- *          - pixd = NULL : result into new pixd, which is returned
- *          - pixd exists, != pixs : puts result into pixd
- *          - pixd == pixs : in-place operation; writes result back to pixs
+ *      (6) There are three cases:
+ *          (a) pixd == null   (result into new pixd)
+ *          (b) pixd == pixs   (in-place; writes result back to pixs)
+ *          (c) pixd != pixs   (puts result into existing pixd)
+ *      (7) For clarity, if the case is known, use these patterns:
+ *          (a) pixd = pixCloseSafeCompBrick(NULL, pixs, ...);
+ *          (b) pixCloseSafeCompBrick(pixs, pixs, ...);
+ *          (c) pixCloseSafeCompBrick(pixd, pixs, ...);
+ *      (8) The dimensions of the resulting image are determined by pixs.
+ *      (9) CAUTION: both hsize and vsize are being decomposed.
+ *          The decomposer chooses a product of sizes (call them
+ *          'terms') for each that is close to the input size,
+ *          but not necessarily equal to it.  It attempts to optimize:
+ *             (a) for consistency with the input values: the product
+ *                 of terms is close to the input size
+ *             (b) for efficiency of the operation: the sum of the
+ *                 terms is small; ideally about twice the square
+ *                 root of the input size.
+ *          So, for example, if the input hsize = 37, which is
+ *          a prime number, the decomposer will break this into two
+ *          terms, 6 and 6, so that the net result is a dilation
+ *          with hsize = 36.
  */
 PIX *
 pixCloseSafeCompBrick(PIX     *pixd,
@@ -1503,10 +1687,10 @@ getMorphBorderPixelColor(l_int32  type,
     if (MORPH_BC == ASYMMETRIC_MORPH_BC || type == L_MORPH_DILATE)
         return 0;
 
-        /* symmetric & erosion */
+        /* Symmetric & erosion */
     if (depth < 32)
         return ((1 << depth) - 1);
-    if (depth == 32)
+    else  /* depth == 32 */
         return 0xffffff00;
 }
 
@@ -1517,8 +1701,9 @@ getMorphBorderPixelColor(l_int32  type,
 /*!
  *  processMorphArgs1()
  *
- *      Input:  pixd (<optional>
- *              pixs
+ *      Input:  pixd (<optional>; this can be null, equal to pixs,
+ *                    or different from pixs)
+ *              pixs (1 bpp)
  *              sel
  *              &pixt (<returned>)
  *      Return: pixd, or null on error.
@@ -1536,6 +1721,9 @@ l_int32  sx, sy;
 
     PROCNAME("processMorphArgs1");
 
+    if (!ppixt)
+        return (PIX *)ERROR_PTR("&pixt not defined", procName, pixd);
+    *ppixt = NULL;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, pixd);
     if (!sel)
@@ -1547,15 +1735,16 @@ l_int32  sx, sy;
     if (sx == 0 || sy == 0)
         return (PIX *)ERROR_PTR("sel of size 0", procName, pixd);
 
+        /* We require pixd to exist and to be the same size as pixs.
+         * Further, pixt must be a copy (or clone) of pixs.  */
     if (!pixd) {
         if ((pixd = pixCreateTemplate(pixs)) == NULL)
             return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
         *ppixt = pixClone(pixs);
     }
     else {
-        if (!pixSizesEqual(pixs, pixd))
-            return (PIX *)ERROR_PTR("pix sizes unequal", procName, pixd);
-        if (pixd == pixs) {
+        pixResizeImageData(pixd, pixs);
+        if (pixd == pixs) {  /* in-place; must make a copy of pixs */
             if ((*ppixt = pixCopy(NULL, pixs)) == NULL)
                 return (PIX *)ERROR_PTR("pixt not made", procName, pixd);
         }
@@ -1591,15 +1780,8 @@ l_int32  sx, sy;
     if (sx == 0 || sy == 0)
         return (PIX *)ERROR_PTR("sel of size 0", procName, pixd);
 
-    if (!pixd) {
-        if ((pixd = pixCreateTemplate(pixs)) == NULL)
-            return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    }
-    else {
-        if (!pixSizesEqual(pixs, pixd))
-            return (PIX *)ERROR_PTR("pix sizes unequal", procName, pixd);
-    }
-
+    if (!pixd)
+        return pixCreateTemplate(pixs);
+    pixResizeImageData(pixd, pixs);
     return pixd;
 }
-
