@@ -33,7 +33,7 @@
  *      standard border size
  *      grayscale intensity scaling flags
  *      morphological tophat flags
- *      arithmetic operator flags
+ *      arithmetic and logical operator flags
  *      grayscale morphology selection flags
  *      distance function b.c. flags
  *      image comparison flags
@@ -115,11 +115,12 @@ enum {
 
 /*-------------------------------------------------------------------------*
  *         Direction flags for grayscale morphology, granulometry,         *
- *                           and composable Sels                           *
+ *                   composable Sels, and convolution                      *
  *-------------------------------------------------------------------------*/
 enum {
-    L_HORIZ      = 1,
-    L_VERT       = 2
+    L_HORIZ            = 1,
+    L_VERT             = 2,
+    L_BOTH_DIRECTIONS  = 3
 };
 
 
@@ -130,7 +131,8 @@ enum {
     L_MORPH_DILATE    = 1,
     L_MORPH_ERODE     = 2,
     L_MORPH_OPEN      = 3,
-    L_MORPH_CLOSE     = 4
+    L_MORPH_CLOSE     = 4,
+    L_MORPH_HMT       = 5
 };
 
 
@@ -153,14 +155,18 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *                      Arithmetic operator flags                          *
+ *                Arithmetic and logical operator flags                    *
  *                 (use on grayscale images and Numas)                     *
  *-------------------------------------------------------------------------*/
 enum {
     L_ARITH_ADD       = 1,
     L_ARITH_SUBTRACT  = 2,
-    L_ARITH_MULTIPLY  = 3,   /* on numas only               */
-    L_ARITH_DIVIDE    = 4    /* on numas only               */
+    L_ARITH_MULTIPLY  = 3,   /* on numas only */
+    L_ARITH_DIVIDE    = 4,   /* on numas only */
+    L_UNION           = 5,   /* on numas only */
+    L_INTERSECTION    = 6,   /* on numas only */
+    L_SUBTRACTION     = 7,   /* on numas only */
+    L_EXCLUSIVE_OR    = 8    /* on numas only */
 };
 
 
@@ -168,8 +174,9 @@ enum {
  *                        Min/max selection flags                          *
  *-------------------------------------------------------------------------*/
 enum {
-    L_CHOOSE_MIN = 1,
-    L_CHOOSE_MAX = 2
+    L_CHOOSE_MIN = 1,           /* useful in a downscaling "erosion"  */
+    L_CHOOSE_MAX = 2,           /* useful in a downscaling "dilation" */
+    L_CHOOSE_MAX_MIN_DIFF = 3   /* useful in a downscaling contrast   */
 };
 
 
