@@ -32,6 +32,8 @@
  *       struct Ptaa
  *       struct Pixacc
  *       struct PixTiling
+ *       struct FPix
+ *       struct DPix
  *
  *   Contains definitions for:
  *       colors for RGB
@@ -52,9 +54,14 @@
  *       text orientation flags
  *       edge orientation flags
  *       line orientation flags
+ *       scan direction flags
  *       thinning flags
  *       runlength flags
  *       edge filter flags
+ *       handling negative values in conversion to unsigned int
+ *       relative to zero flags
+ *       HSV histogram flags
+ *       region flags (inclusion, exclusion)
  */
 
 
@@ -113,11 +120,11 @@ typedef struct RGBA_Quad  RGBA_QUAD;
  *          They are used through the GET/SET_DATA_BYTE accessors.
  *          The 4th byte, typically known as the "alpha channel" and used
  *          for blending, is not explicitly used in leptonica.
- *      (2) If you redefine these values, a lot of functions that
- *          have been made more efficient by assuming them implicitly
- *          will break.  These functions are labelled with "***"
- *          next to their names at the top of the files in which they
- *          are defined.  Advice: Do not change these values!
+ *      (2) If you redefine these values, functions that have the shifts
+ *          hardcoded (instead of using the constants below) will break.
+ *          These functions are labelled with "***" next to their names
+ *          at the top of the files in which they are defined.
+ *          Advice: Do not change these values!
  *      (3) The shifts to extract the red, green and blue components
  *          from a 32 bit pixel are defined in terms of these colors.
  */
@@ -215,10 +222,10 @@ enum {
 
 #define   PIX_PAINT    (PIX_SRC | PIX_DST)
 #define   PIX_MASK     (PIX_SRC & PIX_DST)
+#define   PIX_SUBTRACT (PIX_DST & PIX_NOT(PIX_SRC))
 #define   PIX_XOR      (PIX_SRC ^ PIX_DST)
 
 
-
 /*-------------------------------------------------------------------------*
  *
  *   Important Notes:
@@ -264,26 +271,28 @@ enum {
  *                 green:  byte 1  (2nd MSB)
  *                 blue:   byte 2  (3rd MSB)
  *
- *           For simplicity and consistency, we have defined (COLOR_RED,
- *           COLOR_GREEN, COLOR_BLUE) so that we can retrieve red, green
- *           and blue sample values using
- *              redval = GET_DATA_BYTE(&pixel, COLOR_RED);
- *              greenval = GET_DATA_BYTE(&pixel, COLOR_GREEN);
- *              blueval = GET_DATA_BYTE(&pixel, COLOR_BLUE);
- *           and we can set them in the pixel using
- *              SET_DATA_BYTE(&pixel, COLOR_RED, redval);
- *              SET_DATA_BYTE(&pixel, COLOR_GREEN, greenval);
- *              SET_DATA_BYTE(&pixel, COLOR_BLUE, blueval);
+ *           This specific color assignment is made in this file,
+ *           through the definitions of COLOR_RED, etc.  Then the R, G
+ *           and B sample values can be retrieved using
+ *                 redval = GET_DATA_BYTE(&pixel, COLOR_RED);
+ *                 greenval = GET_DATA_BYTE(&pixel, COLOR_GREEN);
+ *                 blueval = GET_DATA_BYTE(&pixel, COLOR_BLUE);
+ *           and they can be set with
+ *                 SET_DATA_BYTE(&pixel, COLOR_RED, redval);
+ *                 SET_DATA_BYTE(&pixel, COLOR_GREEN, greenval);
+ *                 SET_DATA_BYTE(&pixel, COLOR_BLUE, blueval);
  *
- *           In some functions, for extra speed we extract the R, G and B
- *           colors directly by shifting and masking, implicitly using the
- *           specific values in COLOR_RED, COLOR_GREEN and COLOR_BLUE.
- *           For example:
- *                   pixel32 >> 24;              (red)
- *                   (pixel32 >> 16) & 0xff;     (green)
- *                   (pixel32 >> 8) & 0xff;      (blue)
- *           These operations work properly on both big- and little-endians.
- *           Functions where this is done are marked with ***.
+ *           For extra speed we extract the R, G and B colors directly
+ *           by shifting and masking, explicitly using the values in
+ *           L_RED_SHIFT, L_GREEN_SHIFT and L_BLUE_SHIFT:
+ *                 (pixel32 >> L_RED_SHIFT) & 0xff;         (red)
+ *                 (pixel32 >> L_GREEN_SHIFT) & 0xff;       (green)
+ *                 (pixel32 >> L_BLUE_SHIFT) & 0xff;        (blue)
+ *           All these operations work properly on both big- and little-endians.
+ *
+ *           For a few situations, these color shift values are hard-coded.
+ *           Changing the RGB color component ordering through the assignments
+ *           in this file will cause functions marked with "***" to fail.
  *
  *       (5) A reference count is held within each pix, giving the
  *           number of ptrs to the pix.  When a pixClone() call
@@ -294,13 +303,21 @@ enum {
  *
  *       (6) The version numbers (below) are used in the serialization
  *           of these data structures.  They are placed in the files,
- *           and rarely (if ever) change.  No provision is made for
- *           backward compatibility in either reading or writing.
+ *           and rarely (if ever) change.  Provision is currently made for
+ *           backward compatibility in reading from boxaa version 2.
  *
- *       (7) The PIXA_VERSION_NUMBER is applied to pixa, pixaa, boxa,
- *           and boxaa, and must be changed if any of these serializations
- *           are changed.  That is because pixaa depends on pixa and boxa,
- *           and pixa and boxaa depend on boxa.
+ *       (7) The serialization dependencies are as follows:
+ *               pixaa  :  pixa  :  boxa
+ *               boxaa  :  boxa
+ *           So, for example, pixaa and boxaa can be changed without
+ *           forcing a change in pixa or boxa.  However, if pixa is
+ *           changed, it forces a change in pixaa, and if boxa is
+ *           changed, if forces a change in the other three.
+ *           We define four version numbers:
+ *               PIXAA_VERSION_NUMBER
+ *               PIXA_VERSION_NUMBER
+ *               BOXAA_VERSION_NUMBER
+ *               BOXA_VERSION_NUMBER
  *
  *-------------------------------------------------------------------------*/
 
@@ -309,7 +326,13 @@ enum {
 /*-------------------------------------------------------------------------*
  *                              Array of pix                               *
  *-------------------------------------------------------------------------*/
-#define  PIXA_VERSION_NUMBER      2
+
+    /*  Serialization for primary data structures */
+#define  PIXAA_VERSION_NUMBER      2
+#define  PIXA_VERSION_NUMBER       2
+#define  BOXA_VERSION_NUMBER       2
+#define  BOXAA_VERSION_NUMBER      3
+
 
 struct Pixa
 {
@@ -416,11 +439,49 @@ struct PixTiling
     l_int32              ny;          /* number of tiles vertically        */
     l_int32              w;           /* tile width                        */
     l_int32              h;           /* tile height                       */
-    l_int32              overlap;     /* amount of overlap on each side    */
+    l_int32              xoverlap;    /* overlap on left and right         */
+    l_int32              yoverlap;    /* overlap on top and bottom         */
+    l_int32              strip;       /* strip for paint; default is TRUE  */
 };
 typedef struct PixTiling PIXTILING;
 
 
+/*-------------------------------------------------------------------------*
+ *                       FPix: pix with float array                        *
+ *-------------------------------------------------------------------------*/
+struct FPix
+{
+    l_int32              w;           /* width in pixels                   */
+    l_int32              h;           /* height in pixels                  */
+    l_int32              wpl;         /* 32-bit words/line                 */
+    l_int32              refcount;    /* reference count (1 if no clones)  */
+    l_int32              xres;        /* image res (ppi) in x direction    */
+                                      /* (use 0 if unknown)                */
+    l_int32              yres;        /* image res (ppi) in y direction    */
+                                      /* (use 0 if unknown)                */
+    l_float32           *data;        /* the float image data              */
+};
+typedef struct FPix FPIX;
+
+
+/*-------------------------------------------------------------------------*
+ *                       DPix: pix with double array                       *
+ *-------------------------------------------------------------------------*/
+struct DPix
+{
+    l_int32              w;           /* width in pixels                   */
+    l_int32              h;           /* height in pixels                  */
+    l_int32              wpl;         /* 32-bit words/line                 */
+    l_int32              refcount;    /* reference count (1 if no clones)  */
+    l_int32              xres;        /* image res (ppi) in x direction    */
+                                      /* (use 0 if unknown)                */
+    l_int32              yres;        /* image res (ppi) in y direction    */
+                                      /* (use 0 if unknown)                */
+    l_float64           *data;        /* the double image data             */
+};
+typedef struct DPix DPIX;
+
+
 /*-------------------------------------------------------------------------*
  *                         Access and storage flags                        *
  *-------------------------------------------------------------------------*/
@@ -465,23 +526,24 @@ enum {
 static const l_int32  L_NOCOPY = 0;  /* copyflag value in sarrayGetString() */
 
 
-/*-------------------------------------------------------------------------*
- *                              Sort flags                                 *
- *-------------------------------------------------------------------------*/
+/*--------------------------------------------------------------------------*
+ *                              Sort flags                                  *
+ *--------------------------------------------------------------------------*/
 enum {
-    L_SORT_INCREASING = 1,        /* sort in increasing order              */
-    L_SORT_DECREASING = 2         /* sort in decreasing order              */
+    L_SORT_INCREASING = 1,        /* sort in increasing order               */
+    L_SORT_DECREASING = 2         /* sort in decreasing order               */
 };
 
 enum {
-    L_SORT_BY_X = 3,              /* sort box or c.c. by horiz location    */
-    L_SORT_BY_Y = 4,              /* sort box or c.c. by vert location     */
-    L_SORT_BY_WIDTH = 5,          /* sort box or c.c. by width             */
-    L_SORT_BY_HEIGHT = 6,         /* sort box or c.c. by height            */
-    L_SORT_BY_MIN_DIMENSION = 7,  /* sort box or c.c. by min dimension     */
-    L_SORT_BY_MAX_DIMENSION = 8,  /* sort box or c.c. by max dimension     */
-    L_SORT_BY_PERIMETER = 9,      /* sort box or c.c. by perimeter         */
-    L_SORT_BY_AREA = 10           /* sort box or c.c. by area              */
+    L_SORT_BY_X = 3,              /* sort box or c.c. by horiz location     */
+    L_SORT_BY_Y = 4,              /* sort box or c.c. by vert location      */
+    L_SORT_BY_WIDTH = 5,          /* sort box or c.c. by width              */
+    L_SORT_BY_HEIGHT = 6,         /* sort box or c.c. by height             */
+    L_SORT_BY_MIN_DIMENSION = 7,  /* sort box or c.c. by min dimension      */
+    L_SORT_BY_MAX_DIMENSION = 8,  /* sort box or c.c. by max dimension      */
+    L_SORT_BY_PERIMETER = 9,      /* sort box or c.c. by perimeter          */
+    L_SORT_BY_AREA = 10,          /* sort box or c.c. by area               */
+    L_SORT_BY_ASPECT_RATIO = 11   /* sort box or c.c. by width/height ratio */
 };
 
 
@@ -536,7 +598,8 @@ enum {
  *-------------------------------------------------------------------------*/
 enum {
     L_ROTATE_AREA_MAP = 1,       /* use area map rotation, if possible     */
-    L_ROTATE_SHEAR = 2           /* use shear rotation                     */
+    L_ROTATE_SHEAR = 2,          /* use shear rotation                     */
+    L_ROTATE_SAMPLING = 3        /* use sampling                           */
 };
 
 enum {
@@ -544,6 +607,11 @@ enum {
     L_BRING_IN_BLACK = 2         /* bring in black pixels from the outside */
 };
 
+enum {
+    L_SHEAR_ABOUT_CORNER = 1,    /* shear image about UL corner            */
+    L_SHEAR_ABOUT_CENTER = 2     /* shear image about center               */
+};
+
 
 /*-------------------------------------------------------------------------*
  *                     Affine transform order flags                        *
@@ -594,9 +662,12 @@ enum {
  *-------------------------------------------------------------------------*/
 enum {
     L_MEAN_ABSVAL = 1,           /* average of abs values                  */
-    L_ROOT_MEAN_SQUARE = 2,      /* rms of values                          */
-    L_STANDARD_DEVIATION = 3,    /* standard deviation from mean           */
-    L_VARIANCE = 4               /* variance of values                     */
+    L_MEDIAN_VAL = 2,            /* median value of set                    */
+    L_MODE_VAL = 3,              /* mode value of set                      */
+    L_MODE_COUNT = 4,            /* mode count of set                      */
+    L_ROOT_MEAN_SQUARE = 5,      /* rms of values                          */
+    L_STANDARD_DEVIATION = 6,    /* standard deviation from mean           */
+    L_VARIANCE = 7               /* variance of values                     */
 };
 
 
@@ -642,6 +713,17 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                           Scan direction flags                          *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_FROM_LEFT = 0,           /* scan from left                           */
+    L_FROM_RIGHT = 1,          /* scan from right                          */
+    L_FROM_TOP = 2,            /* scan from top                            */
+    L_FROM_BOTTOM = 3          /* scan from bottom                         */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                             Thinning flags                              *
  *-------------------------------------------------------------------------*/
@@ -669,5 +751,43 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *          Handling negative values in conversion to unsigned int         *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_CLIP_TO_ZERO = 1,        /* Clip negative values to 0                */
+    L_TAKE_ABSVAL = 2          /* Convert to positive using L_ABS()        */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                         Relative to zero flags                          *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_LESS_THAN_ZERO = 1,      /* Choose values less than zero             */
+    L_EQUAL_TO_ZERO = 2,       /* Choose values equal to zero              */
+    L_GREATER_THAN_ZERO = 3    /* Choose values greater than zero          */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                          HSV histogram flags                            *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_HS_HISTO = 1,            /* Use hue-saturation histogram             */
+    L_HV_HISTO = 2,            /* Use hue-value histogram                  */
+    L_SV_HISTO = 3             /* Use saturation-value histogram           */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                    Region flags (inclusion, exclusion)                  *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_INCLUDE_REGION = 1,      /* Use hue-saturation histogram             */
+    L_EXCLUDE_REGION = 2       /* Use hue-value histogram                  */
+};
+
+
 #endif  /* LEPTONICA_PIX_H */
 
