@@ -28,6 +28,7 @@
  *           l_int32     pixSetPixel()
  *           l_int32     pixGetRGBPixel()
  *           l_int32     pixSetRGBPixel()
+ *           l_int32     pixGetRandomPixel()
  *           l_int32     pixClearPixel()
  *           l_int32     pixFlipPixel()
  *           void        setPixelLow()
@@ -36,20 +37,23 @@
  *           l_int32     pixClearAll()
  *           l_int32     pixSetAll()
  *           l_int32     pixSetAllArbitrary()
+ *           l_int32     pixSetBlackOrWhite()
  *
- *      Rectangular region clear/set/set-to-arbitrary-value
+ *      Rectangular region clear/set/set-to-arbitrary-value/blend
  *           l_int32     pixClearInRect()
  *           l_int32     pixSetInRect()
  *           l_int32     pixSetInRectArbitrary()
+ *           l_int32     pixBlendInRect()
  *
  *      Set pad bits
  *           l_int32     pixSetPadBits()
  *           l_int32     pixSetPadBitsBand()
  *
- *      Set border pixels to arbitrary value
+ *      Assign border pixels
  *           l_int32     pixSetOrClearBorder()
  *           l_int32     pixSetBorderVal()
  *           l_int32     pixSetMirroredBorder()
+ *           PIX        *pixCopyBorder()
  *
  *      Add and remove border
  *           PIX        *pixAddBorder()
@@ -57,6 +61,8 @@
  *           PIX        *pixRemoveBorder()
  *           PIX        *pixRemoveBorderGeneral()
  *           PIX        *pixAddMirroredBorder()
+ *           PIX        *pixAddRepeatedBorder()
+ *           PIX        *pixAddMixedBorder()
  *
  *      Color sample setting and extraction
  *           PIX        *pixCreateRGBImage()
@@ -70,6 +76,7 @@
  *      Conversion between big and little endians
  *           PIX        *pixEndianByteSwapNew()
  *           l_int32     pixEndianByteSwap()
+ *           l_int32     lineEndianByteSwap()
  *           PIX        *pixEndianTwoByteSwapNew()
  *           l_int32     pixEndianTwoByteSwap()
  *
@@ -104,6 +111,10 @@ static const l_uint32 rmask32[] = {0x0,
  *              (x,y) pixel coords
  *              &val (<return> pixel value)
  *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) This returns the value in the data array.  If the pix is
+ *          colormapped, it returns the colormap index, not the rgb value.
  */
 l_int32
 pixGetPixel(PIX       *pix,
@@ -116,11 +127,11 @@ l_uint32  *line, *data;
 
     PROCNAME("pixGetPixel");
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
     if (!pval)
         return ERROR_INT("pval not defined", procName, 1);
     *pval = 0;
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
 
     pixGetDimensions(pix, &w, &h, &d);
     if (x < 0 || x >= w)
@@ -228,7 +239,7 @@ l_uint32  *line, *data;
 /*!
  *  pixGetRGBPixel()
  *
- *      Input:  pix
+ *      Input:  pix (32 bpp rgb, not colormapped)
  *              (x,y) pixel coords
  *              &rval (<optional return> red component)
  *              &gval (<optional return> green component)
@@ -271,7 +282,7 @@ l_uint32  *data, *ppixel;
 /*!
  *  pixSetRGBPixel()
  *
- *      Input:  pix
+ *      Input:  pix (32 bpp rgb)
  *              (x,y) pixel coords
  *              rval (red component)
  *              gval (green component)
@@ -311,6 +322,53 @@ l_uint32  *data, *line;
 }
 
 
+/*!
+ *  pixGetRandomPixel()
+ *
+ *      Input:  pix (any depth; can be colormapped)
+ *              &val (<return> pixel value)
+ *              &x (<optional return> x coordinate chosen; can be null)
+ *              &y (<optional return> y coordinate chosen; can be null)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) If the pix is colormapped, it returns the rgb value.
+ */
+l_int32
+pixGetRandomPixel(PIX       *pix,
+                  l_uint32  *pval,
+                  l_int32   *px,
+                  l_int32   *py)
+{
+l_int32   w, h, x, y, rval, gval, bval;
+l_uint32  val;
+PIXCMAP  *cmap;
+
+    PROCNAME("pixGetRandomPixel");
+
+    if (!pval)
+        return ERROR_INT("pval not defined", procName, 1);
+    *pval = 0;
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+
+    pixGetDimensions(pix, &w, &h, NULL);
+    x = rand() % w;
+    y = rand() % h;
+    if (px) *px = x;
+    if (py) *py = y;
+    pixGetPixel(pix, x, y, &val);
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        pixcmapGetColor(cmap, val, &rval, &gval, &bval);
+        composeRGBPixel(rval, gval, bval, pval);
+    }
+    else
+        *pval = val;
+
+    return 0;
+}
+
+
 /*!
  *  pixClearPixel()
  *
@@ -448,7 +506,8 @@ l_uint32  *line, *data;
  *              val (to be inserted)
  *      Return: void
  *
- *  Note: input variables are not checked
+ *  Notes:
+ *      (1) Caution: input variables are not checked!
  */
 void
 setPixelLow(l_uint32  *line,
@@ -493,10 +552,14 @@ setPixelLow(l_uint32  *line,
 /*!
  *  pixClearAll()
  *
- *      Input:  pix
+ *      Input:  pix (all depths; use cmapped with caution)
  *      Return: 0 if OK, 1 on error
  *
- *  Action: clears all data to 0
+ *  Notes:
+ *      (1) Clears all data to 0.  For 1 bpp, this is white; for grayscale
+ *          or color, this is black.
+ *      (2) Caution: for colormapped pix, this sets the color to the first
+ *          one in the colormap.  Be sure that this is the intended color!
  */
 l_int32
 pixClearAll(PIX  *pix)
@@ -505,7 +568,7 @@ pixClearAll(PIX  *pix)
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
-
+        
     pixRasterop(pix, 0, 0, pixGetWidth(pix), pixGetHeight(pix),
                 PIX_CLR, NULL, 0, 0);
     return 0;
@@ -515,18 +578,31 @@ pixClearAll(PIX  *pix)
 /*!
  *  pixSetAll()
  *
- *      Input:  pix
+ *      Input:  pix (all depths; use cmapped with caution)
  *      Return: 0 if OK, 1 on error
  *
- *  Action: sets all data to 1
+ *  Notes:
+ *      (1) Sets all data to 1.  For 1 bpp, this is black; for grayscale
+ *          or color, this is white.
+ *      (2) Caution: for colormapped pix, this sets the pixel value to the
+ *          maximum value supported by the colormap: 2^d - 1.  However, this
+ *          color may not be defined, because the colormap may not be full.
  */
 l_int32
 pixSetAll(PIX  *pix)
 {
+l_int32   n;
+PIXCMAP  *cmap;
+
     PROCNAME("pixSetAll");
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        n = pixcmapGetCount(cmap);
+        if (n < cmap->nalloc)  /* cmap is not full */
+            return ERROR_INT("cmap entry does not exist", procName, 1);
+    }
 
     pixRasterop(pix, 0, 0, pixGetWidth(pix), pixGetHeight(pix),
                 PIX_SET, NULL, 0, 0);
@@ -537,22 +613,41 @@ pixSetAll(PIX  *pix)
 /*!
  *  pixSetAllArbitrary()
  *
- *      Input:  pix
+ *      Input:  pix (all depths; use cmapped with caution)
  *              val  (value to set all pixels)
  *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) For colormapped pix, be sure the value is the intended
+ *          one in the colormap.
+ *      (2) Caution: for colormapped pix, this sets each pixel to the
+ *          color at the index equal to val.  Be sure that this index
+ *          exists in the colormap and that it is the intended one!
  */
 l_int32
 pixSetAllArbitrary(PIX      *pix,
                    l_uint32  val)
 {
-l_int32    i, j, w, h, d, wpl, npix;
+l_int32    n, i, j, w, h, d, wpl, npix;
 l_uint32   maxval, wordval;
 l_uint32  *data, *line;
+PIXCMAP   *cmap;
 
     PROCNAME("pixSetAllArbitrary");
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        n = pixcmapGetCount(cmap);
+        if (val < 0) {
+            L_WARNING("index not in colormap; using first color", procName);
+            val = 0;
+        }
+        else if (val >= n) {
+            L_WARNING("index not in colormap; using last color", procName);
+            val = n - 1;
+        }
+    }
 
     pixGetDimensions(pix, &w, &h, &d);
     if (d == 32)
@@ -588,15 +683,71 @@ l_uint32  *data, *line;
 }
 
 
+/*!
+ *  pixSetBlackOrWhite()
+ *
+ *      Input:  pixs (all depths; cmap ok)
+ *              incolor (L_BRING_IN_BLACK or L_BRING_IN_WHITE)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) Function for setting all pixels in an image to either black
+ *          or white.
+ *      (2) If pixs is colormapped, it adds black or white to the
+ *          colormap if it's not there and there is room.  If the colormap
+ *          is full, it finds the closest color in intensity.
+ *          This index is written to all pixels.
+ */
+l_int32
+pixSetBlackOrWhite(PIX     *pixs,
+                   l_int32  incolor)
+{
+l_int32   d, index;
+PIXCMAP  *cmap;
+
+    PROCNAME("pixSetBlackOrWhite");
+
+    if (!pixs)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (incolor != L_BRING_IN_BLACK && incolor != L_BRING_IN_WHITE)
+        return ERROR_INT("invalid incolor", procName, 1);
+
+    cmap = pixGetColormap(pixs);
+    d = pixGetDepth(pixs);
+    if (!cmap) {
+        if ((d == 1 && incolor == L_BRING_IN_BLACK) ||
+            (d > 1 && incolor == L_BRING_IN_WHITE))
+            pixSetAll(pixs);
+        else 
+            pixClearAll(pixs);
+    }
+    else {  /* handle colormap */
+        if (incolor == L_BRING_IN_BLACK)
+            pixcmapAddBlackOrWhite(cmap, 0, &index);
+        else  /* L_BRING_IN_WHITE */
+            pixcmapAddBlackOrWhite(cmap, 1, &index);
+        pixSetAllArbitrary(pixs, index);
+    }
+
+    return 0;
+}
+
+
 /*-------------------------------------------------------------*
  *     Rectangular region clear/set/set-to-arbitrary-value     *
  *-------------------------------------------------------------*/
 /*!
  *  pixClearInRect()
  *
- *      Input:  pix (all depths)
+ *      Input:  pix (all depths; can be cmapped)
  *              box (in which all pixels will be cleared)
  *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Clears all data in rect to 0.  For 1 bpp, this is white;
+ *          for grayscale or color, this is black.
+ *      (2) Caution: for colormapped pix, this sets the color to the first
+ *          one in the colormap.  Be sure that this is the intended color!
  */
 l_int32
 pixClearInRect(PIX  *pix,
@@ -620,15 +771,23 @@ l_int32  x, y, w, h;
 /*!
  *  pixSetInRect()
  *
- *      Input:  pix (all depths)
+ *      Input:  pix (all depths, can be cmapped)
  *              box (in which all pixels will be set)
  *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Sets all data in rect to 1.  For 1 bpp, this is black;
+ *          for grayscale or color, this is white.
+ *      (2) Caution: for colormapped pix, this sets the pixel value to the
+ *          maximum value supported by the colormap: 2^d - 1.  However, this
+ *          color may not be defined, because the colormap may not be full.
  */
 l_int32
 pixSetInRect(PIX  *pix,
              BOX  *box)
 {
-l_int32  x, y, w, h;
+l_int32   n, x, y, w, h;
+PIXCMAP  *cmap;
 
     PROCNAME("pixSetInRect");
 
@@ -636,6 +795,11 @@ l_int32  x, y, w, h;
         return ERROR_INT("pix not defined", procName, 1);
     if (!box)
         return ERROR_INT("box not defined", procName, 1);
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        n = pixcmapGetCount(cmap);
+        if (n < cmap->nalloc)  /* cmap is not full */
+            return ERROR_INT("cmap entry does not exist", procName, 1);
+    }
 
     boxGetGeometry(box, &x, &y, &w, &h);
     pixRasterop(pix, x, y, w, h, PIX_SET, NULL, 0, 0);
@@ -646,19 +810,27 @@ l_int32  x, y, w, h;
 /*!
  *  pixSetInRectArbitrary()
  *
- *      Input:  pix (all depths)
+ *      Input:  pix (all depths; can be cmapped)
  *              box (in which all pixels will be set to val)
  *              val  (value to set all pixels)
  *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) For colormapped pix, be sure the value is the intended
+ *          one in the colormap.
+ *      (2) Caution: for colormapped pix, this sets each pixel in the
+ *          rect to the color at the index equal to val.  Be sure that
+ *          this index exists in the colormap and that it is the intended one!
  */
 l_int32
 pixSetInRectArbitrary(PIX      *pix,
                       BOX      *box,
                       l_uint32  val)
 {
-l_int32    x, y, xstart, xend, ystart, yend, bw, bh, w, h, d, wpl, maxval;
+l_int32    n, x, y, xstart, xend, ystart, yend, bw, bh, w, h, d, wpl, maxval;
 l_uint32  *data, *line;
 BOX       *boxc;
+PIXCMAP   *cmap;
 
     PROCNAME("pixSetInRectArbitrary");
 
@@ -669,6 +841,17 @@ BOX       *boxc;
     pixGetDimensions(pix, &w, &h, &d);
     if (d != 1 && d != 2 && d != 4 && d !=8 && d != 16 && d != 32)
         return ERROR_INT("depth must be in {1,2,4,8,16,32} bpp", procName, 1);
+    if ((cmap = pixGetColormap(pix)) != NULL) {
+        n = pixcmapGetCount(cmap);
+        if (val < 0) {
+            L_WARNING("index not in colormap; using first color", procName);
+            val = 0;
+        }
+        else if (val >= n) {
+            L_WARNING("index not in colormap; using last color", procName);
+            val = n - 1;
+        }
+    }
 
     if (d == 32)
         maxval = 0xffffffff;
@@ -739,6 +922,61 @@ BOX       *boxc;
 }
 
 
+/*!
+ *  pixBlendInRect()
+ *
+ *      Input:  pixs (32 bpp rgb)
+ *              box (in which all pixels will be blended)
+ *              val  (blend value; 0xrrggbb00)
+ *              fract (fraction of color to be blended with each pixel in pixs)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) This is an in-place function.  It blends the input color @val
+ *          with the pixels in pixs in the specified rectangle.
+ */
+l_int32
+pixBlendInRect(PIX       *pixs,
+               BOX       *box,
+               l_uint32   val,
+               l_float32  fract)
+{
+l_int32    i, j, bx, by, bw, bh, w, h, wpls;
+l_int32    prval, pgval, pbval, rval, gval, bval;
+l_uint32   val32;
+l_uint32  *datas, *lines;
+
+    PROCNAME("pixBlendInRect");
+
+    if (!pixs || pixGetDepth(pixs) != 32)
+        return ERROR_INT("pixs not defined or not 32 bpp", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+
+    boxGetGeometry(box, &bx, &by, &bw, &bh);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    extractRGBValues(val, &rval, &gval, &bval);
+    for (i = 0; i < bh; i++) {   /* scan over box */
+        if (by + i < 0 || by + i >= h) continue;
+        lines = datas + (by + i) * wpls;
+        for (j = 0; j < bw; j++) {
+            if (bx + j < 0 || bx + j >= w) continue;
+            val32 = *(lines + bx + j);
+            extractRGBValues(val32, &prval, &pgval, &pbval);
+            prval = (l_int32)((1. - fract) * prval + fract * rval);
+            pgval = (l_int32)((1. - fract) * pgval + fract * gval);
+            pbval = (l_int32)((1. - fract) * pbval + fract * bval);
+            composeRGBPixel(prval, pgval, pbval, &val32);
+            *(lines + bx + j) = val32;
+        }
+    }
+
+    return 0;
+}
+
+
 /*-------------------------------------------------------------*
  *                         Set pad bits                        *
  *-------------------------------------------------------------*/
@@ -1072,6 +1310,60 @@ l_int32  i, j, w, h;
 }
      
 
+/*!
+ *  pixCopyBorder()
+ *
+ *      Input:  pixd (all depths; colormap ok; can be NULL)
+ *              pixs (same depth and size as pixd)
+ *              left, right, top, bot (number of pixels to copy)
+ *      Return: pixd, or null on error if pixd is not defined
+ *
+ *  Notes:
+ *      (1) pixd can be null, but otherwise it must be the same size
+ *          and depth as pixs.  Always returns pixd.
+ *      (1) This is useful in situations where by setting a few border
+ *          pixels we can avoid having to copy all pixels in pixs into
+ *          pixd as an initialization step for some operation.
+ */
+PIX *
+pixCopyBorder(PIX     *pixd,
+              PIX     *pixs,
+              l_int32  left,
+              l_int32  right,
+              l_int32  top,
+              l_int32  bot)
+{
+l_int32  w, h;
+
+    PROCNAME("pixCopyBorder");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, pixd);
+
+    if (pixd) {
+        if (pixd == pixs) {
+            L_WARNING("same: nothing to do", procName);
+            return pixd;
+        }
+        else if (!pixSizesEqual(pixs, pixd))
+            return (PIX *)ERROR_PTR("pixs and pixd sizes differ",
+                                    procName, pixd);
+    }
+    else {
+        if ((pixd = pixCreateTemplateNoInit(pixs)) == NULL)
+            return (PIX *)ERROR_PTR("pixd not made", procName, pixd);
+    }
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    pixRasterop(pixd, 0, 0, left, h, PIX_SRC, pixs, 0, 0);
+    pixRasterop(pixd, w - right, 0, right, h, PIX_SRC, pixs, w - right, 0);
+    pixRasterop(pixd, 0, 0, w, top, PIX_SRC, pixs, 0, 0);
+    pixRasterop(pixd, 0, h - bot, w, bot, PIX_SRC, pixs, 0, h - bot);
+
+    return pixd;
+}
+
+
 
 /*-------------------------------------------------------------*
  *                     Add and remove border                   *
@@ -1268,9 +1560,12 @@ PIX     *pixd;
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-    pixd = pixAddBorderGeneral(pixs, left, right, top, bot, 0);
-
     pixGetDimensions(pixs, &w, &h, NULL);
+    if (left > w || right > w || top > h || bot > h)
+        return (PIX *)ERROR_PTR("border too large", procName, NULL);
+
+        /* Set pixels on left, right, top and bottom, in that order */
+    pixd = pixAddBorderGeneral(pixs, left, right, top, bot, 0);
     for (j = 0; j < left; j++)
         pixRasterop(pixd, left - 1 - j, top, 1, h, PIX_SRC,
                     pixd, left + j, top);
@@ -1287,6 +1582,107 @@ PIX     *pixd;
     return pixd;
 }
      
+/*!
+ *  pixAddRepeatedBorder()
+ *
+ *      Input:  pixs (all depths; colormap ok)
+ *              left, right, top, bot (number of pixels added)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) This applies a repeated border, as if the central part of
+ *          the image is tiled over the plane.  So, for example, the
+ *          pixels in the left border come from the right side of the image.
+ *      (2) The general pixRasterop() is used for an in-place operation here
+ *          because there is no overlap between the src and dest rectangles.
+ */
+PIX  *
+pixAddRepeatedBorder(PIX      *pixs,
+                     l_int32  left,
+                     l_int32  right,
+                     l_int32  top,
+                     l_int32  bot)
+{
+l_int32  w, h;
+PIX     *pixd;
+
+    PROCNAME("pixAddRepeatedBorder");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    if (left > w || right > w || top > h || bot > h)
+        return (PIX *)ERROR_PTR("border too large", procName, NULL);
+
+    pixd = pixAddBorderGeneral(pixs, left, right, top, bot, 0);
+
+        /* Set pixels on left, right, top and bottom, in that order */
+    pixRasterop(pixd, 0, top, left, h, PIX_SRC, pixd, w, top);
+    pixRasterop(pixd, left + w, top, right, h, PIX_SRC, pixd, left, top);
+    pixRasterop(pixd, 0, 0, left + w + right, top, PIX_SRC, pixd, 0, h);
+    pixRasterop(pixd, 0, top + h, left + w + right, bot, PIX_SRC, pixd, 0, top);
+
+    return pixd;
+}
+     
+
+/*!
+ *  pixAddMixedBorder()
+ *
+ *      Input:  pixs (all depths; colormap ok)
+ *              left, right, top, bot (number of pixels added)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) This applies mirrored boundary conditions horizontally
+ *          and repeated b.c. vertically.
+ *      (2) It is specifically used for avoiding special operations
+ *          near boundaries when convolving a hue-saturation histogram
+ *          with a given window size.  The repeated b.c. are used
+ *          vertically for hue, and the mirrored b.c. are used
+ *          horizontally for saturation.  The number of pixels added
+ *          on each side is approximately (but not quite) half the
+ *          filter dimension.  The image processing operations can
+ *          then proceed over a region equal to the size of the original
+ *          image, and write directly into a dest pix of the same
+ *          size as pixs.
+ *      (3) The general pixRasterop() can be used for an in-place
+ *          operation here because there is no overlap between the
+ *          src and dest rectangles.
+ */
+PIX  *
+pixAddMixedBorder(PIX      *pixs,
+                  l_int32  left,
+                  l_int32  right,
+                  l_int32  top,
+                  l_int32  bot)
+{
+l_int32  j, w, h;
+PIX     *pixd;
+
+    PROCNAME("pixAddMixedBorder");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    if (left > w || right > w || top > h || bot > h)
+        return (PIX *)ERROR_PTR("border too large", procName, NULL);
+
+        /* Set mirrored pixels on left and right;
+         * then set repeated pixels on top and bottom. */
+    pixd = pixAddBorderGeneral(pixs, left, right, top, bot, 0);
+    for (j = 0; j < left; j++)
+        pixRasterop(pixd, left - 1 - j, top, 1, h, PIX_SRC,
+                    pixd, left + j, top);
+    for (j = 0; j < right; j++)
+        pixRasterop(pixd, left + w + j, top, 1, h, PIX_SRC,
+                    pixd, left + w - 1 - j, top);
+    pixRasterop(pixd, 0, 0, left + w + right, top, PIX_SRC, pixd, 0, h);
+    pixRasterop(pixd, 0, top + h, left + w + right, bot, PIX_SRC, pixd, 0, top);
+
+    return pixd;
+}
+     
 
 
 /*-------------------------------------------------------------*
@@ -1307,6 +1703,11 @@ PIX     *pixd;
  *          images, is left with 0 value.
  *      (2) see Note (4) in pix.h for details on storage of
  *          8-bit samples within each 32-bit word.
+ *      (3) This implementation, setting the r, g and b components
+ *          sequentially, is much faster than setting them in parallel
+ *          by constructing an RGB dest pixel and writing it to dest.
+ *          The reason is there are many more cache misses when reading
+ *          from 3 input images simultaneously.
  */
 PIX *
 pixCreateRGBImage(PIX  *pixr,
@@ -1357,6 +1758,12 @@ PIX     *pixd;
  *  Notes:
  *      (1) The alpha channel (in the 4th byte of each RGB pixel)
  *          is not used in leptonica.
+ *      (2) Three calls to this function generate the three 8 bpp component
+ *          images.  This is much faster than generating the three
+ *          images in parallel, by extracting a src pixel and setting
+ *          the pixels of each component image from it.  The reason is
+ *          there are many more cache misses when writing to three
+ *          output images simultaneously.
  */
 PIX *
 pixGetRGBComponent(PIX     *pixs,
@@ -1770,6 +2177,58 @@ l_uint32   word;
 }
 
 
+/*!
+ *  lineEndianByteSwap()
+ *
+ *      Input   datad (dest byte array data, reordered on little-endians)
+ *              datas (a src line of pix data)
+ *              wpl (number of 32 bit words in the line)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This is used on little-endian platforms to swap
+ *          the bytes within each word in the line of image data.
+ *          Bytes 0 <==> 3 and 1 <==> 2 are swapped in the dest
+ *          byte array data8d, relative to the pix data in datas.
+ *      (2) The bytes represent 8 bit pixel values.  They are swapped
+ *          for little endians so that when the dest array (char *)datad
+ *          is addressed by bytes, the pixels are chosen sequentially
+ *          from left to right in the image.
+ */
+l_int32
+lineEndianByteSwap(l_uint32  *datad,
+                   l_uint32  *datas,
+                   l_int32    wpl)
+{
+l_int32   j;
+l_uint32  word;
+
+    PROCNAME("lineEndianByteSwap");
+
+    if (!datad || !datas)
+        return ERROR_INT("datad and datas not both defined", procName, 1);
+
+#ifdef L_BIG_ENDIAN
+
+    memcpy((char *)datad, (char *)datas, 4 * wpl);
+    return 0;
+
+#else   /* L_LITTLE_ENDIAN */
+
+    for (j = 0; j < wpl; j++, datas++, datad++) {
+        word = *datas;
+        *datad = (word >> 24) |
+                 ((word >> 8) & 0x0000ff00) |
+                 ((word << 8) & 0x00ff0000) |
+                 (word << 24);
+    }
+    return 0;
+
+#endif   /* L_BIG_ENDIAN */
+
+}
+
+
 /*!
  *  pixEndianTwoByteSwapNew()
  *
