@@ -20,16 +20,17 @@
  *
  *      (1) Mask-directed operations
  *      (2) Full-image bit-logical operations
- *      (3) Pixel counting operations
- *      (4) Mirrored tiling of a smaller image
+ *      (3) Foreground pixel counting operations on 1 bpp images
+ *      (4) Sum of pixel values
+ *      (5) Mirrored tiling of a smaller image
  *
  *
  *    Masked operations
  *           l_int32     pixSetMasked()
  *           l_int32     pixSetMaskedGeneral()
  *           l_int32     pixCombineMasked()
+ *           l_int32     pixCombineMaskedGeneral()
  *           l_int32     pixPaintThroughMask()
- *           l_int32     pixCombineThroughMask()
  *           PIX        *pixPaintSelfThroughMask()
  *
  *    One and two-image boolean operations on arbitrary depth images
@@ -39,7 +40,7 @@
  *           PIX        *pixXor()
  *           PIX        *pixSubtract()
  *
- *    Pixel counting
+ *    Foreground pixel counting in 1 bpp images
  *           l_int32     pixZero()
  *           l_int32     pixCountPixels()
  *           NUMA       *pixaCountPixels()
@@ -49,6 +50,9 @@
  *           l_int32    *makePixelSumTab8()
  *           l_int32    *makePixelCentroidTab8()
  *
+ *    Sum of pixel values
+ *           l_int32     pixSumPixelValues()
+ *
  *    Mirrored tiling
  *           PIX        *pixMirroredTiling()
  *
@@ -289,18 +293,24 @@ PIX       *pixmu, *pixc;
 /*!
  *  pixCombineMasked()
  *
- *      Input:  pixd (8 or 32 bpp)
- *              pixs (8 or 32 bpp)
+ *      Input:  pixd (1 bpp, 8 bpp gray or 32 bpp rgb; no cmap)
+ *              pixs (1 bpp, 8 bpp gray or 32 bpp rgb; no cmap)
  *              pixm (<optional> 1 bpp mask; no operation if NULL)
  *      Return: 0 if OK; 1 on error
  *
  *  Notes:
- *      (1) This sets each pixel in pixd that co-locates with an ON
+ *      (1) In-place operation; pixd is changed.
+ *      (2) This sets each pixel in pixd that co-locates with an ON
  *          pixel in pixm to the corresponding value of pixs.
- *      (2) Implementation: for 8 bpp selective masking, you might
- *          think that it would be faster to generate an 8 bpp
- *          version of pixm, using pixConvert1To8(pixm, 0, 255),
- *          and then use a general combine operation
+ *      (3) pixs and pixd must be the same depth and not colormapped.
+ *      (4) All three input pix are aligned at the UL corner, and the
+ *          operation is clipped to the intersection of all three images.
+ *      (5) If pixm == NULL, it's a no-op.
+ *      (6) Implementation: see notes in pixCombineMaskedGeneral().
+ *          For 8 bpp selective masking, you might guess that it
+ *          would be faster to generate an 8 bpp version of pixm,
+ *          using pixConvert1To8(pixm, 0, 255), and then use a
+ *          general combine operation
  *               d = (d & ~m) | (s & m)
  *          on a word-by-word basis.  Not always.  The word-by-word
  *          combine takes a time that is independent of the mask data.
@@ -312,59 +322,193 @@ pixCombineMasked(PIX  *pixd,
                  PIX  *pixs,
                  PIX  *pixm)
 {
-l_uint8    val;
-l_int32    d, w, wd, wm, h, hd, hm, wpld, wpls, wplm, i, j;
-l_uint32  *datad, *datas, *datam, *lined, *lines, *linem;
+l_int32    w, h, d, ws, hs, ds, wm, hm, dm, wmin, hmin;
+l_int32    wpl, wpls, wplm, i, j, val;
+l_uint32  *data, *datas, *datam, *line, *lines, *linem;
+PIX       *pixt;
 
     PROCNAME("pixCombineMasked");
 
-    if (!pixd)
-        return ERROR_INT("pixd not defined", procName, 1);
     if (!pixm)  /* nothing to do */
         return 0;
+    if (!pixd)
+        return ERROR_INT("pixd not defined", procName, 1);
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
-
-    d = pixGetDepth(pixd);
-    if (d != 8 && d != 32)
-        return ERROR_INT("pixd not 8 or 32 bpp", procName, 1);
-    if (pixGetDepth(pixm) != 1)
+    pixGetDimensions(pixd, &w, &h, &d);
+    pixGetDimensions(pixs, &ws, &hs, &ds);
+    pixGetDimensions(pixm, &wm, &hm, &dm);
+    if (d != ds)
+        return ERROR_INT("pixs and pixd depths differ", procName, 1);
+    if (dm != 1)
         return ERROR_INT("pixm not 1 bpp", procName, 1);
-    if (!pixSizesEqual(pixd, pixs))
-        return ERROR_INT("pixs and pixd sizes differ", procName, 1);
+    if (d != 1 && d != 8 && d != 32)
+        return ERROR_INT("pixd not 1, 8 or 32 bpp", procName, 1);
+    if (pixGetColormap(pixd) || pixGetColormap(pixs))
+        return ERROR_INT("pixs and/or pixd is cmapped", procName, 1);
 
-    pixGetDimensions(pixd, &wd, &hd, NULL);
-    pixGetDimensions(pixm, &wm, &hm, NULL);
-    w = L_MIN(wd, wm);
-    h = L_MIN(hd, hm);
-    datad = pixGetData(pixd);
+        /* For d = 1, use rasterop.  pixt is the part from pixs, under
+         * the fg of pixm, that is to be combined with pixd.  We also
+         * use pixt to remove all fg of pixd that is under the fg of pixm.
+         * Then pixt and pixd are combined by ORing. */
+    wmin = L_MIN(w, L_MIN(ws, wm));
+    hmin = L_MIN(h, L_MIN(hs, hm));
+    if (d == 1) {
+        pixt = pixAnd(NULL, pixs, pixm);
+        pixRasterop(pixd, 0, 0, wmin, hmin, PIX_DST & PIX_NOT(PIX_SRC),
+                    pixm, 0, 0);
+        pixRasterop(pixd, 0, 0, wmin, hmin, PIX_SRC | PIX_DST, pixt, 0, 0);
+        pixDestroy(&pixt);
+        return 0;
+    }
+
+    data = pixGetData(pixd);
     datas = pixGetData(pixs);
     datam = pixGetData(pixm);
-    wpld = pixGetWpl(pixd);
+    wpl = pixGetWpl(pixd);
     wpls = pixGetWpl(pixs);
     wplm = pixGetWpl(pixm);
-
     if (d == 8) {
-        for (i = 0; i < h; i++) {
-            lined = datad + i * wpld;
+        for (i = 0; i < hmin; i++) {
+            line = data + i * wpl;
             lines = datas + i * wpls;
             linem = datam + i * wplm;
-            for (j = 0; j < w; j++) {
+            for (j = 0; j < wmin; j++) {
                 if (GET_DATA_BIT(linem, j)) {
                    val = GET_DATA_BYTE(lines, j);
-                   SET_DATA_BYTE(lined, j, val);
+                   SET_DATA_BYTE(line, j, val);
                 }
             }
         }
     }
     else {  /* d == 32 */
-        for (i = 0; i < h; i++) {
-            lined = datad + i * wpld;
+        for (i = 0; i < hmin; i++) {
+            line = data + i * wpl;
             lines = datas + i * wpls;
             linem = datam + i * wplm;
-            for (j = 0; j < w; j++) {
+            for (j = 0; j < wmin; j++) {
                 if (GET_DATA_BIT(linem, j))
-                   lined[j] = lines[j];
+                   line[j] = lines[j];
+            }
+        }
+    }
+
+    return 0;
+}
+
+
+/*!
+ *  pixCombineMaskedGeneral()
+ *
+ *      Input:  pixd (1 bpp, 8 bpp gray or 32 bpp rgb)
+ *              pixs (1 bpp, 8 bpp gray or 32 bpp rgb)
+ *              pixm (<optional> 1 bpp mask)
+ *              x, y (origin of pixs and pixm relative to pixd; can be negative)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) In-place operation; pixd is changed.
+ *      (2) This is a generalized version of pixCombinedMasked(), where
+ *          the source and mask can be placed at the same (arbitrary)
+ *          location relative to pixd.
+ *      (3) pixs and pixd must be the same depth and not colormapped.
+ *      (4) The UL corners of both pixs and pixm are aligned with
+ *          the point (x, y) of pixd, and the operation is clipped to
+ *          the intersection of all three images.
+ *      (5) If pixm == NULL, it's a no-op.
+ *      (6) Implementation.  There are two ways to do these.  In the first,
+ *          we use rasterop, ORing the part of pixs under the mask
+ *          with pixd (which has been appropriately cleared there first).
+ *          In the second, the mask is used one pixel at a time to
+ *          selectively replace pixels of pixd with those of pixs.
+ *          Here, we use rasterop for 1 bpp and pixel-wise replacement
+ *          for 8 and 32 bpp.  To use rasterop for 8 bpp, for example,
+ *          we must first generate an 8 bpp version of the mask.
+ *          The code is simple:
+ *
+ *             Pix *pixm8 = pixConvert1To8(NULL, pixm, 0, 255);
+ *             Pix *pixt = pixAnd(NULL, pixs, pixm8);
+ *             pixRasterop(pixd, x, y, wmin, hmin, PIX_DST & PIX_NOT(PIX_SRC),
+ *                         pixm8, 0, 0);
+ *             pixRasterop(pixd, x, y, wmin, hmin, PIX_SRC | PIX_DST,
+ *                         pixt, 0, 0);
+ *             pixDestroy(&pixt);
+ *             pixDestroy(&pixm8);
+ */
+l_int32
+pixCombineMaskedGeneral(PIX      *pixd,
+                        PIX      *pixs,
+                        PIX      *pixm,
+                        l_int32   x,
+                        l_int32   y)
+{
+l_int32    d, w, h, ws, hs, ds, wm, hm, dm, wmin, hmin;
+l_int32    wpl, wpls, wplm, i, j, val;
+l_uint32  *data, *datas, *datam, *line, *lines, *linem;
+PIX       *pixt;
+
+    PROCNAME("pixCombineMaskedGeneral");
+
+    if (!pixm)  /* nothing to do */
+        return 0;
+    if (!pixd)
+        return ERROR_INT("pixd not defined", procName, 1);
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    pixGetDimensions(pixd, &w, &h, &d);
+    pixGetDimensions(pixs, &ws, &hs, &ds);
+    pixGetDimensions(pixm, &wm, &hm, &dm);
+    if (d != ds)
+        return ERROR_INT("pixs and pixd depths differ", procName, 1);
+    if (dm != 1)
+        return ERROR_INT("pixm not 1 bpp", procName, 1);
+    if (d != 1 && d != 8 && d != 32)
+        return ERROR_INT("pixd not 1, 8 or 32 bpp", procName, 1);
+    if (pixGetColormap(pixd) || pixGetColormap(pixs))
+        return ERROR_INT("pixs and/or pixd is cmapped", procName, 1);
+
+        /* For d = 1, use rasterop.  pixt is the part from pixs, under
+         * the fg of pixm, that is to be combined with pixd.  We also
+         * use pixt to remove all fg of pixd that is under the fg of pixm.
+         * Then pixt and pixd are combined by ORing. */
+    wmin = L_MIN(ws, wm);
+    hmin = L_MIN(hs, hm);
+    if (d == 1) {
+        pixt = pixAnd(NULL, pixs, pixm);
+        pixRasterop(pixd, x, y, wmin, hmin, PIX_DST & PIX_NOT(PIX_SRC),
+                    pixm, 0, 0);
+        pixRasterop(pixd, x, y, wmin, hmin, PIX_SRC | PIX_DST, pixt, 0, 0);
+        pixDestroy(&pixt);
+        return 0;
+    }
+
+    wpl = pixGetWpl(pixd);
+    data = pixGetData(pixd);
+    wpls = pixGetWpl(pixs);
+    datas = pixGetData(pixs);
+    wplm = pixGetWpl(pixm);
+    datam = pixGetData(pixm);
+
+    for (i = 0; i < hmin; i++) {
+        if (y + i < 0 || y + i >= h) continue;
+        line = data + (y + i) * wpl;
+        lines = datas + i * wpls;
+        linem = datam + i * wplm;
+        for (j = 0; j < wmin; j++) {
+            if (x + j < 0 || x + j >= w) continue;
+            if (GET_DATA_BIT(linem, j)) {
+                switch (d)
+                {
+                case 8:
+                    val = GET_DATA_BYTE(lines, j);
+                    SET_DATA_BYTE(line, x + j, val);
+                    break;
+                case 32:
+                    *(line + x + j) = *(lines + j);
+                    break;
+                default:
+                    return ERROR_INT("shouldn't get here", procName, 1);
+                }
             }
         }
     }
@@ -521,94 +665,6 @@ l_uint32  *data, *datam, *line, *linem;
 }
     
 
-/*!
- *  pixCombineThroughMask()
- *
- *      Input:  pixd (8 bpp gray or 32 bpp rgb)
- *              pixs (8 bpp gray or 32 bpp rgb)
- *              pixm (<optional> 1 bpp mask)
- *              x, y (origin of pixs and pixm relative to pixd; can be negative)
- *      Return: 0 if OK; 1 on error
- *
- *  Notes:
- *      (1) In-place operation; pixd is changed.
- *      (2) This is a general version of pixCombinedMasked(), where
- *          the source and mask can be placed at the same (arbitrary)
- *          location relative to pixd.
- *      (3) pixs and pixd must be the same depth and not colormapped.
- *      (4) The UL corners of both pixs and pixm are aligned with
- *          the point (x, y) of pixd, and the operation is clipped to
- *          the intersection of all three images.
- *      (5) If pixm == NULL, it's a no-op.
- */
-l_int32
-pixCombineThroughMask(PIX      *pixd,
-                      PIX      *pixs,
-                      PIX      *pixm,
-                      l_int32   x,
-                      l_int32   y)
-{
-l_int32    d, w, h, ws, hs, ds, wm, hm, dm, wmin, hmin;
-l_int32    wpl, wpls, wplm, i, j, val;
-l_uint32  *data, *datas, *datam, *line, *lines, *linem;
-
-    PROCNAME("pixCombineThroughMask");
-
-    if (!pixm)  /* nothing to do */
-        return 0;
-    if (!pixd)
-        return ERROR_INT("pixd not defined", procName, 1);
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
-    pixGetDimensions(pixd, &w, &h, &d);
-    pixGetDimensions(pixs, &ws, &hs, &ds);
-    pixGetDimensions(pixm, &wm, &hm, &dm);
-    if (d != ds)
-        return ERROR_INT("pixs and pixd depths differ", procName, 1);
-    if (dm != 1)
-        return ERROR_INT("pixm not 1 bpp", procName, 1);
-    if (d != 8 && d != 32)
-        return ERROR_INT("pixd not 8 or 32 bpp", procName, 1);
-    if (pixGetColormap(pixd) || pixGetColormap(pixs))
-        return ERROR_INT("pixs and/or pixd is cmapped", procName, 1);
-
-    wpl = pixGetWpl(pixd);
-    data = pixGetData(pixd);
-    wpls = pixGetWpl(pixs);
-    datas = pixGetData(pixs);
-    wplm = pixGetWpl(pixm);
-    datam = pixGetData(pixm);
-    wmin = L_MIN(ws, wm);
-    hmin = L_MIN(hs, hm);
-
-    for (i = 0; i < hmin; i++) {
-        if (y + i < 0 || y + i >= h) continue;
-        line = data + (y + i) * wpl;
-        lines = datas + i * wpls;
-        linem = datam + i * wplm;
-        for (j = 0; j < wmin; j++) {
-            if (x + j < 0 || x + j >= w) continue;
-            if (GET_DATA_BIT(linem, j)) {
-                switch (d)
-                {
-                case 8:
-                    val = GET_DATA_BYTE(lines, j);
-                    SET_DATA_BYTE(line, x + j, val);
-                    break;
-                case 32:
-                    *(line + x + j) = *(lines + j);
-                    break;
-                default:
-                    return ERROR_INT("shouldn't get here", procName, 1);
-                }
-            }
-        }
-    }
-
-    return 0;
-}
-    
-
 /*!
  *  pixPaintSelfThroughMask()
  *
@@ -726,7 +782,7 @@ PIXA     *pixa;
                          cctilesize, cctilesize);
         pixt = pixClipRectangle(pixd, boxt, NULL);
         pixc = pixMirroredTiling(pixt, bw, bh);
-        pixCombineThroughMask(pixd, pixc, pix, bx, by);
+        pixCombineMaskedGeneral(pixd, pixc, pix, bx, by);
         pixDestroy(&pix);
         pixDestroy(&pixt);
         pixDestroy(&pixc);
@@ -789,8 +845,8 @@ pixInvert(PIX  *pixd,
  *      Input:  pixd  (<optional>; this can be null, equal to pixs1,
  *                     different from pixs1)
  *              pixs1 (can be == pixd)
- *              pixs2
- *      Return: pixd, or null on error
+ *              pixs2 (must be != pixd)
+ *      Return: pixd always
  *
  *  Notes:
  *      (1) This gives the union of two images with equal depth,
@@ -805,7 +861,8 @@ pixInvert(PIX  *pixd,
  *            (b) pixOr(pixs1, pixs1, pixs2);
  *            (c) pixOr(pixd, pixs1, pixs2);
  *      (4) The size of the result is determined by pixs1.
- *      (5) Note carefully that the order of pixs1 and pixs2 only matters
+ *      (5) The depths of pixs1 and pixs2 must be equal.
+ *      (6) Note carefully that the order of pixs1 and pixs2 only matters
  *          for the in-place case.  For in-place, you must have
  *          pixd == pixs1.  Setting pixd == pixs2 gives an incorrect
  *          result: the copy puts pixs1 image data in pixs2, and
@@ -819,9 +876,13 @@ pixOr(PIX  *pixd,
     PROCNAME("pixOr");
 
     if (!pixs1)
-        return (PIX *)ERROR_PTR("pixs1 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs1 not defined", procName, pixd);
     if (!pixs2)
-        return (PIX *)ERROR_PTR("pixs2 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs2 not defined", procName, pixd);
+    if (pixd == pixs2)
+        return (PIX *)ERROR_PTR("cannot have pixs2 == pixd", procName, pixd);
+    if (pixGetDepth(pixs1) != pixGetDepth(pixs2))
+        return (PIX *)ERROR_PTR("depths of pixs* unequal", procName, pixd);
 
 #if  EQUAL_SIZE_WARNING
     if (!pixSizesEqual(pixs1, pixs2))
@@ -830,7 +891,7 @@ pixOr(PIX  *pixd,
 
         /* Prepare pixd to be a copy of pixs1 */
     if ((pixd = pixCopy(pixd, pixs1)) == NULL)
-        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+        return (PIX *)ERROR_PTR("pixd not made", procName, pixd);
 
         /* src1 | src2 --> dest */
     pixRasterop(pixd, 0, 0, pixGetWidth(pixd), pixGetHeight(pixd),
@@ -846,8 +907,8 @@ pixOr(PIX  *pixd,
  *      Input:  pixd  (<optional>; this can be null, equal to pixs1,
  *                     different from pixs1)
  *              pixs1 (can be == pixd)
- *              pixs2
- *      Return: pixd, or null on error
+ *              pixs2 (must be != pixd)
+ *      Return: pixd always
  *
  *  Notes:
  *      (1) This gives the intersection of two images with equal depth,
@@ -862,7 +923,8 @@ pixOr(PIX  *pixd,
  *            (b) pixAnd(pixs1, pixs1, pixs2);
  *            (c) pixAnd(pixd, pixs1, pixs2);
  *      (4) The size of the result is determined by pixs1.
- *      (5) Note carefully that the order of pixs1 and pixs2 only matters
+ *      (5) The depths of pixs1 and pixs2 must be equal.
+ *      (6) Note carefully that the order of pixs1 and pixs2 only matters
  *          for the in-place case.  For in-place, you must have
  *          pixd == pixs1.  Setting pixd == pixs2 gives an incorrect
  *          result: the copy puts pixs1 image data in pixs2, and
@@ -876,9 +938,13 @@ pixAnd(PIX  *pixd,
     PROCNAME("pixAnd");
 
     if (!pixs1)
-        return (PIX *)ERROR_PTR("pixs1 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs1 not defined", procName, pixd);
     if (!pixs2)
-        return (PIX *)ERROR_PTR("pixs2 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs2 not defined", procName, pixd);
+    if (pixd == pixs2)
+        return (PIX *)ERROR_PTR("cannot have pixs2 == pixd", procName, pixd);
+    if (pixGetDepth(pixs1) != pixGetDepth(pixs2))
+        return (PIX *)ERROR_PTR("depths of pixs* unequal", procName, pixd);
 
 #if  EQUAL_SIZE_WARNING
     if (!pixSizesEqual(pixs1, pixs2))
@@ -887,7 +953,7 @@ pixAnd(PIX  *pixd,
 
         /* Prepare pixd to be a copy of pixs1 */
     if ((pixd = pixCopy(pixd, pixs1)) == NULL)
-        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+        return (PIX *)ERROR_PTR("pixd not made", procName, pixd);
 
         /* src1 & src2 --> dest */
     pixRasterop(pixd, 0, 0, pixGetWidth(pixd), pixGetHeight(pixd),
@@ -903,8 +969,8 @@ pixAnd(PIX  *pixd,
  *      Input:  pixd  (<optional>; this can be null, equal to pixs1,
  *                     different from pixs1)
  *              pixs1 (can be == pixd)
- *              pixs2
- *      Return: pixd, or null on error
+ *              pixs2 (must be != pixd)
+ *      Return: pixd always
  *
  *  Notes:
  *      (1) This gives the XOR of two images with equal depth,
@@ -919,7 +985,8 @@ pixAnd(PIX  *pixd,
  *            (b) pixXor(pixs1, pixs1, pixs2);
  *            (c) pixXor(pixd, pixs1, pixs2);
  *      (4) The size of the result is determined by pixs1.
- *      (5) Note carefully that the order of pixs1 and pixs2 only matters
+ *      (5) The depths of pixs1 and pixs2 must be equal.
+ *      (6) Note carefully that the order of pixs1 and pixs2 only matters
  *          for the in-place case.  For in-place, you must have
  *          pixd == pixs1.  Setting pixd == pixs2 gives an incorrect
  *          result: the copy puts pixs1 image data in pixs2, and
@@ -933,9 +1000,13 @@ pixXor(PIX  *pixd,
     PROCNAME("pixXor");
 
     if (!pixs1)
-        return (PIX *)ERROR_PTR("pixs1 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs1 not defined", procName, pixd);
     if (!pixs2)
-        return (PIX *)ERROR_PTR("pixs2 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs2 not defined", procName, pixd);
+    if (pixd == pixs2)
+        return (PIX *)ERROR_PTR("cannot have pixs2 == pixd", procName, pixd);
+    if (pixGetDepth(pixs1) != pixGetDepth(pixs2))
+        return (PIX *)ERROR_PTR("depths of pixs* unequal", procName, pixd);
 
 #if  EQUAL_SIZE_WARNING
     if (!pixSizesEqual(pixs1, pixs2))
@@ -944,7 +1015,7 @@ pixXor(PIX  *pixd,
 
         /* Prepare pixd to be a copy of pixs1 */
     if ((pixd = pixCopy(pixd, pixs1)) == NULL)
-        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+        return (PIX *)ERROR_PTR("pixd not made", procName, pixd);
 
         /* src1 ^ src2 --> dest */
     pixRasterop(pixd, 0, 0, pixGetWidth(pixd), pixGetHeight(pixd),
@@ -961,7 +1032,7 @@ pixXor(PIX  *pixd,
  *                     equal to pixs2, or different from both pixs1 and pixs2)
  *              pixs1 (can be == pixd)
  *              pixs2 (can be == pixd)
- *      Return: pixd, or null on error
+ *      Return: pixd always
  *
  *  Notes:
  *      (1) This gives the set subtraction of two images with equal depth,
@@ -982,6 +1053,7 @@ pixXor(PIX  *pixd,
  *            (c) pixSubtract(pixs2, pixs1, pixs2);
  *            (d) pixSubtract(pixd, pixs1, pixs2);
  *      (5) The size of the result is determined by pixs1.
+ *      (6) The depths of pixs1 and pixs2 must be equal.
  */
 PIX *
 pixSubtract(PIX  *pixd,
@@ -993,9 +1065,11 @@ l_int32  w, h;
     PROCNAME("pixSubtract");
 
     if (!pixs1)
-        return (PIX *)ERROR_PTR("pixs1 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs1 not defined", procName, pixd);
     if (!pixs2)
-        return (PIX *)ERROR_PTR("pixs2 not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("pixs2 not defined", procName, pixd);
+    if (pixGetDepth(pixs1) != pixGetDepth(pixs2))
+        return (PIX *)ERROR_PTR("depths of pixs* unequal", procName, pixd);
 
 #if  EQUAL_SIZE_WARNING
     if (!pixSizesEqual(pixs1, pixs2))
@@ -1052,21 +1126,20 @@ l_uint32  *data, *line;
 
     PROCNAME("pixZero");
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
     if (!pempty)
         return ERROR_INT("pempty not defined", procName, 1);
+    *pempty = 1;
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
 
     w = pixGetWidth(pix) * pixGetDepth(pix);
     h = pixGetHeight(pix);
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
-
     fullwords = w / 32;
     endbits = w & 31;
     endmask = 0xffffffff << (32 - endbits);
 
-    *pempty = 1;
     for (i = 0; i < h; i++) {
         line = data + wpl * i;
         for (j = 0; j < fullwords; j++)
@@ -1107,25 +1180,20 @@ l_uint32  *data;
 
     PROCNAME("pixCountPixels");
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
     if (!pcount)
         return ERROR_INT("pcount not defined", procName, 1);
-    if (pixGetDepth(pix) != 1)
-        return ERROR_INT("pix not 1 bpp", procName, 1);
-
     *pcount = 0;
+    if (!pix || pixGetDepth(pix) != 1)
+        return ERROR_INT("pix not defined or not 1 bpp", procName, 1);
 
     if (!tab8)
         tab = makePixelSumTab8();
     else
         tab = tab8;
 
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    pixGetDimensions(pix, &w, &h, NULL);
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
-
     fullwords = w >> 5;
     endbits = w & 31;
     endmask = 0xffffffff << (32 - endbits);
@@ -1225,22 +1293,17 @@ l_uint32  *line;
 
     PROCNAME("pixCountPixelsInRow");
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
     if (!pcount)
         return ERROR_INT("pcount not defined", procName, 1);
-    if (pixGetDepth(pix) != 1)
-        return ERROR_INT("pix not 1 bpp", procName, 1);
-
     *pcount = 0;
+    if (!pix || pixGetDepth(pix) != 1)
+        return ERROR_INT("pix not defined or not 1 bpp", procName, 1);
 
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    pixGetDimensions(pix, &w, &h, NULL);
     if (row < 0 || row >= h)
         return ERROR_INT("row out of bounds", procName, 1);
     wpl = pixGetWpl(pix);
     line = pixGetData(pix) + row * wpl;
-
     fullwords = w >> 5;
     endbits = w & 31;
     endmask = 0xffffffff << (32 - endbits);
@@ -1288,19 +1351,14 @@ NUMA *
 pixCountPixelsByRow(PIX      *pix,
                     l_int32  *tab8)
 {
-l_int32   w, h, i, count;
+l_int32   h, i, count;
 l_int32  *tab;
 NUMA     *na;
 
     PROCNAME("pixCountPixelsByRow");
 
-    if (!pix)
-        return (NUMA *)ERROR_PTR("pix not defined", procName, NULL);
-    if (pixGetDepth(pix) != 1)
-        return (NUMA *)ERROR_PTR("pix not 1 bpp", procName, NULL);
-
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    if (!pix || pixGetDepth(pix) != 1)
+        return (NUMA *)ERROR_PTR("pix undefined or not 1 bpp", procName, NULL);
 
     if (!tab8)
         tab = makePixelSumTab8();
@@ -1310,6 +1368,7 @@ NUMA     *na;
     if ((na = numaCreate(h)) == NULL)
         return (NUMA *)ERROR_PTR("na not made", procName, NULL);
 
+    h = pixGetHeight(pix);
     for (i = 0; i < h; i++) {
         pixCountPixelsInRow(pix, i, &count, tab);
         numaAddNumber(na, count);
@@ -1353,24 +1412,20 @@ l_uint32  *line, *data;
 
     PROCNAME("pixThresholdPixels");
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
-    if (pixGetDepth(pix) != 1)
-        return ERROR_INT("pix not 1 bpp", procName, 1);
     if (!pabove)
         return ERROR_INT("pabove not defined", procName, 1);
-    *pabove = 0;  /* init */
+    *pabove = 0;
+    if (!pix || pixGetDepth(pix) != 1)
+        return ERROR_INT("pix not defined or not 1 bpp", procName, 1);
 
     if (!tab8)
         tab = makePixelSumTab8();
     else
         tab = tab8;
 
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    pixGetDimensions(pix, &w, &h, NULL);
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
-
     fullwords = w >> 5;
     endbits = w & 31;
     endmask = 0xffffffff << (32 - endbits);
@@ -1503,6 +1558,83 @@ l_int32  *tab;
 }
 
 
+/*-------------------------------------------------------------*
+ *                       Sum of pixel values                   *
+ *-------------------------------------------------------------*/
+/*!
+ *  pixSumPixelValues()
+ *
+ *      Input:  pix (1, 2, 4, 8, 16, 32 bpp; not cmapped)
+ *              box (<optional> if null, use entire image)
+ *              &sum (<return> sum of pixel values in region)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixSumPixelValues(PIX        *pix,
+                  BOX        *box,
+                  l_float64  *psum)
+{
+l_int32    w, h, d, wpl, i, j, xstart, xend, ystart, yend, bw, bh;
+l_uint32  *data, *line;
+l_float64  sum;
+BOX       *boxc;
+
+    PROCNAME("pixSumPixelValues");
+
+    if (!psum)
+        return ERROR_INT("psum not defined", procName, 1);
+    *psum = 0;
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (pixGetColormap(pix) != NULL)
+        return ERROR_INT("pix is colormapped", procName, 1);
+    pixGetDimensions(pix, &w, &h, &d);
+    if (d != 1 && d != 2 && d != 4 && d != 8 && d != 16 && d != 32)
+        return ERROR_INT("pix not 1, 2, 4, 8, 16 or 32 bpp", procName, 1);
+
+    wpl = pixGetWpl(pix);
+    data = pixGetData(pix);
+    boxc = NULL;
+    if (box) {
+        boxc = boxClipToRectangle(box, w, h);
+        if (!boxc)
+            return ERROR_INT("box outside image", procName, 1);
+    }
+    xstart = ystart = 0;
+    xend = w;
+    yend = h;
+    if (boxc) {
+        boxGetGeometry(boxc, &xstart, &ystart, &bw, &bh);
+        xend = xstart + bw;  /* 1 past the end */
+        yend = ystart + bh;  /* 1 past the end */
+        boxDestroy(&boxc);
+    }
+
+    sum = 0;
+    for (i = ystart; i < yend; i++) {
+        line = data + i * wpl;
+        for (j = xstart; j < xend; j++) {
+            if (d == 1)
+                sum += GET_DATA_BIT(line, j);
+            else if (d == 2)
+                sum += GET_DATA_DIBIT(line, j);
+            else if (d == 4)
+                sum += GET_DATA_QBIT(line, j);
+            else if (d == 8)
+                sum += GET_DATA_BYTE(line, j);
+            else if (d == 16)
+                sum += GET_DATA_TWO_BYTES(line, j);
+            else if (d == 32)
+                sum += line[j];
+        }
+    }
+    *psum = sum;
+
+    return 0;
+}
+
+
+
 /*-------------------------------------------------------------*
  *              Mirrored tiling of a smaller image             *
  *-------------------------------------------------------------*/
