@@ -26,6 +26,11 @@
  *          l_int32     pixWritePng()  [ special top level ]
  *          l_int32     pixWriteStreamPng()
  *          
+ *    Setting flags for special modes
+ *          void        l_pngSetStrip16To8()
+ *          void        l_pngSetStripAlpha()
+ *          void        l_pngSetZlibCompression()
+ *
  *    Read/write to memory   [not on windows]
  *          PIX        *pixReadMemPng()
  *          l_int32     pixWriteMemPng()
@@ -40,18 +45,28 @@
  *    full color images are written compressed as a 24 bpp,
  *    3 component color image.
  *
- *    Note these particular limitations, which are invoked by
- *    two transform flags in pixReadStreamPng():
- *
+ *    In the following, we use these abbreviations:
  *       bpc == bit/component
  *       cpp == component/pixel
  *       bpp == bits/pixel of image in Pix (memory)
  *
- *    (1) 16 bpc input images are read into a Pix with 8 bpc.
- *        - For 16 bpc rgb (16 bpc, 3 cpp) --> 32 bpp rgb Pix
- *        - For 16 bpc gray (16 bpc, 1 cpp) --> 8 bpp grayscale Pix
- *    (2) The alpha layer is stripped out
- *        - For 8 bpc rgba (8 bpc, 4 cpp) --> 32 bpp rgb Pix
+ *    Flags can be set for special reading from png compression
+ *    into a pix, with either 16 bpc or 32 bpp with alpha channel.
+ *    Otherwise, the default reading values are as follows:
+ *      (1) 16 bpc input images are read into a Pix with 8 bpc.
+ *         - For 16 bpc rgb (16 bpc, 3 cpp) --> 32 bpp rgb Pix
+ *         - For 16 bpc gray (16 bpc, 1 cpp) --> 8 bpp grayscale Pix
+ *      (2) The alpha layer is stripped out
+ *         - For 8 bpc rgba (8 bpc, 4 cpp) --> 32 bpp rgb Pix
+ *
+ *    The zlib compression value can be set [0 ... 9], with
+ *         0     no compression (huge files)
+ *         1     fastest compression
+ *         6     default compression
+ *         9     best compression
+ *    If not set, we use the default compression in zlib.
+ *    Note that if you are using the defined constants in zlib instead
+ *    of the compression integers given above, you must include zlib.h.
  */
 
 #include <stdio.h>
@@ -69,6 +84,15 @@
 
 #include "png.h"
 
+/* ----------------Set defaults for read/write options ----------------- */
+    /* strip 16 bpp --> 8 bpp on reading png; default is for stripping */
+static l_int32   L_PNG_STRIP_16_TO_8 = 1;
+    /* strip alpha on reading png; default is for stripping */
+static l_int32   L_PNG_STRIP_ALPHA = 1;
+    /* zlib compression in png; default (5) is for standard compression */
+static l_int32   L_ZLIB_COMPRESSION = Z_DEFAULT_COMPRESSION;
+
+
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG     0
 #endif  /* ~NO_CONSOLE_IO */
@@ -139,17 +163,20 @@ PIXCMAP     *cmap;
 
     png_init_io(png_ptr, fp);
 
-        /* Set the transforms flags.  Whatever you do here,
-         * DO NOT invert binary using PNG_TRANSFORM_INVERT_MONO!!
-         * To remove alpha channel, use PNG_TRANSFORM_STRIP_ALPHA
-         * To strip 16 --> 8 bit depth, use PNG_TRANSFORM_STRIP_16 */
-#if 1  /* this does both */
-    png_transforms = PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_STRIP_ALPHA;
-#else  /* this just strips alpha */
-    png_transforms = PNG_TRANSFORM_STRIP_ALPHA;
-#endif
+        /* ---------------------------------------------------------- *
+         *  Set the transforms flags.  Whatever happens here,
+         *  NEVER invert 1 bpp using PNG_TRANSFORM_INVERT_MONO.
+         * ---------------------------------------------------------- */
+        /* To strip 16 --> 8 bit depth, use PNG_TRANSFORM_STRIP_16 */
+    if (L_PNG_STRIP_16_TO_8 == 1)   /* our default */
+        png_transforms = PNG_TRANSFORM_STRIP_16;
+    else
+        png_transforms = PNG_TRANSFORM_IDENTITY;
+        /* To remove alpha channel, use PNG_TRANSFORM_STRIP_ALPHA */
+    if (L_PNG_STRIP_ALPHA == 1)   /* our default */
+        png_transforms |= PNG_TRANSFORM_STRIP_ALPHA;
 
-        /* Do it! */
+        /* Read it */
     png_read_png(png_ptr, info_ptr, png_transforms, NULL);
 
     row_pointers = png_get_rows(png_ptr, info_ptr);
@@ -520,12 +547,24 @@ FILE  *fp;
  *              > 0.4545, the image is rendered lighter than baseline
  *              < 0.4545, the image is rendered darker than baseline
  *          In contrast, gqview seems to ignore the gamma chunk in png.
+ *      (7) The only valid pixel depths in leptonica are 1, 2, 4, 8, 16
+ *          and 32.  However, it is possible, and in some cases desirable,
+ *          to write out a png file using an rgb pix that has 24 bpp.
+ *          For example, the open source xpdf SplashBitmap class generates
+ *          24 bpp rgb images.  To write these directly to file in leptonica,
+ *          you can make a 24 bpp pix without data and assign the data array
+ *          to the pix; e.g.,
+ *              pix = pixCreateHeader(w, h, 24);
+ *              pixSetData(pix, rgbdata);
+ *              pixSetPadBits(pix, 0);
+ *          Consequently, we enable writing 24 bpp pix.
  */
 l_int32
 pixWriteStreamPng(FILE      *fp,
                   PIX       *pix,
                   l_float32  gamma)
 {
+char         commentstring[] = "Comment";
 l_int32      i, j, k;
 l_int32      wpl, d, cmflag;
 l_int32      ncolors;
@@ -568,6 +607,11 @@ char        *text;
 
     png_init_io(png_ptr, fp);
 
+        /* With best zlib compression (9), get between 1 and 10% improvement
+         * over default (5), but the compression is 3 to 10 times slower.
+         * Our default compression is the zlib default (5). */
+    png_set_compression_level(png_ptr, L_ZLIB_COMPRESSION);
+
     w = pixGetWidth(pix);
     h = pixGetHeight(pix);
     d = pixGetDepth(pix);
@@ -575,7 +619,7 @@ char        *text;
         cmflag = 1;
     else
         cmflag = 0;
-    if (d == 32) {
+    if ((d == 32) || (d == 24)) {
         bit_depth = 8;
         color_type = PNG_COLOR_TYPE_RGB;
         cmflag = 0;  /* ignore if it exists */
@@ -634,7 +678,7 @@ char        *text;
     if ((text = pixGetText(pix))) {
         png_text text_chunk;
         text_chunk.compression = PNG_TEXT_COMPRESSION_NONE;
-        text_chunk.key = "Comment";
+        text_chunk.key = commentstring;
         text_chunk.text = text;
         text_chunk.text_length = strlen(text);
 #ifdef PNG_ITXT_SUPPORTED
@@ -648,7 +692,7 @@ char        *text;
         /* Write header and palette info */
     png_write_info(png_ptr, info_ptr);
 
-    if (d != 32) {  /* not 24 bit color */
+    if ((d != 32) && (d != 24)) {  /* not rgb color */
             /* Generate a temporary pix with bytes swapped.
              * For a binary image, there are two conditions in
              * which you must first invert the data for writing png:
@@ -692,39 +736,112 @@ char        *text;
         return 0;
     }
 
-        /* 24 bit color; write a row at a time */
-    if ((rowbuffer = (png_bytep)CALLOC(w, 3)) == NULL)
-        return ERROR_INT("rowbuffer not made", procName, 1);
+        /* For rgb, compose and write a row at a time */
     data = pixGetData(pix);
     wpl = pixGetWpl(pix);
-    for (i = 0; i < h; i++) {
-        ppixel = data + i * wpl;
-        for (j = k = 0; j < w; j++) {
-            rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_RED);
-            rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_GREEN);
-            rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_BLUE);
-            ppixel++;
+    if (d == 24) {  /* See note 7 above: special case of 24 bpp rgb */
+        for (i = 0; i < h; i++) {
+            ppixel = data + i * wpl;
+            png_write_rows(png_ptr, (png_bytepp)&ppixel, 1);
         }
+    }
+    else {  /* standard 32 bpp rgb */
+        if ((rowbuffer = (png_bytep)CALLOC(w, 3)) == NULL)
+            return ERROR_INT("rowbuffer not made", procName, 1);
+        for (i = 0; i < h; i++) {
+            ppixel = data + i * wpl;
+            for (j = k = 0; j < w; j++) {
+                rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_RED);
+                rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_GREEN);
+                rowbuffer[k++] = GET_DATA_BYTE(ppixel, COLOR_BLUE);
+                ppixel++;
+            }
 
-        png_write_rows(png_ptr, &rowbuffer, 1);
+            png_write_rows(png_ptr, &rowbuffer, 1);
+        }
+        FREE(rowbuffer);
     }
 
     png_write_end(png_ptr, info_ptr);
 
     if (cmflag)
         FREE(palette);
-    FREE(rowbuffer);
     png_destroy_write_struct(&png_ptr, &info_ptr);
     return 0;
 
 }
 
 
+/*---------------------------------------------------------------------*
+ *                   Setting flags for special modes                   *
+ *---------------------------------------------------------------------*/
+/*!
+ *  l_pngSetStrip16To8()
+ *
+ *      Input:  flag (1 for stripping 16 bpp to 8 bpp; 0 for leaving 16 bpp)
+ *      Return: void
+ */
+void
+l_pngSetStrip16To8(l_int32  flag)
+{
+    if (flag == 1)
+        L_PNG_STRIP_16_TO_8 = 1;
+    else
+        L_PNG_STRIP_16_TO_8 = 0;
+}
+
+
+/*!
+ *  l_pngSetStripAlpha()
+ *
+ *      Input:  flag (1 for stripping alpha channel on read; 0 for leaving it)
+ *      Return: void
+ */
+void
+l_pngSetStripAlpha(l_int32  flag)
+{
+    if (flag == 1)
+        L_PNG_STRIP_ALPHA = 1;
+    else
+        L_PNG_STRIP_ALPHA = 0;
+}
+
+
+/*!
+ *  l_pngSetZlibCompression()
+ *
+ *      Input:  val (zlib compression value)
+ *      Return: void
+ *
+ *  Notes:
+ *      (1) Valid zlib compression values are in the interval [0 ... 9],
+ *          where, as defined in zlib.h:
+ *            0         Z_NO_COMPRESSION
+ *            1         Z_BEST_SPEED    (poorest compression)
+ *            9         Z_BEST_COMPRESSION
+ *          For the default value, use either of these:
+ *            6         Z_DEFAULT_COMPRESSION
+ *           -1         (resolves to Z_DEFAULT_COMPRESSION)
+ *      (2) If you use the defined constants in zlib.h instead of the
+ *          compression integers given above, you must include zlib.h.
+ */
+void
+l_pngSetZlibCompression(l_int32  val)
+{
+    PROCNAME("l_pngSetZlibCompression");
+    if (val < -1 || val > 9) {
+        L_ERROR("Invalid zlib comp val; using default", procName);
+        val = Z_DEFAULT_COMPRESSION;
+    }
+    L_ZLIB_COMPRESSION = val;
+}
+
+
+
 /*---------------------------------------------------------------------*
  *                         Read/write to memory                        *
  *---------------------------------------------------------------------*/
-#if HAVE_FMEMOPEN || \
- (!defined(__MINGW32__) && !defined(_CYGWIN_ENVIRON) && !defined(_STANDARD_C_))
+#if HAVE_FMEMOPEN
 
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
@@ -822,7 +939,7 @@ pixWriteMemPng(l_uint8  **pdata,
         "pixWriteMemPng", 1);
 }
 
-#endif  /* HAVE_FMEMOPEN || (!defined(__MINGW32__) && etc ) */
+#endif  /* HAVE_FMEMOPEN */
 
 /* --------------------------------------------*/
 #endif  /* HAVE_LIBPNG */
