@@ -95,6 +95,7 @@ PIX *
 pixReadStreamPnm(FILE  *fp)
 {
 l_uint8    val8, rval8, gval8, bval8;
+l_uint16   val16;
 l_int32    w, h, d, bpl, wpl, i, j, type;
 l_int32    maxval, val, rval, gval, bval;
 l_uint32   rgbval;
@@ -171,8 +172,8 @@ PIX       *pix;
         return pix;
     }
 
-        /* "raw" format; binary and grayscale */
-    if (type == 4 || type == 5) {
+        /* "raw" format for 1 bpp */
+    if (type == 4) {
         bpl = (d * w + 7) / 8;
         for (i = 0; i < h; i++) {
             line = data + i * wpl;
@@ -184,6 +185,32 @@ PIX       *pix;
         return pix;
     }
 
+        /* "raw" format for grayscale */
+    if (type == 5) {
+        bpl = (d * w + 7) / 8;
+        for (i = 0; i < h; i++) {
+            line = data + i * wpl;
+            if (d != 16) {
+                for (j = 0; j < w; j++) {
+                    fread(&val8, 1, 1, fp);
+                    if (d == 2)
+                        SET_DATA_DIBIT(line, j, val8);
+                    else if (d == 4)
+                        SET_DATA_QBIT(line, j, val8);
+                    else  /* d == 8 */
+                        SET_DATA_BYTE(line, j, val8);
+                }
+            }
+            else {  /* d == 16 */
+                for (j = 0; j < w; j++) {
+                    fread(&val16, 2, 1, fp);
+                    SET_DATA_TWO_BYTES(line, j, val16);
+                }
+            }
+        }
+        return pix;
+    }
+
         /* "raw" format, type == 6; rgb */
     for (i = 0; i < h; i++) {
         line = data + i * wpl;
@@ -206,18 +233,23 @@ PIX       *pix;
  *              pix
  *      Return: 0 if OK; 1 on error
  *
- *  Writes "raw" packed format only:
- *      1 bpp --> pbm (P4)
- *      2, 4, 8, 16 bpp, no colormap or grayscale colormap --> pgm (P5)
- *      2, 4, 8 bpp with color-valued colormap, or rgb --> rgb ppm (P6)
+ *  Notes:
+ *      (1) This writes "raw" packed format only:
+ *          1 bpp --> pbm (P4)
+ *          2, 4, 8, 16 bpp, no colormap or grayscale colormap --> pgm (P5)
+ *          2, 4, 8 bpp with color-valued colormap, or rgb --> rgb ppm (P6)
+ *      (2) 24 bpp rgb are not supported in leptonica, but this will
+ *          write them out as a packed array of bytes (3 to a pixel).
  */
 l_int32
 pixWriteStreamPnm(FILE  *fp,
                   PIX   *pix)
 {
-l_uint8    byteval, rval, gval, bval;
-l_int32    h, w, d, ds, i, j, wpls, bpl, maxval;
-l_uint32  *datas, *lines;
+l_uint8    val8;
+l_uint8    pel[4];
+l_uint16   val16;
+l_int32    h, w, d, ds, i, j, wpls, bpl, filebpl, writeerror, maxval;
+l_uint32  *pword, *datas, *lines;
 PIX       *pixs;
 
     PROCNAME("pixWriteStreamPnm");
@@ -228,8 +260,8 @@ PIX       *pixs;
         return ERROR_INT("pix not defined", procName, 1);
 
     pixGetDimensions(pix, &w, &h, &d);
-    if (d != 1 && d != 2 && d != 4 && d != 8 && d != 16 && d != 32)
-        return ERROR_INT("d not in {1,2,4,8,16,32}", procName, 1);
+    if (d != 1 && d != 2 && d != 4 && d != 8 && d != 16 && d != 24 && d != 32)
+        return ERROR_INT("d not in {1,2,4,8,16,24,32}", procName, 1);
 
         /* If a colormap exists, remove and convert to grayscale or rgb */
     if (pixGetColormap(pix) != NULL)
@@ -240,6 +272,7 @@ PIX       *pixs;
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
 
+    writeerror = 0;
     if (ds == 1) {  /* binary */
         fprintf(fp, "P4\n# Raw PBM file written by leptonlib (www.leptonica.com)\n%d %d\n", w, h);
 
@@ -247,8 +280,8 @@ PIX       *pixs;
         for (i = 0; i < h; i++) {
             lines = datas + i * wpls;
             for (j = 0; j < bpl; j++) {
-                byteval = GET_DATA_BYTE(lines, j);
-                fwrite(&byteval, 1, 1, fp);
+                val8 = GET_DATA_BYTE(lines, j);
+                fwrite(&val8, 1, 1, fp);
             }
         }
     }
@@ -256,32 +289,59 @@ PIX       *pixs;
         maxval = (1 << ds) - 1;
         fprintf(fp, "P5\n# Raw PGM file written by leptonlib (www.leptonica.com)\n%d %d\n%d\n", w, h, maxval);
 
-        bpl = (ds * w + 7) / 8;
-        for (i = 0; i < h; i++) {
-            lines = datas + i * wpls;
-            for (j = 0; j < bpl; j++) {
-                byteval = GET_DATA_BYTE(lines, j);
-                fwrite(&byteval, 1, 1, fp);
+        if (ds != 16) {
+            for (i = 0; i < h; i++) {
+                lines = datas + i * wpls;
+                for (j = 0; j < w; j++) {
+                    if (ds == 2)
+                        val8 = GET_DATA_DIBIT(lines, j);
+                    else if (ds == 4)
+                        val8 = GET_DATA_QBIT(lines, j);
+                    else  /* ds == 8 */
+                        val8 = GET_DATA_BYTE(lines, j);
+                    fwrite(&val8, 1, 1, fp);
+                }
+            }
+        }
+        else {  /* ds == 16 */
+            for (i = 0; i < h; i++) {
+                lines = datas + i * wpls;
+                for (j = 0; j < w; j++) {
+                    val16 = GET_DATA_TWO_BYTES(lines, j);
+                    fwrite(&val16, 2, 1, fp);
+                }
             }
         }
     }
     else {  /* rgb color */
         fprintf(fp, "P6\n# Raw PPM file written by leptonlib (www.leptonica.com)\n%d %d\n255\n", w, h);
 
-        for (i = 0; i < h; i++) {
-            lines = datas + i * wpls;
-            for (j = 0; j < wpls; j++) {
-                rval = GET_DATA_BYTE(lines + j, COLOR_RED);
-                gval = GET_DATA_BYTE(lines + j, COLOR_GREEN);
-                bval = GET_DATA_BYTE(lines + j, COLOR_BLUE);
-                fwrite(&rval, 1, 1, fp);
-                fwrite(&gval, 1, 1, fp);
-                fwrite(&bval, 1, 1, fp);
+        if (d == 24) {   /* packed, 3 bytes to a pixel */
+            filebpl = 3 * w;
+            for (i = 0; i < h; i++) {  /* write out each raster line */
+                lines = datas + i * wpls;
+                if (fwrite(lines, 1, filebpl, fp) != filebpl)
+                    writeerror = 1;
+            }
+        }
+        else {  /* 32 bpp rgb */
+            for (i = 0; i < h; i++) {
+                lines = datas + i * wpls;
+                for (j = 0; j < wpls; j++) {
+                    pword = lines + j;
+                    pel[0] = *((l_uint8 *)pword + 3);   /* red   */
+                    pel[1] = *((l_uint8 *)pword + 2);   /* green */
+                    pel[2] = *((l_uint8 *)pword + 1);   /* blue  */
+                    if (fwrite(&pel, 1, 3, fp) != 3)
+                        writeerror = 1;
+                }
             }
         }
     }
 
     pixDestroy(&pixs);
+    if (writeerror)
+        return ERROR_INT("image write fail", procName, 1);
     return 0;
 }
 
@@ -414,8 +474,7 @@ PIX       *pixs;
 #include "config_auto.h"
 #endif  /* HAVE_CONFIG_H */
 
-#if HAVE_FMEMOPEN || \
-  (!defined(__MINGW32__) && !defined(_CYGWIN_ENVIRON) && !defined(_STANDARD_C_))
+#if HAVE_FMEMOPEN
 
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
@@ -510,7 +569,7 @@ pixWriteMemPnm(l_uint8  **pdata,
         "pixWriteMemPnm", 1);
 }
 
-#endif  /* HAVE_FMEMOPEN || (!defined(__MINGW32__) && etc ) */
+#endif  /* HAVE_FMEMOPEN */
 
 
 /*--------------------------------------------------------------------*
@@ -587,4 +646,3 @@ l_int32  c;
 /* --------------------------------------------*/
 #endif  /* USE_PNMIO */
 /* --------------------------------------------*/
-
