@@ -16,23 +16,25 @@
 /*
  *  projective.c
  *
- *      Projective (4-pt) image transformation using a sampled
- *      (to nearest integer) transform on each point
+ *      Projective (4 pt) image transformation using a sampled
+ *      (to nearest integer) transform on each dest point
+ *           PIX      *pixProjectiveSampledPta()
  *           PIX      *pixProjectiveSampled()
  *
- *      Projective (4-pt) image transformation using interpolation
+ *      Projective (4 pt) image transformation using interpolation 
  *      (or area mapping) for anti-aliasing images that are
  *      2, 4, or 8 bpp gray, or colormapped, or 32 bpp RGB
- *           PIX      *pixProjectiveInterpolated()
- *           PIX      *pixProjectiveInterpolatedColor()
- *           PIX      *pixProjectiveInterpolatedGray()
- *           void      projectiveInterpolatedColorLow()
- *           void      projectiveInterpolatedGrayLow()
+ *           PIX      *pixProjectivePta()
+ *           PIX      *pixProjective()
+ *           PIX      *pixProjectivePtaColor()
+ *           PIX      *pixProjectiveColor()
+ *           PIX      *pixProjectivePtaGray()
+ *           PIX      *pixProjectiveGray()
  *
  *      Projective coordinate transformation
- *           l_int32   projectiveXformCoeffs()
- *           l_int32   projectiveXformSampled()
- *           l_int32   projectiveXformInterpolated()
+ *           l_int32   getProjectiveXformCoeffs()
+ *           l_int32   projectiveXformSampledPt()
+ *           l_int32   projectiveXformPt()
  *
  *      A projective transform can be specified as a specific functional
  *      mapping between 4 points in the source and 4 points in the dest.
@@ -97,10 +99,10 @@
 
 
 /*-------------------------------------------------------------*
- *             Sampled projective image transformation           *
+ *            Sampled projective image transformation          *
  *-------------------------------------------------------------*/
 /*!
- *  pixProjectiveSampled()
+ *  pixProjectiveSampledPta()
  *
  *      Input:  pixs (all depths)
  *              ptad  (4 pts of final coordinate space)
@@ -113,23 +115,19 @@
  *      (2) Retains colormap, which you can do for a sampled transform..
  *      (3) No 3 of the 4 points may be collinear.
  *      (4) For 8 and 32 bpp pix, better quality is obtained by the
- *          somewhat slower pixProjectiveInterpolated().  See that
+ *          somewhat slower pixProjectivePta().  See that
  *          function for relative timings between sampled and interpolated.
  */
 PIX *
-pixProjectiveSampled(PIX     *pixs,
-                     PTA     *ptad,
-                     PTA     *ptas,
-                     l_int32  incolor)
+pixProjectiveSampledPta(PIX     *pixs,
+                        PTA     *ptad,
+                        PTA     *ptas,
+                        l_int32  incolor)
 {
-l_int32     i, j, w, h, d, x, y, wpls, wpld, color, cmapindex;
-l_uint32    val;
 l_float32  *vc;
-l_uint32   *datas, *datad, *lines, *lined;
 PIX        *pixd;
-PIXCMAP    *cmap;
 
-    PROCNAME("pixProjectiveSampled");
+    PROCNAME("pixProjectiveSampledPta");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
@@ -144,19 +142,55 @@ PIXCMAP    *cmap;
     if (ptaGetCount(ptad) != 4)
         return (PIX *)ERROR_PTR("ptad count not 4", procName, NULL);
 
-        /* Get backwards transform from dest to src */
-    projectiveXformCoeffs(ptad, ptas, &vc);
+        /* Get backwards transform from dest to src, and apply it */
+    getProjectiveXformCoeffs(ptad, ptas, &vc);
+    pixd = pixProjectiveSampled(pixs, vc, incolor);
+    FREE(vc);
 
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
-    datas = pixGetData(pixs);
-    wpls = pixGetWpl(pixs);
-    pixd = pixCreateTemplate(pixs);
-    datad = pixGetData(pixd);
-    wpld = pixGetWpl(pixd);
-    d = pixGetDepth(pixs);
+    return pixd;
+}
+
+
+/*!
+ *  pixProjectiveSampled()
+ *
+ *      Input:  pixs (all depths)
+ *              vc  (vector of 8 coefficients for projective transformation)
+ *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) Brings in either black or white pixels from the boundary.
+ *      (2) Retains colormap, which you can do for a sampled transform..
+ *      (3) For 8 or 32 bpp, much better quality is obtained by the
+ *          somewhat slower pixProjective().  See that function
+ *          for relative timings between sampled and interpolated.
+ */
+PIX *
+pixProjectiveSampled(PIX        *pixs,
+                     l_float32  *vc,
+                     l_int32     incolor)
+{
+l_int32     i, j, w, h, d, x, y, wpls, wpld, color, cmapindex;
+l_uint32    val;
+l_uint32   *datas, *datad, *lines, *lined;
+PIX        *pixd;
+PIXCMAP    *cmap;
+
+    PROCNAME("pixProjectiveSampled");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (!vc)
+        return (PIX *)ERROR_PTR("vc not defined", procName, NULL);
+    if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
+        return (PIX *)ERROR_PTR("invalid incolor", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 1 && d != 2 && d != 4 && d != 8 && d != 32)
+        return (PIX *)ERROR_PTR("depth not 1, 2, 4, 8 or 16", procName, NULL);
 
         /* Init all dest pixels to color to be brought in from outside */
+    pixd = pixCreateTemplate(pixs);
     if ((cmap = pixGetColormap(pixs)) != NULL) {
         if (incolor == L_BRING_IN_WHITE)
             color = 1;
@@ -173,46 +207,51 @@ PIXCMAP    *cmap;
             pixSetAll(pixd);
     }
 
-        /* Scan over dest pixels */
+        /* Scan over the dest pixels */
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
     for (i = 0; i < h; i++) {
         lined = datad + i * wpld;
         for (j = 0; j < w; j++) {
-            projectiveXformSampled(vc, j, i, &x, &y);
+            projectiveXformSampledPt(vc, j, i, &x, &y);
             if (x < 0 || y < 0 || x >=w || y >= h)
                 continue;
+            lines = datas + y * wpls;
             if (d == 1) {
-                lines = datas + y * wpls;
-                if (GET_DATA_BIT(lines, x))
-                    SET_DATA_BIT(lined, j);
+                val = GET_DATA_BIT(lines, x);
+                SET_DATA_BIT_VAL(lined, j, val);
             }
             else if (d == 8) {
-                lines = datas + y * wpls;
                 val = GET_DATA_BYTE(lines, x);
                 SET_DATA_BYTE(lined, j, val);
             }
             else if (d == 32) {
-                lines = datas + y * wpls;
                 lined[j] = lines[x];
             }
-            else {  /* all other depths */
-                pixGetPixel(pixs, x, y, &val);
-                pixSetPixel(pixd, j, i, val);
+            else if (d == 2) {
+                val = GET_DATA_DIBIT(lines, x);
+                SET_DATA_DIBIT(lined, j, val);
+            }
+            else if (d == 4) {
+                val = GET_DATA_QBIT(lines, x);
+                SET_DATA_QBIT(lined, j, val);
             }
         }
     }
 
-    FREE(vc);
     return pixd;
 }
 
 
-/*-------------------------------------------------------------*
- *         Interpolated projective image transformation        *
- *-------------------------------------------------------------*/
+/*---------------------------------------------------------------------*
+ *            Interpolated projective image transformation             *
+ *---------------------------------------------------------------------*/
 /*!
- *  pixProjectiveInterpolated()
+ *  pixProjectivePta()
  *
- *      Input:  pixs (8 bpp gray or colormapped or 32 bpp)
+ *      Input:  pixs (all depths; colormap ok)
  *              ptad  (4 pts of final coordinate space)
  *              ptas  (4 pts of initial coordinate space)
  *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
@@ -223,21 +262,19 @@ PIXCMAP    *cmap;
  *      (2) Removes any existing colormap, if necessary, before transforming
  */
 PIX *
-pixProjectiveInterpolated(PIX      *pixs,
-                          PTA      *ptad,
-                          PTA      *ptas,
-                          l_uint32  incolor)
+pixProjectivePta(PIX     *pixs,
+                 PTA     *ptad,
+                 PTA     *ptas,
+                 l_int32  incolor)
 {
 l_int32   d;
 l_uint32  colorval;
 PIX      *pixt1, *pixt2, *pixd;
 
-    PROCNAME("pixProjectiveInterpolated");
+    PROCNAME("pixProjectivePta");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (pixGetDepth(pixs) == 1)
-        return (PIX *)ERROR_PTR("pixs is 1 bpp", procName, NULL);
     if (!ptas)
         return (PIX *)ERROR_PTR("ptas not defined", procName, NULL);
     if (!ptad)
@@ -249,6 +286,9 @@ PIX      *pixt1, *pixt2, *pixd;
     if (ptaGetCount(ptad) != 4)
         return (PIX *)ERROR_PTR("ptad count not 4", procName, NULL);
 
+    if (pixGetDepth(pixs) == 1)
+        return pixProjectiveSampledPta(pixs, ptad, ptas, incolor);
+
         /* Remove cmap if it exists, and unpack to 8 bpp if necessary */
     pixt1 = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
     d = pixGetDepth(pixt1);
@@ -266,11 +306,11 @@ PIX      *pixt1, *pixt2, *pixd;
         else  /* d == 32 */
             colorval = 0xffffff00;
     }
-
+    
     if (d == 8)
-        pixd = pixProjectiveInterpolatedGray(pixt2, ptad, ptas, colorval);
+        pixd = pixProjectivePtaGray(pixt2, ptad, ptas, colorval);
     else  /* d == 32 */
-        pixd = pixProjectiveInterpolatedColor(pixt2, ptad, ptas, colorval);
+        pixd = pixProjectivePtaColor(pixt2, ptad, ptas, colorval);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     return pixd;
@@ -278,28 +318,83 @@ PIX      *pixt1, *pixt2, *pixd;
 
 
 /*!
- *  pixProjectiveInterpolatedColor()
+ *  pixProjective()
+ *
+ *      Input:  pixs (all depths; colormap ok)
+ *              vc  (vector of 8 coefficients for affine transformation)
+ *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) Brings in either black or white pixels from the boundary
+ *      (2) Removes any existing colormap, if necessary, before transforming
+ */
+PIX *
+pixProjective(PIX        *pixs,
+              l_float32  *vc,
+              l_int32     incolor)
+{
+l_int32   d;
+l_uint32  colorval;
+PIX      *pixt1, *pixt2, *pixd;
+
+    PROCNAME("pixProjective");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (!vc)
+        return (PIX *)ERROR_PTR("vc not defined", procName, NULL);
+
+    if (pixGetDepth(pixs) == 1)
+        return pixProjectiveSampled(pixs, vc, incolor);
+
+        /* Remove cmap if it exists, and unpack to 8 bpp if necessary */
+    pixt1 = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
+    d = pixGetDepth(pixt1);
+    if (d < 8)
+        pixt2 = pixConvertTo8(pixt1, FALSE);
+    else
+        pixt2 = pixClone(pixt1);
+    d = pixGetDepth(pixt2);
+
+        /* Compute actual color to bring in from edges */
+    colorval = 0;
+    if (incolor == L_BRING_IN_WHITE) {
+        if (d == 8)
+            colorval = 255;
+        else  /* d == 32 */
+            colorval = 0xffffff00;
+    }
+    
+    if (d == 8)
+        pixd = pixProjectiveGray(pixt2, vc, colorval);
+    else  /* d == 32 */
+        pixd = pixProjectiveColor(pixt2, vc, colorval);
+    pixDestroy(&pixt1);
+    pixDestroy(&pixt2);
+    return pixd;
+}
+
+
+/*!
+ *  pixProjectivePtaColor()
  *
  *      Input:  pixs (32 bpp)
  *              ptad  (4 pts of final coordinate space)
  *              ptas  (4 pts of initial coordinate space)
  *              colorval (e.g., 0 to bring in BLACK, 0xffffff00 for WHITE)
  *      Return: pixd, or null on error
- *
- *  *** Warning: implicit assumption about RGB component ordering ***
  */
 PIX *
-pixProjectiveInterpolatedColor(PIX      *pixs,
-                               PTA      *ptad,
-                               PTA      *ptas,
-                               l_uint32  colorval)
+pixProjectivePtaColor(PIX      *pixs,
+                      PTA      *ptad,
+                      PTA      *ptas,
+                      l_uint32  colorval)
 {
-l_int32     w, h, wpls, wpld;
 l_float32  *vc;
-l_uint32   *datas, *datad;
 PIX        *pixd;
 
-    PROCNAME("pixProjectiveInterpolatedColor");
+    PROCNAME("pixProjectivePtaColor");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
@@ -314,19 +409,9 @@ PIX        *pixd;
     if (ptaGetCount(ptad) != 4)
         return (PIX *)ERROR_PTR("ptad count not 4", procName, NULL);
 
-        /* Get backwards transform from dest to src */
-    projectiveXformCoeffs(ptad, ptas, &vc);
-
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
-    datas = pixGetData(pixs);
-    wpls = pixGetWpl(pixs);
-    pixd = pixCreateTemplate(pixs);
-    pixSetAllArbitrary(pixd, colorval);
-    datad = pixGetData(pixd);
-    wpld = pixGetWpl(pixd);
-
-    projectiveInterpolatedColorLow(datad, w, h, wpld, datas, wpls, vc);
+        /* Get backwards transform from dest to src, and apply it */
+    getProjectiveXformCoeffs(ptad, ptas, &vc);
+    pixd = pixProjectiveColor(pixs, vc, colorval);
     FREE(vc);
 
     return pixd;
@@ -334,7 +419,59 @@ PIX        *pixd;
 
 
 /*!
- *  pixProjectiveInterpolatedGray()
+ *  pixProjectiveColor()
+ *
+ *      Input:  pixs (32 bpp)
+ *              vc  (vector of 6 coefficients for affine transformation)
+ *              colorval (e.g., 0 to bring in BLACK, 0xffffff00 for WHITE)
+ *      Return: pixd, or null on error
+ */
+PIX *
+pixProjectiveColor(PIX        *pixs,
+                   l_float32  *vc,
+                   l_uint32    colorval)
+{
+l_int32    i, j, w, h, d, wpls, wpld;
+l_uint32   val;
+l_uint32  *datas, *datad, *lined;
+l_float32  x, y;
+PIX       *pixd;
+
+    PROCNAME("pixProjectiveColor");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 32)
+        return (PIX *)ERROR_PTR("pixs must be 32 bpp", procName, NULL);
+    if (!vc)
+        return (PIX *)ERROR_PTR("vc not defined", procName, NULL);
+
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    pixd = pixCreateTemplate(pixs);
+    pixSetAllArbitrary(pixd, colorval);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+
+        /* Iterate over destination pixels */
+    for (i = 0; i < h; i++) {
+        lined = datad + i * wpld;
+        for (j = 0; j < w; j++) {
+                /* Compute float src pixel location corresponding to (i,j) */
+            projectiveXformPt(vc, j, i, &x, &y);
+            linearInterpolatePixelColor(datas, wpls, w, h, x, y, colorval,
+                                        &val);
+            *(lined + j) = val;
+        }
+    }
+
+    return pixd;
+}
+
+
+/*!
+ *  pixProjectivePtaGray()
  *
  *      Input:  pixs (8 bpp)
  *              ptad  (4 pts of final coordinate space)
@@ -343,17 +480,15 @@ PIX        *pixd;
  *      Return: pixd, or null on error
  */
 PIX *
-pixProjectiveInterpolatedGray(PIX     *pixs,
-                              PTA     *ptad,
-                              PTA     *ptas,
-                              l_uint8  grayval)
+pixProjectivePtaGray(PIX     *pixs,
+                     PTA     *ptad,
+                     PTA     *ptas,
+                     l_uint8  grayval)
 {
-l_int32     w, h, wpls, wpld;
 l_float32  *vc;
-l_uint32   *datas, *datad;
 PIX        *pixd;
 
-    PROCNAME("pixProjectiveInterpolatedGray");
+    PROCNAME("pixProjectivePtaGray");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
@@ -368,11 +503,44 @@ PIX        *pixd;
     if (ptaGetCount(ptad) != 4)
         return (PIX *)ERROR_PTR("ptad count not 4", procName, NULL);
 
-        /* Get backwards transform from dest to src */
-    projectiveXformCoeffs(ptad, ptas, &vc);
+        /* Get backwards transform from dest to src, and apply it */
+    getProjectiveXformCoeffs(ptad, ptas, &vc);
+    pixd = pixProjectiveGray(pixs, vc, grayval);
+    FREE(vc);
+
+    return pixd;
+}
+
+
+
+/*!
+ *  pixProjectiveGray()
+ *
+ *      Input:  pixs (8 bpp)
+ *              vc  (vector of 8 coefficients for affine transformation)
+ *              grayval (0 to bring in BLACK, 255 for WHITE)
+ *      Return: pixd, or null on error
+ */
+PIX *
+pixProjectiveGray(PIX        *pixs,
+                  l_float32  *vc,
+                  l_uint8     grayval)
+{
+l_int32    i, j, w, h, wpls, wpld, val;
+l_uint32  *datas, *datad, *lined;
+l_float32  x, y;
+PIX       *pixd;
+
+    PROCNAME("pixProjectiveGray");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    if (pixGetDepth(pixs) != 8)
+        return (PIX *)ERROR_PTR("pixs must be 8 bpp", procName, NULL);
+    if (!vc)
+        return (PIX *)ERROR_PTR("vc not defined", procName, NULL);
 
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
     pixd = pixCreateTemplate(pixs);
@@ -380,119 +548,18 @@ PIX        *pixd;
     datad = pixGetData(pixd);
     wpld = pixGetWpl(pixd);
 
-    projectiveInterpolatedGrayLow(datad, w, h, wpld, datas, wpls, vc);
-    FREE(vc);
-
-    return pixd;
-}
-
-
-/*!
- *  projectiveInterpolatedColorLow()
- */
-void
-projectiveInterpolatedColorLow(l_uint32   *datad,
-                               l_int32     w,
-                               l_int32     h,
-                               l_int32     wpld,
-                               l_uint32   *datas,
-                               l_int32     wpls,
-                               l_float32  *vc)
-{
-l_int32    i, j, x, y, xf, yf, wm2, hm2;
-l_int32    rval, gval, bval;
-l_uint32   word00, word01, word10, word11, val;
-l_uint32  *lines, *lined;
-
         /* Iterate over destination pixels */
-    wm2 = w - 2;
-    hm2 = h - 2;
     for (i = 0; i < h; i++) {
         lined = datad + i * wpld;
         for (j = 0; j < w; j++) {
-                /* Compute src pixel and fraction corresponding to (i,j) */
-            projectiveXformInterpolated(vc, j, i, &x, &y, &xf, &yf);
-
-                /* Skip if off the edge; omit x = 0 and y = 0 because
-                 * xf and yf can be < 0, in which case overflow is
-                 * possible for val, and black pixels can be rendered
-                 * on pixels at the src boundaries. */
-            if (x < 1 || y < 1 || x > wm2 || y > hm2)
-                continue;
-
-                /* Do area weighting (eqiv. to linear interpolation) */
-            lines = datas + y * wpls;
-            word00 = *(lines + x);
-            word10 = *(lines + x + 1);
-            word01 = *(lines + wpls + x);
-            word11 = *(lines + wpls + x + 1);
-            rval = ((16 - xf) * (16 - yf) * ((word00 >> L_RED_SHIFT) & 0xff) +
-                xf * (16 - yf) * ((word10 >> L_RED_SHIFT) & 0xff) +
-                (16 - xf) * yf * ((word01 >> L_RED_SHIFT) & 0xff) +
-                xf * yf * ((word11 >> L_RED_SHIFT) & 0xff) + 128) / 256;
-            gval = ((16 - xf) * (16 - yf) * ((word00 >> L_GREEN_SHIFT) & 0xff) +
-                xf * (16 - yf) * ((word10 >> L_GREEN_SHIFT) & 0xff) +
-                (16 - xf) * yf * ((word01 >> L_GREEN_SHIFT) & 0xff) +
-                xf * yf * ((word11 >> L_GREEN_SHIFT) & 0xff) + 128) / 256;
-            bval = ((16 - xf) * (16 - yf) * ((word00 >> L_BLUE_SHIFT) & 0xff) +
-                xf * (16 - yf) * ((word10 >> L_BLUE_SHIFT) & 0xff) +
-                (16 - xf) * yf * ((word01 >> L_BLUE_SHIFT) & 0xff) +
-                xf * yf * ((word11 >> L_BLUE_SHIFT) & 0xff) + 128) / 256;
-            val = (rval << L_RED_SHIFT) | (gval << L_GREEN_SHIFT) |
-                  (bval << L_BLUE_SHIFT);
-            *(lined + j) = val;
-        }
-    }
-
-    return;
-}
-
-
-/*!
- *  projectiveInterpolatedGrayLow()
- */
-void
-projectiveInterpolatedGrayLow(l_uint32   *datad,
-                              l_int32     w,
-                              l_int32     h,
-                              l_int32     wpld,
-                              l_uint32   *datas,
-                              l_int32     wpls,
-                              l_float32  *vc)
-{
-l_int32     i, j, x, y, xf, yf, wm2, hm2;
-l_int32     v00, v01, v10, v11;
-l_uint8     val;
-l_uint32   *lines, *lined;
-
-        /* Iterate over destination pixels */
-    wm2 = w - 2;
-    hm2 = h - 2;
-    for (i = 0; i < h; i++) {
-        lined = datad + i * wpld;
-        for (j = 0; j < w; j++) {
-                /* Compute src pixel and fraction corresponding to (i,j) */
-            projectiveXformInterpolated(vc, j, i, &x, &y, &xf, &yf);
-
-                /* Skip if off the edge; omit x = 0 and y = 0 because
-                 * xf and yf can be < 0, in which case overflow is
-                 * possible for val, and black pixels can be rendered
-                 * on pixels at the src boundaries. */
-            if (x < 1 || y < 1 || x > wm2 || y > hm2)
-                continue;
-
-                /* Do area weighting (eqiv. to linear interpolation) */
-            lines = datas + y * wpls;
-            v00 = (16 - xf) * (16 - yf) * GET_DATA_BYTE(lines, x);
-            v10 = xf * (16 - yf) * GET_DATA_BYTE(lines, x + 1);
-            v01 = (16 - xf) * yf * GET_DATA_BYTE(lines + wpls, x);
-            v11 = xf * yf * GET_DATA_BYTE(lines + wpls, x + 1);
-            val = (l_uint8)((v00 + v01 + v10 + v11 + 128) / 256);
+                /* Compute float src pixel location corresponding to (i,j) */
+            projectiveXformPt(vc, j, i, &x, &y);
+            linearInterpolatePixelGray(datas, wpls, w, h, x, y, grayval, &val);
             SET_DATA_BYTE(lined, j, val);
         }
     }
 
-    return;
+    return pixd;
 }
 
 
@@ -500,7 +567,7 @@ l_uint32   *lines, *lined;
  *                Projective coordinate transformation         *
  *-------------------------------------------------------------*/
 /*!
- *  projectiveXformCoeffs()
+ *  getProjectiveXformCoeffs()
  *
  *      Input:  ptas  (source 4 points; unprimed)
  *              ptad  (transformed 4 points; primed)
@@ -552,16 +619,16 @@ l_uint32   *lines, *lined;
  *  projectiveXFormInterpolated().
  */
 l_int32
-projectiveXformCoeffs(PTA         *ptas,
-                      PTA         *ptad,
-                      l_float32  **pvc)
+getProjectiveXformCoeffs(PTA         *ptas,
+                         PTA         *ptad,
+                         l_float32  **pvc)
 {
 l_int32     i;
 l_float32   x1, y1, x2, y2, x3, y3, x4, y4;
 l_float32  *b;   /* rhs vector of primed coords X'; coeffs returned in *pvc */
 l_float32  *a[8];  /* 8x8 matrix A  */
 
-    PROCNAME("projectiveXformCoeffs");
+    PROCNAME("getProjectiveXformCoeffs");
 
     if (!ptas)
         return ERROR_INT("ptas not defined", procName, 1);
@@ -639,25 +706,27 @@ l_float32  *a[8];  /* 8x8 matrix A  */
 
 
 /*!
- *  projectiveXformSampled()
+ *  projectiveXformSampledPt()
  *
  *      Input:  vc (vector of 8 coefficients)
  *              (x, y)  (initial point)
  *              (&xp, &yp)   (<return> transformed point)
  *      Return: 0 if OK; 1 on error
  *
- *  Note: this does not check ptrs for returned data!
+ *  Notes:
+ *      (1) This finds the nearest pixel coordinates of the transformed point.
+ *      (2) It does not check ptrs for returned data!
  */
 l_int32
-projectiveXformSampled(l_float32  *vc,
-                       l_int32     x,
-                       l_int32     y,
-                       l_int32    *pxp,
-                       l_int32    *pyp)
+projectiveXformSampledPt(l_float32  *vc,
+                         l_int32     x,
+                         l_int32     y,
+                         l_int32    *pxp,
+                         l_int32    *pyp)
 {
 l_float32  factor;
 
-    PROCNAME("projectiveXformSampled");
+    PROCNAME("projectiveXformSampledPt");
 
     if (!vc)
         return ERROR_INT("vc not defined", procName, 1);
@@ -665,46 +734,40 @@ l_float32  factor;
     factor = 1. / (vc[6] * x + vc[7] * y + 1.);
     *pxp = (l_int32)(factor * (vc[0] * x + vc[1] * y + vc[2]) + 0.5);
     *pyp = (l_int32)(factor * (vc[3] * x + vc[4] * y + vc[5]) + 0.5);
-
     return 0;
 }
 
 
 /*!
- *  projectiveXformInterpolated()
+ *  projectiveXformPt()
  *
  *      Input:  vc (vector of 8 coefficients)
  *              (x, y)  (initial point)
  *              (&xp, &yp)   (<return> transformed point)
- *              (&fxp, &fyp)   (<return> fractional transformed point)
  *      Return: 0 if OK; 1 on error
  *
- *  Note: this does not check ptrs for returned data!
+ *  Notes:
+ *      (1) This computes the floating point location of the transformed point.
+ *      (2) It does not check ptrs for returned data!
  */
 l_int32
-projectiveXformInterpolated(l_float32  *vc,
-                            l_int32     x,
-                            l_int32     y,
-                            l_int32    *pxp,
-                            l_int32    *pyp,
-                            l_int32    *pfxp,
-                            l_int32    *pfyp)
+projectiveXformPt(l_float32  *vc,
+                  l_int32     x,
+                  l_int32     y,
+                  l_float32  *pxp,
+                  l_float32  *pyp)
 {
-l_float32  xp, yp, factor;
+l_float32  factor;
 
-    PROCNAME("projectiveXformInterpolated");
+    PROCNAME("projectiveXformPt");
 
     if (!vc)
         return ERROR_INT("vc not defined", procName, 1);
 
     factor = 1. / (vc[6] * x + vc[7] * y + 1.);
-    xp = factor * (vc[0] * x + vc[1] * y + vc[2]);
-    yp = factor * (vc[3] * x + vc[4] * y + vc[5]);
-    *pxp = (l_int32)xp;
-    *pyp = (l_int32)yp;
-    *pfxp = (l_int32)(16. * (xp - *pxp));
-    *pfyp = (l_int32)(16. * (yp - *pyp));
-
+    *pxp = factor * (vc[0] * x + vc[1] * y + vc[2]);
+    *pyp = factor * (vc[3] * x + vc[4] * y + vc[5]);
     return 0;
 }
 
+
