@@ -51,17 +51,17 @@ enum {
 
     /* Output files for ioFormatTest().
      * Note that the test for jpeg is not yet implemented */
-static const char *FILE_BMP  =  "/usr/tmp/junkout.bmp";
-static const char *FILE_PNG  =  "/usr/tmp/junkout.png";
-static const char *FILE_PNM  =  "/usr/tmp/junkout.pnm";
-static const char *FILE_G3   =  "/usr/tmp/junkout_g3.tif";
-static const char *FILE_G4   =  "/usr/tmp/junkout_g4.tif";
-static const char *FILE_RLE  =  "/usr/tmp/junkout_rle.tif";
-static const char *FILE_PB   =  "/usr/tmp/junkout_packbits.tif";
-static const char *FILE_LZW  =  "/usr/tmp/junkout_lzw.tif";
-static const char *FILE_ZIP  =  "/usr/tmp/junkout_zip.tif";
-static const char *FILE_TIFF =  "/usr/tmp/junkout.tif";
-static const char *FILE_JPG  =  "/usr/tmp/junkout.jpg"; 
+static const char *FILE_BMP  =  "/tmp/junkout.bmp";
+static const char *FILE_PNG  =  "/tmp/junkout.png";
+static const char *FILE_PNM  =  "/tmp/junkout.pnm";
+static const char *FILE_G3   =  "/tmp/junkout_g3.tif";
+static const char *FILE_G4   =  "/tmp/junkout_g4.tif";
+static const char *FILE_RLE  =  "/tmp/junkout_rle.tif";
+static const char *FILE_PB   =  "/tmp/junkout_packbits.tif";
+static const char *FILE_LZW  =  "/tmp/junkout_lzw.tif";
+static const char *FILE_ZIP  =  "/tmp/junkout_zip.tif";
+static const char *FILE_TIFF =  "/tmp/junkout.tif";
+static const char *FILE_JPG  =  "/tmp/junkout.jpg"; 
 
 
 /*---------------------------------------------------------------------*
@@ -168,10 +168,12 @@ PIX   *pix;
  *  pixReadWithHint()
  *
  *      Input:  filename (with full pathname or in local directory)
- *              hint: a bitwise OR of L_HINT_* values. These are not
- *                    binding, but may be used to optimize the
- *                    decoding of images.
+ *              hint (bitwise OR of L_HINT_* values for jpeg; use 0 for no hint)
  *      Return: pix if OK; null on error
+ *
+ *  Notes:
+ *      (1) The hint is not binding, but may be used to optimize jpeg decoding.
+ *          Use 0 for no hinting.
  */
 PIX *
 pixReadWithHint(const char  *filename,
@@ -199,9 +201,12 @@ PIX   *pix;
 /*!
  *  pixReadStream()
  *
- *      Input:  file stream
- *              hint: a bitwise OR of L_HINT_* values
+ *      Input:  fp (file stream)
+ *              hint (bitwise OR of L_HINT_* values for jpeg; use 0 for no hint)
  *      Return: pix if OK; null on error
+ *
+ *  Notes:
+ *      (1) The hint only applies to jpeg.
  */
 PIX *
 pixReadStream(FILE    *fp,
@@ -275,7 +280,7 @@ PIX     *pix;
 /*!
  *  findFileFormat()
  *
- *      Input:  fp
+ *      Input:  fp (file stream)
  *      Return: format integer; 0 on error or if format not recognized
  *
  *  N.B.: this resets fp to BOF
@@ -283,7 +288,7 @@ PIX     *pix;
 l_int32
 findFileFormat(FILE  *fp)
 {
-l_uint8  firstbytes[12]; 
+l_uint8  firstbytes[8]; 
 l_int32  format, ret;
 
     PROCNAME("findFileFormat");
@@ -292,12 +297,12 @@ l_int32  format, ret;
         return ERROR_INT("stream not defined", procName, 0);
 
     rewind(fp);
-    if (fnbytesInFile(fp) < 12)
+    if (fnbytesInFile(fp) < 8)
         return ERROR_INT("truncated file", procName, 0);
 
-    ret = fread((char *)&firstbytes, 1, 12, fp);
-    if (ret != 12)
-        return ERROR_INT("failed to read first 12 bytes of file", procName, 0);
+    ret = fread((char *)&firstbytes, 1, 8, fp);
+    if (ret != 8)
+        return ERROR_INT("failed to read first 8 bytes of file", procName, 0);
     rewind(fp);
 
     format = findFileFormatBuffer(firstbytes);
@@ -312,11 +317,11 @@ l_int32  format, ret;
 /*!
  *  findFileFormatBuffer()
  *
- *      Input:  byte buffer (at least 12 bytes in size; we can't check) 
+ *      Input:  byte buffer (at least 8 bytes in size; we can't check) 
  *      Return: format integer; 0 on error or if format not recognized
  *
  *  Notes:
- *      (1) This determines the file format from the first 12 bytes in
+ *      (1) This determines the file format from the first 8 bytes in
  *          the compressed data stream, which are stored in memory.
  *      (2) For tiff files, this returns IFF_TIFF.  The specific tiff
  *          compression is then determined using findTiffCompression().
@@ -388,7 +393,7 @@ l_uint16  twobytepw;
 /*!
  *  fileFormatIsTiff()
  *
- *      Input:  fp
+ *      Input:  fp (file stream)
  *      Return: 1 if file is tiff; 0 otherwise or on error
  */
 l_int32
@@ -428,6 +433,8 @@ l_int32  format;
  *      (2) On windows, this will only read tiff formatted files from
  *          memory.  For other formats, it requires fmemopen(3).
  *          Attempts to read those formats will fail at runtime.
+ *      (3) findFileFormatBuffer() requires up to 8 bytes to decide on
+ *          the format.  That determines the constraint here.
  */
 PIX *
 pixReadMem(const l_uint8  *data,
@@ -440,8 +447,8 @@ PIX     *pix;
 
     if (!data)
         return (PIX *)ERROR_PTR("data not defined", procName, NULL);
-    if (size < 12)
-        return (PIX *)ERROR_PTR("size < 12", procName, NULL);
+    if (size < 8)
+        return (PIX *)ERROR_PTR("size < 8", procName, NULL);
     pix = NULL;
 
     format = findFileFormatBuffer(data);
@@ -492,8 +499,14 @@ PIX     *pix;
         break;
     }
 
-    if (pix)
+        /* Set the input format.  For tiff reading from memory we lose
+         * the actual input format; for 1 bpp, default to G4.  */
+    if (pix) {
+        if (format == IFF_TIFF && pixGetDepth(pix) == 1)
+            format = IFF_TIFF_G4;
         pixSetInputFormat(pix, format);
+    }
+
     return pix;
 }
 
@@ -509,7 +522,7 @@ PIX     *pix;
  *
  *  Notes:
  *      (1) This writes and reads a set of output files losslessly
- *          in different formats to /usr/tmp/, and tests that the
+ *          in different formats to /tmp, and tests that the
  *          result before and after is unchanged.
  *      (2) This should work properly on input images of any depth,
  *          with and without colormaps.
