@@ -21,6 +21,9 @@
  *              PIX     *pixRotate()
  *              PIX     *pixEmbedForRotation()
  *
+ *     General rotation by sampling
+ *              PIX     *pixRotateBySampling()
+ *
  *     Rotations are measured in radians; clockwise is positive.
  *
  *     The general rotation pixRotate() does the best job for
@@ -32,7 +35,7 @@
  */
 
 #include <stdio.h>
-#include <string.h>
+#include <stdlib.h>
 #include <math.h>
 #include "allheaders.h"
 
@@ -47,7 +50,7 @@ static const l_float32  VERY_SMALL_ANGLE = 0.001;  /* radians; ~0.06 degrees */
  *
  *      Input:  pixs (1, 2, 4, 8, 32 bpp rgb)
  *              angle (radians; clockwise is positive)
- *              type (L_ROTATE_AREA_MAP, L_ROTATE_SHEAR)
+ *              type (L_ROTATE_AREA_MAP, L_ROTATE_SHEAR, L_ROTATE_SAMPLING)
  *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
  *              width (original width; use 0 to avoid embedding)
  *              height (original height; use 0 to avoid embedding)
@@ -57,12 +60,10 @@ static const l_float32  VERY_SMALL_ANGLE = 0.001;  /* radians; ~0.06 degrees */
  *      (1) Rotation is about the center of the image.
  *      (2) For very small rotations, just return a clone.
  *      (3) Rotation brings either white or black pixels in
- *          from outside the image.  For colormapped images where
- *          there is no white or black, a new color is added if
- *          possible for these pixels; otherwise, either the
- *          lightest or darkest color is used.
- *      (4) Binary images are always rotated by shear.
- *      (5) Colormaps are removed.
+ *          from outside the image.
+ *      (4) Above 20 degrees, if rotation by shear is requested, we rotate
+ *          by sampling.
+ *      (5) Colormaps are removed for rotation by area map and shear.
  *      (6) The dest can be expanded so that no image pixels
  *          are lost.  To invoke expansion, input the original
  *          width and height.  For repeated rotation, use of the
@@ -80,7 +81,7 @@ pixRotate(PIX       *pixs,
           l_int32    width,
           l_int32    height)
 {
-l_int32    d;
+l_int32    w, h, d;
 l_uint32   fillval;
 PIX       *pixt1, *pixt2, *pixt3, *pixd;
 PIXCMAP   *cmap;
@@ -89,7 +90,8 @@ PIXCMAP   *cmap;
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (type != L_ROTATE_SHEAR && type != L_ROTATE_AREA_MAP)
+    if (type != L_ROTATE_SHEAR && type != L_ROTATE_AREA_MAP &&
+        type != L_ROTATE_SAMPLING)
         return (PIX *)ERROR_PTR("invalid type", procName, NULL);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)ERROR_PTR("invalid incolor", procName, NULL);
@@ -97,27 +99,53 @@ PIXCMAP   *cmap;
     if (L_ABS(angle) < VERY_SMALL_ANGLE)
         return pixClone(pixs);
 
-        /* If there is a colormap, remove it. */
-    if ((cmap = pixGetColormap(pixs)) != NULL)
+        /* Don't rotate by shear more than 20 degrees */
+    if (L_ABS(angle) > 0.35 && type == L_ROTATE_SHEAR) {
+        L_WARNING("large angle; rotating by sampling", procName);
+        type = L_ROTATE_SAMPLING;
+    }
+
+        /* If 1 bpp and area map is requested, rotate by sampling */
+    d = pixGetDepth(pixs);
+    if (d == 1 && type == L_ROTATE_AREA_MAP) {
+        L_WARNING("1 bpp; rotating by sampling", procName);
+        type = L_ROTATE_SAMPLING;
+    }
+
+        /* Remove colormap if we're rotating by area mapping. */
+    cmap = pixGetColormap(pixs);
+    if (cmap && type == L_ROTATE_AREA_MAP)
         pixt1 = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
     else
         pixt1 = pixClone(pixs);
+    cmap = pixGetColormap(pixt1);
+
+        /* Otherwise, if there is a colormap and we're not embedding,
+         * add white color if it doesn't exist. */
+    if (cmap && width == 0) {  /* no embedding; generate @incolor */
+        if (incolor == L_BRING_IN_BLACK)
+            pixcmapAddBlackOrWhite(cmap, 0, NULL);
+        else  /* L_BRING_IN_WHITE */
+            pixcmapAddBlackOrWhite(cmap, 1, NULL);
+    }
 
         /* Request to embed in a larger image; do if necessary */
     pixt2 = pixEmbedForRotation(pixt1, angle, incolor, width, height);
 
         /* Area mapping requires 8 or 32 bpp.
-         * If 1 bpp, always use shear. */
+         * If 1 bpp, default to sampling. */
     d = pixGetDepth(pixt2);
-    if (type == L_ROTATE_AREA_MAP && d > 1 && d < 8)
+    if (type == L_ROTATE_AREA_MAP && d < 8)
         pixt3 = pixConvertTo8(pixt2, FALSE);
     else
         pixt3 = pixClone(pixt2);
 
         /* Rotate by shear or area mapping */
-    d = pixGetDepth(pixt3);
-    if (d == 1 || type == L_ROTATE_SHEAR)
+    pixGetDimensions(pixt3, &w, &h, &d);
+    if (type == L_ROTATE_SHEAR)
         pixd = pixRotateShearCenter(pixt3, angle, incolor);
+    else if (type == L_ROTATE_SAMPLING)
+        pixd = pixRotateBySampling(pixt3, w / 2, h / 2, angle, incolor);
     else {  /* rotate by area mapping */
         fillval = 0;
         if (incolor == L_BRING_IN_WHITE) {
@@ -180,10 +208,9 @@ pixEmbedForRotation(PIX       *pixs,
                     l_int32    width,
                     l_int32    height)
 {
-l_int32    w, h, d, maxside, wnew, hnew, xoff, yoff, index;
+l_int32    w, h, d, maxside, wnew, hnew, xoff, yoff;
 l_float64  pi, theta, absangle, alpha, beta, diag;
 PIX       *pixd;
-PIXCMAP   *cmap;
 
     PROCNAME("pixEmbedForRotation");
 
@@ -193,9 +220,7 @@ PIXCMAP   *cmap;
         return pixClone(pixs);
 
         /* Test if big enough to hold any rotation */
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
-    d = pixGetDepth(pixs);
+    pixGetDimensions(pixs, &w, &h, &d);
     maxside = (l_int32)(sqrt((l_float64)(width * width) +
                              (l_float64)(height * height)) + 0.5);
     if (w >= maxside && h >= maxside)  /* big enough */
@@ -226,22 +251,135 @@ PIXCMAP   *cmap;
     yoff = (hnew - h) / 2;
 
         /* Set background to color to be rotated in */
-    cmap = pixGetColormap(pixd);
-    if (!cmap) {
-        if ((d == 1 && incolor == L_BRING_IN_BLACK) ||
-            (d > 1 && incolor == L_BRING_IN_WHITE))
-            pixSetAll(pixd);
-    }
-    else {  /* handle colormap */
-        if (incolor == L_BRING_IN_BLACK)
-            pixcmapAddBlackOrWhite(cmap, 0, &index);
-        else  /* L_BRING_IN_WHITE */
-            pixcmapAddBlackOrWhite(cmap, 1, &index);
-        pixSetAllArbitrary(pixd, index);
-    }
+    pixSetBlackOrWhite(pixd, incolor);
 
     pixRasterop(pixd, xoff, yoff, w, h, PIX_SRC, pixs, 0, 0);
     return pixd;
 }
 
 
+/*------------------------------------------------------------------*
+ *                    General rotation by sampling                  *
+ *------------------------------------------------------------------*/
+/*!
+ *  pixRotateBySampling()
+ *
+ *      Input:  pixs (1, 2, 4, 8, 16, 32 bpp rgb; can be cmapped)
+ *              xcen (x value of center of rotation)
+ *              ycen (y value of center of rotation)
+ *              angle (radians; clockwise is positive)
+ *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) For very small rotations, just return a clone.
+ *      (2) Rotation brings either white or black pixels in
+ *          from outside the image.
+ *      (3) Colormaps are retained.
+ */
+PIX *
+pixRotateBySampling(PIX       *pixs,
+                    l_int32    xcen,
+                    l_int32    ycen,
+                    l_float32  angle,
+                    l_int32    incolor)
+{
+l_int32    w, h, d, i, j, x, y, xdif, ydif, wm1, hm1, wpld;
+l_uint32   val;
+l_float32  sina, cosa;
+l_uint32  *datad, *lined;
+void     **lines;
+PIX       *pixd;
+
+    PROCNAME("pixRotateBySampling");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
+        return (PIX *)ERROR_PTR("invalid incolor", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 1 && d != 2 && d != 4 && d != 8 && d != 16 && d != 32)
+        return (PIX *)ERROR_PTR("invalid depth", procName, NULL);
+
+    if (L_ABS(angle) < VERY_SMALL_ANGLE)
+        return pixClone(pixs);
+
+    if ((pixd = pixCreateTemplateNoInit(pixs)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    pixSetBlackOrWhite(pixd, incolor);
+
+    sina = sin(angle);
+    cosa = cos(angle);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+    wm1 = w - 1;
+    hm1 = h - 1;
+    lines = pixGetLinePtrs(pixs, NULL);
+
+        /* Treat 1 bpp case specially */
+    if (d == 1) {
+        for (i = 0; i < h; i++) {  /* scan over pixd */
+            lined = datad + i * wpld;
+            ydif = ycen - i;
+            for (j = 0; j < w; j++) {
+                xdif = xcen - j;
+                x = xcen + (l_int32)(-xdif * cosa - ydif * sina);
+                if (x < 0 || x > wm1) continue;
+                y = ycen + (l_int32)(-ydif * cosa + xdif * sina);
+                if (y < 0 || y > hm1) continue;
+                if (incolor == L_BRING_IN_WHITE) {
+                    if (GET_DATA_BIT(lines[y], x))
+                        SET_DATA_BIT(lined, j);
+                }
+                else {
+                    if (!GET_DATA_BIT(lines[y], x))
+                        CLEAR_DATA_BIT(lined, j);
+                }
+            }
+        }
+        FREE(lines);
+        return pixd;
+    }
+
+    for (i = 0; i < h; i++) {  /* scan over pixd */
+        lined = datad + i * wpld;
+        ydif = ycen - i;
+        for (j = 0; j < w; j++) {
+            xdif = xcen - j;
+            x = xcen + (l_int32)(-xdif * cosa - ydif * sina);
+            if (x < 0 || x > wm1) continue;
+            y = ycen + (l_int32)(-ydif * cosa + xdif * sina);
+            if (y < 0 || y > hm1) continue;
+            switch (d)
+            {
+            case 8:
+                val = GET_DATA_BYTE(lines[y], x);
+                SET_DATA_BYTE(lined, j, val);
+                break;
+            case 32:
+                val = GET_DATA_FOUR_BYTES(lines[y], x);
+                SET_DATA_FOUR_BYTES(lined, j, val);
+                break;
+            case 2:
+                val = GET_DATA_DIBIT(lines[y], x);
+                SET_DATA_DIBIT(lined, j, val);
+                break;
+            case 4:
+                val = GET_DATA_QBIT(lines[y], x);
+                SET_DATA_QBIT(lined, j, val);
+                break;
+            case 16:
+                val = GET_DATA_TWO_BYTES(lines[y], x);
+                SET_DATA_TWO_BYTES(lined, j, val);
+                break;
+            default:
+                return (PIX *)ERROR_PTR("invalid depth", procName, NULL);
+            }
+        }
+    }
+
+    FREE(lines);
+    return pixd;
+}
+
+
