@@ -137,13 +137,13 @@ l_float32  scx, scy;
 
         /* iterate over the destination pixels */
     for (i = 0; i < hd; i++) {
-        ypm = (l_int32)(scy * (l_float32)i + 0.5);
+        ypm = (l_int32)(scy * (l_float32)i);
         yp = ypm >> 4;
         yf = ypm & 0x0f;
         lined = datad + i * wpld;
         lines = datas + yp * wpls;
         for (j = 0; j < wd; j++) {
-            xpm = (l_int32)(scx * (l_float32)j + 0.5);
+            xpm = (l_int32)(scx * (l_float32)j);
             xp = xpm >> 4;
             xf = xpm & 0x0f;
 
@@ -226,25 +226,25 @@ l_float32  scx, scy;
     wm2 = ws - 2;
     hm2 = hs - 2;
 
-        /* iterate over the destination pixels */
+        /* Iterate over the destination pixels */
     for (i = 0; i < hd; i++) {
-        ypm = (l_int32)(scy * (l_float32)i + 0.5);
+        ypm = (l_int32)(scy * (l_float32)i);
         yp = ypm >> 4;
         yf = ypm & 0x0f;
         lined = datad + i * wpld;
         lines = datas + yp * wpls;
         for (j = 0; j < wd; j++) {
-            xpm = (l_int32)(scx * (l_float32)j + 0.5);
+            xpm = (l_int32)(scx * (l_float32)j);
             xp = xpm >> 4;
             xf = xpm & 0x0f;
 
-                /* if near the edge, just use the src pixel value */
+                /* If near the dest boundary, just use the src pixel value */
             if (xp > wm2 || yp > hm2) {
                 SET_DATA_BYTE(lined, j, GET_DATA_BYTE(lines, xp));
                 continue;
             }
 
-                /* do bilinear interpolation.  Without this, we could
+                /* Do bilinear interpolation.  Without this, we could
                  * simply subsample:
                  *   SET_DATA_BYTE(lined, j, GET_DATA_BYTE(lines, xp));
                  * which is faster but gives lousy results!
@@ -1113,9 +1113,9 @@ l_float32  wratio, hratio, norm;
     wratio = (l_float32)ws / (l_float32)wd;
     hratio = (l_float32)hs / (l_float32)hd;
     for (i = 0; i < hd; i++)
-        srow[i] = L_MIN((l_int32)(hratio * i + 0.5), hs - size);
+        srow[i] = L_MIN((l_int32)(hratio * i), hs - size);
     for (j = 0; j < wd; j++)
-        scol[j] = L_MIN((l_int32)(wratio * j + 0.5), ws - size);
+        scol[j] = L_MIN((l_int32)(wratio * j), ws - size);
 
         /* For each dest pixel, compute average */
     if (d == 8) {
@@ -1276,8 +1276,8 @@ l_float32  scx, scy;
 
         /* Iterate over the destination pixels */
     for (i = 0; i < hd; i++) {
-        yu = (l_int32)(scy * i + 0.5);
-        yl = (l_int32)(scy * (i + 1.0) + 0.5);
+        yu = (l_int32)(scy * i);
+        yl = (l_int32)(scy * (i + 1.0));
         yup = yu >> 4;
         yuf = yu & 0x0f;
         ylp = yl >> 4;
@@ -1286,8 +1286,8 @@ l_float32  scx, scy;
         lined = datad + i * wpld;
         lines = datas + yup * wpls;
         for (j = 0; j < wd; j++) {
-            xu = (l_int32)(scx * j + 0.5);
-            xl = (l_int32)(scx * (j + 1.0) + 0.5);
+            xu = (l_int32)(scx * j);
+            xl = (l_int32)(scx * (j + 1.0));
             xup = xu >> 4;
             xuf = xu & 0x0f;
             xlp = xl >> 4;
@@ -1432,8 +1432,8 @@ l_float32  scx, scy;
 
         /* Iterate over the destination pixels */
     for (i = 0; i < hd; i++) {
-        yu = (l_int32)(scy * i + 0.5);
-        yl = (l_int32)(scy * (i + 1.0) + 0.5);
+        yu = (l_int32)(scy * i);
+        yl = (l_int32)(scy * (i + 1.0));
         yup = yu >> 4;
         yuf = yu & 0x0f;
         ylp = yl >> 4;
@@ -1442,8 +1442,8 @@ l_float32  scx, scy;
         lined = datad + i * wpld;
         lines = datas + yup * wpls;
         for (j = 0; j < wd; j++) {
-            xu = (l_int32)(scx * j + 0.5);
-            xl = (l_int32)(scx * (j + 1.0) + 0.5);
+            xu = (l_int32)(scx * j);
+            xl = (l_int32)(scx * (j + 1.0));
             xup = xu >> 4;
             xuf = xu & 0x0f;
             xlp = xl >> 4;
@@ -1674,7 +1674,7 @@ scaleToGray2Low(l_uint32  *datad,
                 l_uint32  *sumtab,
                 l_uint8   *valtab)
 {
-l_int32    i, j, l, k;
+l_int32    i, j, l, k, m, wd4, extra;
 l_uint32   sbyte1, sbyte2, sum;
 l_uint32  *lines, *lined;
 
@@ -1684,10 +1684,12 @@ l_uint32  *lines, *lined;
          * k indexes the source bytes
          * We take two bytes from the source (in 2 lines of 8 pixels
          * each) and convert them into four 8 bpp bytes of the dest. */
+    wd4 = wd & 0xfffffffc;
+    extra = wd - wd4;
     for (i = 0, l = 0; i < hd; i++, l += 2) {
         lines = datas + l * wpls;
         lined = datad + i * wpld; 
-        for (j = 0, k = 0; j < wd; j += 4, k++) {
+        for (j = 0, k = 0; j < wd4; j += 4, k++) {
             sbyte1 = GET_DATA_BYTE(lines, k);
             sbyte2 = GET_DATA_BYTE(lines + wpls, k);
             sum = sumtab[sbyte1] + sumtab[sbyte2];
@@ -1696,6 +1698,16 @@ l_uint32  *lines, *lined;
             SET_DATA_BYTE(lined, j + 2, valtab[(sum >> 8) & 0xff]);
             SET_DATA_BYTE(lined, j + 3, valtab[sum & 0xff]);
         }
+        if (extra > 0) {
+            sbyte1 = GET_DATA_BYTE(lines, k);
+            sbyte2 = GET_DATA_BYTE(lines + wpls, k);
+            sum = sumtab[sbyte1] + sumtab[sbyte2];
+            for (m = 0; m < extra; m++) {
+                SET_DATA_BYTE(lined, j + m,
+                              valtab[((sum >> (24 - 8 * m)) & 0xff)]);
+            }
+        }
+        
     }
 
     return;
