@@ -31,9 +31,10 @@
  *     Information about tiff file
  *             l_int32    fprintTiffInfo()
  *             l_int32    tiffGetCount()
+ *      static l_int32    tiffGetResolution()
  *             l_int32    readHeaderTiff()
- *             l_int32    readHeaderMemTiff()
  *             l_int32    freadHeaderTiff()
+ *             l_int32    readHeaderMemTiff()
  *      static l_int32    tiffReadHeaderTiff()
  *             l_int32    findTiffCompression()
  *
@@ -49,8 +50,8 @@
  *             l_int32    pixWriteMemTiff();
  *             l_int32    pixWriteMemTiffCustom();
  *
- *   Note:  You should be using tifflib 3.7.4 to be certain that all
- *          the necessary functions are included.
+ *   Note:  You should be using version 3.7.4 of libtiff to be certain
+ *          that all the necessary functions are included.
  */
 
 #include <stdio.h>
@@ -82,18 +83,19 @@ static const l_int32  MAX_PAGES_IN_TIFF_FILE = 3000;  /* should be enough */
 
 
     /* All functions with TIFF interfaces are static. */
-static PIX       *pixReadFromTiffStream(TIFF *tif);
-static l_int32    tiffReadHeaderTiff(TIFF *tif, l_int32 *pwidth,
-                                     l_int32 *pheight, l_int32 *pbps,
-                                     l_int32 *pspp, l_int32 *pres,
-                                     l_int32 *pcmap);
-static l_int32    writeCustomTiffTags(TIFF *tif, NUMA *natags,
-                                      SARRAY *savals, SARRAY  *satypes,
-                                      NUMA *nasizes);
-static l_int32    pixWriteToTiffStream(TIFF *tif, PIX *pix, l_int32 comptype,
-                                       NUMA *natags, SARRAY *savals,
-                                       SARRAY *satypes, NUMA *nasizes);
-static TIFF      *fopenTiff(FILE *fp, const char *modestr); 
+static PIX      *pixReadFromTiffStream(TIFF *tif);
+static l_int32   tiffGetResolution(TIFF *tif, l_uint32 *pxres, l_uint32 *pyres);
+static l_int32   tiffReadHeaderTiff(TIFF *tif, l_int32 *pwidth,
+                                    l_int32 *pheight, l_int32 *pbps,
+                                    l_int32 *pspp, l_int32 *pres,
+                                    l_int32 *pcmap);
+static l_int32   writeCustomTiffTags(TIFF *tif, NUMA *natags,
+                                     SARRAY *savals, SARRAY  *satypes,
+                                     NUMA *nasizes);
+static l_int32   pixWriteToTiffStream(TIFF *tif, PIX *pix, l_int32 comptype,
+                                      NUMA *natags, SARRAY *savals,
+                                      SARRAY *satypes, NUMA *nasizes);
+static TIFF     *fopenTiff(FILE *fp, const char *modestr); 
 
     /* Static function for memory I/O */
 static TIFF *fopenTiffMemstream(const char *filename, const char *operation,
@@ -236,12 +238,11 @@ static PIX *
 pixReadFromTiffStream(TIFF  *tif)
 {
 l_uint8   *linebuf, *data;
-l_uint16   spp, bps, bpp, tiffbpl, photometry, orientation;
+l_uint16   spp, bps, bpp, tiffbpl, photometry, compress, orientation;
 l_uint16  *redmap, *greenmap, *bluemap;
-l_int32    d, wpl, bpl, i, j, k, ncolors;
-l_uint32   w, h, res;
-l_uint32  *line, *ppixel;
-l_float32  fres;
+l_int32    d, wpl, bpl, i, j, k, ncolors, rval, gval, bval;
+l_uint32   w, h, xres, yres, tiffword;
+l_uint32  *line, *ppixel, *tiffdata;
 PIX       *pix;
 PIXCMAP   *cmap;
 
@@ -267,19 +268,17 @@ PIXCMAP   *cmap;
     TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
     tiffbpl = TIFFScanlineSize(tif);
 
-    if ((linebuf = (l_uint8 *)CALLOC(tiffbpl + 1, sizeof(l_uint8))) == NULL)
-        return (PIX *)ERROR_PTR("calloc fail for linebuf", procName, NULL);
-        
-    if ((pix = pixCreate(w, h, d)) == NULL) {
-        FREE(linebuf);
+    if ((pix = pixCreate(w, h, d)) == NULL)
         return (PIX *)ERROR_PTR("pix not made", procName, NULL);
-    }
     data = (l_uint8 *)pixGetData(pix);
     wpl = pixGetWpl(pix);
     bpl = 4 * wpl;
 
         /* Read the data */
     if (spp == 1) {
+        if ((linebuf = (l_uint8 *)CALLOC(tiffbpl + 1, sizeof(l_uint8))) == NULL)
+            return (PIX *)ERROR_PTR("calloc fail for linebuf", procName, NULL);
+        
         for (i = 0 ; i < h ; i++) {
             if (TIFFReadScanline(tif, linebuf, i, 0) < 0) {
                 FREE(linebuf);
@@ -293,32 +292,38 @@ PIXCMAP   *cmap;
             pixEndianByteSwap(pix);
         else   /* bps == 16 */
             pixEndianTwoByteSwap(pix);
+        FREE(linebuf);
     }
     else {  /* rgb */
+        if ((tiffdata = (l_uint32 *)CALLOC(w * h, sizeof(l_uint32))) == NULL) {
+            pixDestroy(&pix);
+            return (PIX *)ERROR_PTR("calloc fail for tiffdata", procName, NULL);
+        }
+        if (!TIFFReadRGBAImageOriented(tif, w, h, (uint32 *)tiffdata,
+                                       ORIENTATION_TOPLEFT, 0)) {
+            FREE(tiffdata);
+            pixDestroy(&pix);
+            return (PIX *)ERROR_PTR("failed to read tiffdata", procName, NULL);
+        }
+
         line = pixGetData(pix);
-        for (i = 0 ; i < h ; i++, line += wpl)
-        {
-            if (TIFFReadScanline(tif, linebuf, i, 0) < 0) {
-                FREE(linebuf);
-                pixDestroy(&pix);
-                return (PIX *)ERROR_PTR("line read fail", procName, NULL);
-            }
+        for (i = 0 ; i < h ; i++, line += wpl) {
             for (j = 0, k = 0, ppixel = line; j < w; j++) {
-                SET_DATA_BYTE(ppixel, COLOR_RED, linebuf[k++]);
-                SET_DATA_BYTE(ppixel, COLOR_GREEN, linebuf[k++]);
-                SET_DATA_BYTE(ppixel, COLOR_BLUE, linebuf[k++]);
+                    /* TIFFGet* are macros */
+                tiffword = tiffdata[i * w + j];
+                rval = TIFFGetR(tiffword);
+                gval = TIFFGetG(tiffword);
+                bval = TIFFGetB(tiffword);
+                composeRGBPixel(rval, gval, bval, ppixel);
                 ppixel++;
             } 
         }
+        FREE(tiffdata);
     }
 
-    if (TIFFGetField(tif, TIFFTAG_XRESOLUTION, &fres)) {
-        res = (l_uint32)fres;
-        pixSetXRes(pix, res);
-    }
-    if (TIFFGetField(tif, TIFFTAG_YRESOLUTION, &fres)) {
-        res = (l_uint32)fres;
-        pixSetYRes(pix, res);
+    if (tiffGetResolution(tif, &xres, &yres)) {
+        pixSetXRes(pix, xres);
+        pixSetYRes(pix, yres);
     }
 
     if (TIFFGetField(tif, TIFFTAG_COLORMAP, &redmap, &greenmap, &bluemap)) {
@@ -327,7 +332,6 @@ PIXCMAP   *cmap;
              * and go from black (0) to white (0xffff), the
              * the pix cmap takes the most significant byte. */
         if ((cmap = pixcmapCreate(bps)) == NULL) {
-            FREE(linebuf);
             pixDestroy(&pix);
             return (PIX *)ERROR_PTR("cmap not made", procName, NULL);
         }
@@ -338,7 +342,19 @@ PIXCMAP   *cmap;
         pixSetColormap(pix, cmap);
     }
     else {   /* No colormap: check photometry and invert if necessary */
-        TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &photometry);
+        if (!TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &photometry)) {
+                /* Guess default photometry setting.  Assume min_is_white
+                 * if compressed 1 bpp; min_is_black otherwise. */
+            TIFFGetFieldDefaulted(tif, TIFFTAG_COMPRESSION, &compress);
+            if (compress == COMPRESSION_CCITTFAX3 ||
+                compress == COMPRESSION_CCITTFAX4 ||
+                compress == COMPRESSION_CCITTRLE ||
+                compress == COMPRESSION_CCITTRLEW) {
+                photometry = PHOTOMETRIC_MINISWHITE;
+            }
+            else
+                photometry = PHOTOMETRIC_MINISBLACK;
+        }
         if ((d == 1 && photometry == PHOTOMETRIC_MINISBLACK) ||
             (d == 8 && photometry == PHOTOMETRIC_MINISWHITE))
             pixInvert(pix, pix);
@@ -358,7 +374,6 @@ PIXCMAP   *cmap;
         }
     }
 
-    FREE(linebuf);
     return pix;
 }
 
@@ -551,6 +566,13 @@ TIFF  *tif;
  *          it typically expands images that are not synthetically generated.
  *      (3) See pixWriteTiffCustom() for details on how to use
  *          the last four parameters for customized tiff tags.
+ *      (4) The only valid pixel depths in leptonica are 1, 2, 4, 8, 16
+ *          and 32.  However, it is possible, and in some cases desirable,
+ *          to write out a tiff file using an rgb pix that has 24 bpp.
+ *          This can be created by appending the raster data for a 24 bpp
+ *          image (with proper scanline padding) directly to a 24 bpp
+ *          pix that was created without a data array.  See note in
+ *          pixWriteStreamPng() for an example.
  */
 static l_int32
 pixWriteToTiffStream(TIFF    *tif,
@@ -598,7 +620,7 @@ char      *text;
         
     if (d == 1)
         TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISWHITE);
-    else if (d == 32) {
+    else if (d == 32 || d == 24) {
         TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE,
                        (l_uint16)8, (l_uint16)8, (l_uint16)8);
@@ -633,7 +655,7 @@ char      *text;
         TIFFSetField(tif, TIFFTAG_COLORMAP, redmap, greenmap, bluemap);
     }
 
-    if (d != 32) {
+    if (d != 24 && d != 32) {
         TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, (l_uint16)d);
         TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, (l_uint16)1);
     }
@@ -673,22 +695,28 @@ char      *text;
         /* Use single strip for image */
     TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, h);
 
-    if (d != 32) {
+    if (d != 24 && d != 32) {
         if (d == 16)
             pixt = pixEndianTwoByteSwapNew(pix);
         else
             pixt = pixEndianByteSwapNew(pix);
         data = (l_uint8 *)pixGetData(pixt);
-        for (i = 0 ; i < h; i++, data += bpl) {
+        for (i = 0; i < h; i++, data += bpl) {
             memcpy((char *)linebuf, (char *)data, tiffbpl);
             if (TIFFWriteScanline(tif, linebuf, i, 0) < 0)
                 break;
         }
         pixDestroy(&pixt);
     }
-    else {  /* d == 32 */
-        line = pixGetData(pix);
-        for (i = 0 ; i < h; i++, line += wpl) {
+    else if (d == 24) {  /* See note 4 above: special case of 24 bpp rgb */
+        for (i = 0; i < h; i++) {
+            line = pixGetData(pix) + i * wpl;
+            if (TIFFWriteScanline(tif, (l_uint8 *)line, i, 0) < 0)
+                break;
+        }
+    }
+    else {  /* standard 32 bpp rgb */
+        for (i = 0; i < h; i++) {
             line = pixGetData(pix) + i * wpl;
             for (j = 0, k = 0, ppixel = line; j < w; j++) {
                 linebuf[k++] = GET_DATA_BYTE(ppixel, COLOR_RED);
@@ -908,6 +936,59 @@ TIFF    *tif;
 }
 
 
+/*--------------------------------------------------------------*
+ *                   Get resolution from tif                    *
+ *--------------------------------------------------------------*/
+/*
+ *  tiffGetResolution()
+ * 
+ *      Input:  tiff stream (opened for read)
+ *              &xres, &yres (<return> resolution in ppi)
+ *      Return: 1 if OK; 0 on error   (nonstandard)
+ *
+ *  Notes:
+ *      (1) If neither resolution field is set, this is not an error;
+ *          the returned resolution values are 0 (designating 'unknown').
+ */
+static l_int32
+tiffGetResolution(TIFF      *tif,
+                  l_uint32  *pxres,
+                  l_uint32  *pyres)
+{
+l_uint16   resunit;
+l_int32    foundxres, foundyres;
+l_float32  fxres, fyres;
+
+    PROCNAME("tiffGetResolution");
+
+    if (!tif)
+        return ERROR_INT("tif not opened", procName, 0);
+    if (!pxres || !pyres)
+        return ERROR_INT("&xres and &yres not both defined", procName, 0);
+    *pxres = *pyres = 0;
+
+    TIFFGetFieldDefaulted(tif, TIFFTAG_RESOLUTIONUNIT, &resunit);
+    foundxres = TIFFGetField(tif, TIFFTAG_XRESOLUTION, &fxres);
+    foundyres = TIFFGetField(tif, TIFFTAG_YRESOLUTION, &fyres);
+    if (!foundxres && !foundyres) return 1;
+    if (!foundxres && foundyres)
+        fxres = fyres;
+    else if (foundxres && !foundyres)
+        fyres = fxres;
+
+    if (resunit == RESUNIT_CENTIMETER) {  /* convert to ppi */
+        *pxres = (l_uint32)(2.54 * fxres + 0.5);
+        *pyres = (l_uint32)(2.54 * fyres + 0.5);
+    }
+    else {
+        *pxres = (l_uint32)fxres;
+        *pyres = (l_uint32)fyres;
+    }
+
+    return 1;
+}
+
+
 /*--------------------------------------------------------------*
  *              Get some tiff header information                *
  *--------------------------------------------------------------*/
@@ -915,6 +996,7 @@ TIFF    *tif;
  *  readHeaderTiff()
  *
  *      Input:  filename
+ *              n (page image number: 0-based)
  *              &width (<return>)
  *              &height (<return>)
  *              &bps (<return> bits per sample -- 1, 2, 4 or 8)
@@ -925,9 +1007,11 @@ TIFF    *tif;
  * 
  *  Notes:
  *      (1) If there is a colormap, cmap is returned as 1; else 0.
+ *      (2) If @n is equal to or greater than the number of images, returns 1.
  */
 l_int32
 readHeaderTiff(const char *filename,
+               l_int32     n,
                l_int32    *pwidth,
                l_int32    *pheight,
                l_int32    *pbps,
@@ -935,7 +1019,7 @@ readHeaderTiff(const char *filename,
                l_int32    *pres,
                l_int32    *pcmap)
 {
-l_int32  ret, format;
+l_int32  ret;
 FILE    *fp;
 
     PROCNAME("readHeaderTiff");
@@ -947,16 +1031,75 @@ FILE    *fp;
     *pwidth = *pheight = *pbps = *pspp = 0;
     if (pres) *pres = 0;
     if (pcmap) *pcmap = 0;
+
     if ((fp = fopenReadStream(filename)) == NULL)
         return ERROR_INT("image file not found", procName, 1);
+    ret = freadHeaderTiff(fp, n, pwidth, pheight, pbps, pspp, pres, pcmap);
+    fclose(fp);
+    return ret;
+}
+
+
+/*!
+ *  freadHeaderTiff()
+ *
+ *      Input:  stream
+ *              n (page image number: 0-based)
+ *              &width (<return>)
+ *              &height (<return>)
+ *              &bps (<return> bits per sample -- 1, 2, 4 or 8)
+ *              &spp (<return>; samples per pixel -- 1 or 3)
+ *              &res (<optional return>; resolution in x dir; NULL to ignore)
+ *              &cmap (<optional return>; colormap exists; input NULL to ignore)
+ *      Return: 0 if OK, 1 on error
+ * 
+ *  Notes:
+ *      (1) If there is a colormap, cmap is returned as 1; else 0.
+ *      (2) If @n is equal to or greater than the number of images, returns 1.
+ */
+l_int32
+freadHeaderTiff(FILE     *fp,
+                l_int32   n,
+                l_int32  *pwidth,
+                l_int32  *pheight,
+                l_int32  *pbps,
+                l_int32  *pspp,
+                l_int32  *pres,
+                l_int32  *pcmap)
+{
+l_int32  i, ret, format;
+TIFF    *tif;
+
+    PROCNAME("freadHeaderTiff");
+
+    if (!fp)
+        return ERROR_INT("stream not defined", procName, 1);
+    if (n < 0)
+        return ERROR_INT("image index must be >= 0", procName, 1);
+    if (!pwidth || !pheight || !pbps || !pspp)
+        return ERROR_INT("input ptr(s) not all defined", procName, 1);
+    *pwidth = *pheight = *pbps = *pspp = 0;
+    if (pres) *pres = 0;
+    if (pcmap) *pcmap = 0;
+
     format = findFileFormat(fp);
     if (format != IFF_TIFF &&
         format != IFF_TIFF_G3 && format != IFF_TIFF_G4 &&
         format != IFF_TIFF_RLE && format != IFF_TIFF_PACKBITS &&
         format != IFF_TIFF_LZW && format != IFF_TIFF_ZIP)
         return ERROR_INT("file not tiff format", procName, 1);
-    ret = freadHeaderTiff(fp, pwidth, pheight, pbps, pspp, pres, pcmap);
-    fclose(fp);
+
+    if ((tif = fopenTiff(fp, "r")) == NULL)
+        return ERROR_INT("tif not open for read", procName, 1);
+
+    for (i = 0; i < n; i++) {
+        if (TIFFReadDirectory(tif) == 0)
+            return ERROR_INT("image n not found in file", procName, 1);
+    }
+
+    ret = tiffReadHeaderTiff(tif, pwidth, pheight, pbps, pspp, pres, pcmap);
+
+    TIFFCleanup(tif);
     return ret;
 }
 
@@ -966,6 +1109,7 @@ FILE    *fp;
  *
  *      Input:  cdata (const; tiff-encoded)
  *              size (size of data)
+ *              n (page image number: 0-based)
  *              &width (<return>)
  *              &height (<return>)
  *              &bps (<return> bits per sample -- 1, 2, 4 or 8)
@@ -980,6 +1124,7 @@ FILE    *fp;
 l_int32
 readHeaderMemTiff(const l_uint8  *cdata,
                   size_t          size,
+                  l_int32         n,
                   l_int32        *pwidth,
                   l_int32        *pheight,
                   l_int32        *pbps,
@@ -988,7 +1133,7 @@ readHeaderMemTiff(const l_uint8  *cdata,
                   l_int32        *pcmap)
 {
 l_uint8  *data;
-l_int32   ret;
+l_int32   i, ret;
 TIFF     *tif;
 
     PROCNAME("readHeaderMemTiff");
@@ -1006,6 +1151,13 @@ TIFF     *tif;
     if ((tif = fopenTiffMemstream("tifferror", "r", &data, &size)) == NULL)
         return ERROR_INT("tiff stream not opened", procName, 1);
 
+    for (i = 0; i < n; i++) {
+        if (TIFFReadDirectory(tif) == 0) {
+            TIFFClose(tif);
+            return ERROR_INT("image n not found in file", procName, 1);
+        }
+    }
+
     ret = tiffReadHeaderTiff(tif, pwidth, pheight, pbps, pspp, pres, pcmap);
 
     TIFFClose(tif);
@@ -1013,48 +1165,6 @@ TIFF     *tif;
 }
 
 
-/*!
- *  freadHeaderTiff()
- *
- *      Input:  stream
- *              &width (<return>)
- *              &height (<return>)
- *              &bps (<return> bits per sample -- 1, 2, 4 or 8)
- *              &spp (<return>; samples per pixel -- 1 or 3)
- *              &res (<optional return>; resolution in x dir; NULL to ignore)
- *              &cmap (<optional return>; colormap exists; input NULL to ignore)
- *      Return: 0 if OK, 1 on error
- */
-l_int32
-freadHeaderTiff(FILE     *fp,
-                l_int32  *pwidth,
-                l_int32  *pheight,
-                l_int32  *pbps,
-                l_int32  *pspp,
-                l_int32  *pres,
-                l_int32  *pcmap)
-{
-l_int32  ret;
-TIFF    *tif;
-
-    PROCNAME("freadHeaderTiff");
-
-    if (!fp)
-        return ERROR_INT("stream not defined", procName, 1);
-    if (!pwidth || !pheight || !pbps || !pspp)
-        return ERROR_INT("input ptr(s) not all defined", procName, 1);
-    
-        /* Open a tiff stream to file */
-    if ((tif = fopenTiff(fp, "r")) == NULL)
-        return ERROR_INT("tif not opened", procName, 1);
-
-    ret = tiffReadHeaderTiff(tif, pwidth, pheight, pbps, pspp, pres, pcmap);
-
-    TIFFCleanup(tif);
-    return ret;
-}
-
-
 /*!
  *  tiffReadHeaderTiff()
  *
@@ -1078,8 +1188,7 @@ tiffReadHeaderTiff(TIFF     *tif,
 {
 l_uint16   bps, spp;
 l_uint16  *rmap, *gmap, *bmap;
-l_uint32   w, h;
-l_float32  fres;
+l_uint32   w, h, xres, yres;
 
     PROCNAME("tiffReadHeaderTiff");
 
@@ -1096,9 +1205,9 @@ l_float32  fres;
     *pspp = spp;
 
     if (pres) {
-        *pres = 300;
-        if (TIFFGetField(tif, TIFFTAG_XRESOLUTION, &fres))
-            *pres = (l_int32)fres;
+        *pres = 300;  /* default ppi */
+        if (tiffGetResolution(tif, &xres, &yres))
+            *pres = (l_int32)xres;
     }
             
     if (pcmap) {
@@ -1590,7 +1699,7 @@ L_MEMSTREAM  *mstream;
  *
  *      Input:  data (const; tiff-encoded)
  *              datasize (size of data)
- *              n (page number: 0 based)
+ *              n (page image number: 0-based)
  *      Return: pix, or null on error
  *
  *  Notes:
