@@ -47,6 +47,7 @@
  *
  *       find and replace procs
  *           char      *stringRemoveChars()
+ *           l_int32    stringFindSubstr()
  *           char      *stringReplaceSubstr()
  *           char      *stringReplaceEachSubstr()
  *           l_int32    arrayFindSequence()
@@ -76,6 +77,8 @@
  *           l_int32    splitPathAtDirectory()
  *           l_int32    splitPathAtExtension()
  *           char      *genPathname()
+ *           char      *genTempFilename()
+ *           l_int32    extractNumberFromFilename()
  *
  *       timing
  *           void       startTimer()
@@ -85,8 +88,15 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include "allheaders.h"
 
+#if COMPILER_MSVC
+static const char sepchar = '\\';
+#else
+static const char sepchar = '/';
+#endif
+
 
 /*----------------------------------------------------------------------*
  *                 Error, warning and info message procs                *
@@ -910,6 +920,49 @@ l_int32  nsrc, i, k;
 }
 
 
+/*!
+ *  stringFindSubstr()
+ *
+ *      Input:  src (input string; can be of zero length)
+ *              sub (substring to be searched for)
+ *              &loc (<return optional> location of substring in src)
+ *      Return: 1 if found; 0 if not found or on error
+ *
+ *  Notes:
+ *      (1) This is a wrapper around strstr().
+ *      (2) Both @src and @sub must be defined, and @sub must have
+ *          length of at least 1.
+ *      (3) If the substring is not found and loc is returned, it has
+ *          the value -1.
+ */
+l_int32
+stringFindSubstr(const char  *src,
+                 const char  *sub,
+                 l_int32     *ploc)
+{
+char  *ptr;
+
+    PROCNAME("stringFindSubstr");
+
+    if (!src)
+        return ERROR_INT("src not defined", procName, 0);
+    if (!sub)
+        return ERROR_INT("sub not defined", procName, 0);
+    if (ploc) *ploc = -1;
+    if (strlen(sub) == 0)
+        return ERROR_INT("substring length 0", procName, 0);
+    if (strlen(src) == 0)
+        return 0;
+
+    if ((ptr = (char *)strstr(src, sub)) == NULL)  /* not found */
+        return 0;
+
+    if (*ploc)
+        *ploc = ptr - src;
+    return 1;
+}
+
+
 /*!
  *  stringReplaceSubstr()
  *
@@ -918,7 +971,6 @@ l_int32  nsrc, i, k;
  *              sub2 (substring to put in; can be "")
  *              &found (<return optional> 1 if sub1 is found; 0 otherwise)
  *              &loc (<return optional> location of ptr after replacement)
- *
  *      Return: dest (string with substring replaced), or null if the
  *              substring not found or on error.
  *
@@ -1042,8 +1094,8 @@ l_int32  loc;
  *              datalen (length of data, in bytes)
  *              sequence (subarray of bytes to find in data)
  *              seqlen (length of sequence, in bytes)
- *              &offset (<return> offset from beginning of data where
- *                       the sequence begins)
+ *              &offset (return> offset from beginning of
+ *                       data where the sequence begins)
  *              &found (<return> 1 if sequence is found; 0 otherwise)
  *      Return: 0 if OK, 1 on error
  *
@@ -1054,8 +1106,7 @@ l_int32  loc;
  *      (2) This searches for the first occurrence in 'data' of
  *          the first 'seqlen' bytes of 'sequence'.  The parameter 'seqlen'
  *          must not exceed the actual length of the 'sequence' byte array.
- *      (3) The caller should always check the return value of 'found'.
- *          If not found, the offset will be set to 0.
+ *      (3) If the sequence is not found, the offset will be set to -1.
  */
 l_int32
 arrayFindSequence(const l_uint8  *data,
@@ -1074,7 +1125,8 @@ l_int32  i, j, found, lastpos;
     if (!poffset || !pfound)
         return ERROR_INT("&offset and &found not both defined", procName, 1);
 
-    *poffset = *pfound = 0;
+    *pfound = 0;
+    *poffset = -1;
     lastpos = datalen - seqlen + 1;
     found = 0;
     for (i = 0; i < lastpos; i++) {
@@ -1583,8 +1635,8 @@ char *
 genPathname(const char  *dir,
             const char  *fname)
 {
-l_int32  dirlen, namelen;
 char    *charbuf;
+l_int32  dirlen, namelen;
     
     PROCNAME("genPathname");
 
@@ -1599,8 +1651,12 @@ char    *charbuf;
             == NULL)
         return (char *)ERROR_PTR("charbuf not made", procName, NULL);
 
-    if (dir[dirlen - 1] != '/')
+    if (dir[dirlen - 1] != sepchar)
+#if COMPILER_MSVC
+        sprintf(charbuf, "%s\\", dir);
+#else
         sprintf(charbuf, "%s/", dir);
+#endif
     else
         strcpy(charbuf, dir);
     strcat(charbuf, fname);
@@ -1608,6 +1664,95 @@ char    *charbuf;
 }
 
 
+/*! 
+ *  genTempFilename()
+ *
+ *      Input:  dir (directory name; use '.' for local dir; no trailing '/')
+ *              extension (<optional> filename extention with '.'; can be null)
+ *      Return: tempname (with pid embedded in file name), or null on error
+ *
+ *  Notes:
+ *      (1) This function is useful when there can be more than one
+ *          process writing and reading temporary files.  It will not
+ *          work properly when multiple threads from a single process call
+ *          this function.  Furthermore, as with any function that
+ *          provides easily guessed temporary filenames, it is not designed
+ *          to be safe from an attack where the intruder is logged onto
+ *          the server.
+ */
+char *
+genTempFilename(const char  *dir,
+                const char  *extension)
+{
+char     buf[256];
+char    *tempname;
+l_int32  pid, nchars;
+    
+    PROCNAME("genTempFilename");
+
+    if (!dir)
+        return (char *)ERROR_PTR("dir not defined", procName, NULL);
+    pid = getpid();
+    if (extension)
+        nchars = strlen(extension);
+    else
+        nchars = 0;
+
+#if COMPILER_MSVC
+    snprintf(buf, 255 - nchars, "%s\\%d", dir, pid);
+#else
+    snprintf(buf, 25 - nchars, "%s/%d", dir, pid);
+#endif
+
+    tempname = stringJoin(buf, extension);
+    return tempname;
+}
+
+
+/*! 
+ *  extractNumberFromFilename()
+ *
+ *      Input:  fname
+ *              numpre (number of characters before the digits to be found)
+ *              numpost (number of characters after the digits to be found)
+ *      Return: num (number embedded in the filename); -1 on error or if
+ *                   not found
+ */
+l_int32
+extractNumberFromFilename(const char  *fname,
+                          l_int32      numpre,
+                          l_int32      numpost)
+{
+char    *tail, *basename;
+l_int32  len, nret, num;
+    
+    PROCNAME("extractNumberFromFilename");
+
+    if (!fname)
+        return ERROR_INT("fname not defined", procName, -1);
+
+    splitPathAtDirectory(fname, NULL, &tail);
+    splitPathAtExtension(tail, &basename, NULL);
+    FREE(tail);
+
+    len = strlen(basename);
+    if (numpre + numpost > len - 1) {
+        FREE(basename);
+        return ERROR_INT("numpre + numpost too big", procName, -1);
+    }
+
+    basename[len - numpost] = '\n';
+    nret = sscanf(basename + numpre, "%d", &num);
+    FREE(basename);
+
+    if (nret == 1)
+        return num;
+    else
+        return ERROR_INT("no number found", procName, -1);
+}
+
+
+
 /*---------------------------------------------------------------------*
  *                           Timing procs                              *
  *---------------------------------------------------------------------*/
