@@ -21,6 +21,12 @@
  *   handle the preprocessor macros, and then parses the cpp output.
  *   In use, it is convenient to redirect stdout to a file.
  *
+ *   An optional 'prestring' can be prepended to each declaration.
+ *   Without this string, use:
+ *      xtractprotos [list of C files]
+ *   With this string, use:
+ *      xtractprotos -prestring=[string] [list of C files]
+ *
  *   Update the version number when making a new version.
  *
  *   For simple C prototype extraction, xtractprotos has essentially
@@ -43,33 +49,52 @@
 
 static const char *tempfile = "/tmp/temp_cpp_output";
 static const l_int32  L_BUF_SIZE = 512;
-static const char *version = "1.3";
+static const char *version = "1.4";
 
 main(int    argc,
      char **argv)
 {
-char        *filename, *filein, *str;
+char        *filename, *filein, *str, *prestring;
+const char  *spacestr = " ";
 char         buf[L_BUF_SIZE];
-l_int32      i, len, ret;
+l_int32      i, firstfile, len, ret;
 SARRAY      *sa;
 static char  mainName[] = "xtractprotos";
 
         /* Output extern C head */
     sa = sarrayCreate(0);
-    sarrayAddString(sa, "/*", 1);
+    sarrayAddString(sa, (char *)"/*", 1);
     snprintf(buf, L_BUF_SIZE,
              " *  This file was autogen'd by xtractprotos, v. %s", version);
     sarrayAddString(sa, buf, 1);
-    sarrayAddString(sa, " */", 1);
-    sarrayAddString(sa, "#ifdef __cplusplus", 1);
-    sarrayAddString(sa, "extern \"C\" {", 1);
-    sarrayAddString(sa, "#endif  /* __cplusplus */\n", 1);
+    sarrayAddString(sa, (char *)" */", 1);
+    sarrayAddString(sa, (char *)"#ifdef __cplusplus", 1);
+    sarrayAddString(sa, (char *)"extern \"C\" {", 1);
+    sarrayAddString(sa, (char *)"#endif  /* __cplusplus */\n", 1);
     str = sarrayToString(sa, 1);
     fprintf(stdout, str);
     sarrayDestroy(&sa);
     FREE(str);
 
-    for (i = 1; i < argc; i++) {
+        /* Prepend 'prestring' if requested */
+    firstfile = 1;
+    prestring = NULL;
+    if (argv[1][0] == '-') {
+        firstfile = 2;
+        if (sscanf(argv[1], "-prestring=%s", buf) != 1)
+            L_WARNING("Failure to parse prestring; omitting!", mainName);
+        else {
+            if ((len = strlen(buf)) > L_BUF_SIZE - 3)
+                L_WARNING("prestring too large; omitting!", mainName);
+            else {
+                buf[len] = ' ';
+                buf[len + 1] = '\0';
+                prestring = stringNew(buf);
+            }
+        }
+    }
+        
+    for (i = firstfile; i < argc; i++) {
         filein = argv[i];
 	len = strlen(filein);
 	if (filein[len - 1] == 'h')
@@ -88,26 +113,27 @@ static char  mainName[] = "xtractprotos";
         filename = stringJoin(tempfile, ".exe");
 #endif  /* ~ _CYGWIN_ENVIRON */
 
-	if ((str = parseForProtos(filename)) == NULL) {
+	if ((str = parseForProtos(filename, prestring)) == NULL) {
             fprintf(stderr, "parse failure for %s; continuing\n", filein);
 	    continue;
 	}
-	if (strlen(str) > 1) {  /* strlen == 1 is a file without protos */
+	if (strlen(str) > 1)  /* strlen(str) == 1 is a file without protos */
             fprintf(stdout, str);
-            FREE(str);
-        }
+        FREE(str);
 	FREE(filename);
     }
 
         /* Output extern C tail */
     sa = sarrayCreate(0);
-    sarrayAddString(sa, "\n#ifdef __cplusplus", 1);
-    sarrayAddString(sa, "}", 1);
-    sarrayAddString(sa, "#endif  /* __cplusplus */", 1);
+    sarrayAddString(sa, (char *)"\n#ifdef __cplusplus", 1);
+    sarrayAddString(sa, (char *)"}", 1);
+    sarrayAddString(sa, (char *)"#endif  /* __cplusplus */", 1);
     str = sarrayToString(sa, 1);
     fprintf(stdout, str);
     sarrayDestroy(&sa);
     FREE(str);
+    if (prestring)
+        FREE(prestring);
 
     return 0;
 }
