@@ -13,25 +13,34 @@
  -  or altered from any source or modified source distribution.
  *====================================================================*/
 
-#include <stdio.h>
-#include "allheaders.h"
-
 /*
  *  zlibmemstub.c
  *
  *     Stubs for zlibmem.c functions
  */
 
-l_uint8 * zlibCompress( l_uint8  *datain, l_int32   nin, l_int32  *pnout )
+#include <stdio.h>
+#include "allheaders.h"
+
+#ifdef HAVE_CONFIG_H
+#include "config_auto.h"
+#endif  /* HAVE_CONFIG_H */
+
+/* --------------------------------------------*/
+#if  !HAVE_LIBZ   /* defined in environ.h */
+/* --------------------------------------------*/
+
+l_uint8 * zlibCompress(l_uint8 *datain, l_int32 nin, l_int32 *pnout)
 {
-    PROCNAME("zlibCompress");
-    return (l_uint8 *)ERROR_PTR("function not present", procName, NULL);
+    return (l_uint8 *)ERROR_PTR("function not present", "zlibCompress", NULL);
 }
 
-l_uint8 * zlibUncompress( l_uint8  *datain, l_int32   nin, l_int32  *pnout )
+l_uint8 * zlibUncompress(l_uint8 *datain, l_int32 nin, l_int32 *pnout)
 {
-    PROCNAME("zlibUncompress");
-    return (l_uint8 *)ERROR_PTR("function not present", procName, NULL);
+    return (l_uint8 *)ERROR_PTR("function not present", "zlibUncompress", NULL);
 }
 
+/* --------------------------------------------*/
+#endif  /* !HAVE_LIBZ */
+/* --------------------------------------------*/
 
